C***********************************************************************
C      THE FOLLOWING 2 SUBROUTINES
C          SUBROUTINE READSAO
C          SUBROUTINE WRITSAO      <<<----------- NOT INCLUDED HERE (LFM)
C      ARE USED IN NHPC 4.00 (OR HIGHER) & VIEWNHPC 4.00 (OR HIGHER)
C
C-----------------------------------------------------------------------
C
C      SUBROUTINE READSAO(EOF,IU,IDFI,GCONST,Sysdes,IPREF,SCALED,IAF,
C     +   DTT,IOTF,IOThF,IOAF,IODF,ftOF,IOTF1,IOThF1,IOAF1,IODF1,ftOF1,
C     +   IOTE,IOThE,IOAE,IODE,ftOE,IXTF,IXAF,IXDF,ftXF,IXTF1,IXAF1,
C     +   IXDF1,ftXF1,IXTE,IXAE,IXDE,ftXE,MEDF,MEDE,MEDES,THF2,THF1,
C     +   THE,QPCOEF,THVAL,IedF,IOTsE,IOAsE,IODsE,ftOsE,
C     +   OTF,OThF,OTF1,OThF1,OTE,OThE,XTF,XTF1,XTE,OTsE,
C     +   HTAB,FTAB,QL,DL,IedFTP,
C     +   IREAD,IERR)
C
C  Version 96050810


C  Input Variables:
C    Arrays to store SAO data
C    EOF      -   end of file check
C    IU       -   file unit number
C   array     code    description
C             group
C    IDFI     --      data file index
C    GCONST        1       Geophysical constants
C    Sysdes        2       System description
C    IPREF         3       ionogram sounding settings
C    SCALED        4       scaled ionogram parameters
C    IAF           5       analysis flags
C    DTT           6       Doppler translation table
C    IOTF/OTF      7       O-trace F2 virtual height
C    IOThF/OThF    8       O-trace F2 true height
C    IOAF          9       O-trace F2 amplitudes
C    IODF         10       O-trace F2 Doppler numbers
C    ftOF         11       O-trace F2 frequency table
C    IOTF1/OTF1   12       O-trace F1 virtual height
C    IOThF1/OThF1 13       O-trace F1 true height
C    IOAF1        14       O-trace F1 amplitudes
C    IODF1        15       O-trace F1 Doppler numbers
C    ftOF1        16       O-trace F1 frequency table
C    IOTE/OTE     17       O-trace E  virtual height
C    IOThE/OThE   18       O-trace E  true height
C    IOAE         19       O-trace E  amplitudes
C    IODE         20       O-trace E  Doppler numbers
C    ftOE         21       O-trace E  frequency table
C    IXTF/XTF     22       X-trace F2 virtual height
C    IXAF         23       X-trace F2 amplitudes
C    IXDF         24       X-trace F2 Doppler numbers
C    ftXF         25       X-trace F2 frequency table
C    IXTF1/XTF1   26       X-trace F1 virtual height
C    IXAF1        27       X-trace F1 amplitudes
C    IXDF1        28       X-trace F1 Doppler numbers
C    ftXF1        29       X-trace F1 frequency table
C    IXTE/XTE     30       X-trace E  virtual height
C    IXAE         31       X-trace E  amplitudes
C    IXDE         32       X-trace E  Doppler numbers
C    ftXE         33       X-trace E  frequency table
C    MEDF         34       Median amplitude of F echo
C    MEDE         35       Median amplitude of E echo
C    MEDES        36       Median amplitude of Es echo
C    THF2         37       F2 layer true height parameters
C    THF1         38       F1 layer true height parameters
C    THE          39       E layer true height parameters
C    THVAL        40       Valley parameters from Polan and NhPc version 2.01
C    IedF         41       Edit Flags
C    THVAL        42       Valley parameters from NhPc version 3.01 or higher
C    IOTsE/OTsE   43       O-trace sporadic-E  virtual height
C    IOAsE        44       O-trace sporadic-E  amplitudes
C    IODsE        45       O-trace sporadic-E  Doppler numbers
C    ftOsE        46       O-trace sporadic-E  frequency table
C
C  Output Variables
C    Arrays with corresponding SAO data
C
C  Purpose:  READ SAO files with all versions
C
C  Revision Log
C     IDFI(80) is used as an indicator of the format of the data
C              in the SAO file
C     IDFI(80) = 0 --- SAO version 3.0 format
C     IDFI(80) = 1 --- SAO version 3.0 format with true height data
C                      format enhanced
C     IDFI(80) = 2 --- SAO version 4.0 format
C     IDFI(80) = 3 --- SAO version 4.0 format, oblique ionogram
C                      converted to vertical
C
C---------------------------
C
C**********************************************************************
C**********************************************************************

c----- Sporadic E dimensions increased to 250 to cope with a Ramey ionogram
c      FF199917606252130

C**********************************************************************
C**********************************************************************
      SUBROUTINE READSAO(EOF,IU,IDFI,GCONST,SYSDES,IPREF,SCALED,IAF,
     +   DTT,IOTF,IOTHF,IOAF,IODF,FTOF,IOTF1,IOTHF1,IOAF1,IODF1,FTOF1,
     +   IOTE,IOTHE,IOAE,IODE,FTOE,IXTF,IXAF,IXDF,FTXF,IXTF1,IXAF1,
     +   IXDF1,FTXF1,IXTE,IXAE,IXDE,FTXE,MEDF,MEDE,MEDES,THF2,THF1,
     +   THE,QPCOEF,THVAL,IEDF,IOTSE,IOASE,IODSE,FTOSE,
     +   OTF,OTHF,OTF1,OTHF1,OTE,OTHE,XTF,XTF1,XTE,OTSE,
     +   HTAB,FTAB,QL,DL,IEDFTP,
     +   IREAD,IERR)
C
      LOGICAL   EOF
      INTEGER*4 IU      
      INTEGER*2 I,IREAD,IERR
C
      INTEGER*2 IDFI(80),IAF(60),IOTF(666),IOTHF(666),
     +IOAF(666),IODF(666),IOTF1(150),IOTHF1(150),IOAF1(150),
     +IODF1(150),IOTE(150),IOTHE(150),IOAE(150),IODE(150),
     +IXTF(400),IXAF(400),IXDF(400),IXTF1(150),IXAF1(150),
     +IXDF1(150),IXTE(150),IXAE(150),IXDE(150),MEDF(20),
     +MEDE(20),MEDES(20),IEDF(120),IOTSE(250),IOASE(250),IODSE(250),
     +IEDFTP(120)
C
      REAL*4 OTF(666),OTHF(1000),OTF1(150),OTHF1(1000),OTE(150),
     +       OTHE(1000),XTF(400),XTF1(150),XTE(150),OTSE(250),
     +       HTAB(999),FTAB(999),NTAB
C
      REAL*4 SCALED(120),GCONST(16),DTT(16),FTOF(666),FTOF1(150),
     +       FTOE(150),FTXF(400),FTXF1(150),FTXE(150),THF2(20),
     +       THF1(20),THE(20),THVAL(20),FTOSE(250)
      REAL*8 QPCOEF(121)
      CHARACTER*120 SYSDES,OPMSG
      CHARACTER*1 IPREF(120),QL(120),DL(120)
C
      CHARACTER*12 FM1,FM2,FM3,FM4,FM5,FM6,FM7,FM8,FM9,FM10,FM11,FM12

C
C-----------------------------------------------------------------------
C...formats
 100  FORMAT (40I3)
      IERR=1
C
    1 CONTINUE
C...data file index
C     The data file index integers take two lines.
      READ(IU,100,ERR=2,END=9) (IDFI(I),I=1,80)
      GOTO 3
    2 CONTINUE
      READ(IU,'(A120)',END=9) SYSDES
      GOTO 1
C
    3 CONTINUE
      IF(IDFI(1).GT.0.AND.IDFI(1).LT.17) GOTO 4
      GOTO 99
C
    4 CONTINUE
    
      IERR=0
      IF(IDFI(80).EQ.0) THEN  !  VERSION 3 (AND LOWER) SAO DATA
        WRITE(FM1,'(4X,A8)')  '(40I3)'
        WRITE(FM2,'(2X,A10)') '(16F7.3)'
        WRITE(FM3,'(4X,A8)')  '(A120)'
        WRITE(FM4,'(3X,A9)')  '(120A1)'
        WRITE(FM5,'(2X,A10)') '(15F8.3)'
        WRITE(FM6,'(4X,A8)')  '(60I2)'
        WRITE(FM7,'(3X,A9)')  '(120I1)'
        WRITE(FM8,'(2X,A10)') '(20F6.3)'
        WRITE(FM9,'(A12)')    '(13E9.4E1)'
        WRITE(FM10,'(3X,A8)') '(60I2)'
      ENDIF
C
      IF(IDFI(80).EQ.1) THEN  ! VERSION 3 WITH ENHANCED TRUE HEIGHT DATA
        WRITE(FM1,'(4X,A8)')  '(40I3)'
        WRITE(FM2,'(2X,A10)') '(16F7.3)'
        WRITE(FM3,'(4X,A8)')  '(A120)'
        WRITE(FM4,'(3X,A9)')  '(120A1)'
        WRITE(FM5,'(2X,A10)') '(15F8.3)'
        WRITE(FM6,'(4X,A8)')  '(60I2)'
        WRITE(FM7,'(3X,A9)')  '(120I1)'
        WRITE(FM8,'(2X,A10)') '(20F6.3)'
        WRITE(FM9,'(A12)')    '(10E11.6E1)' ! <--- TRUE HEIGHT DATA
        WRITE(FM10,'(3X,A8)') '(60I2)'
      ENDIF
C
      IF(IDFI(80).GE.2) THEN                !  VERSION 4.00 OR HIGHER
        WRITE(FM1,'(2X,A10)') '(15F8.3)'    ! <--- H' AND Z
        WRITE(FM2,'(2X,A10)') '(16F7.3)'
        WRITE(FM3,'(4X,A8)')  '(A120)'
        WRITE(FM4,'(3X,A9)')  '(120A1)'     ! <--- PREFACE
        WRITE(FM5,'(2X,A10)') '(15F8.3)'
        WRITE(FM6,'(4X,A8)')  '(60I2)'
        WRITE(FM7,'(3X,A9)')  '(120I1)'
        WRITE(FM8,'(2X,A10)') '(15F8.3)'    ! <--- FREQUENCY TABLE
        WRITE(FM9,'(A12)')    '(10E11.6E1)' ! <--- TRUE HEIGHT DATA
        WRITE(FM10,'(3X,A8)') '(40I3)'
        WRITE(FM11,'(A12)')   '(15E8.3E1)'
        WRITE(FM12,'(A11)')   '(6E20.12)'
      ENDIF
C
C...Geophysical constants -- Code 1
      IF(IDFI(1).GT.0) READ(IU,FM2,END=9,ERR=99)
     +                      (GCONST(I),I=1,IDFI(1))
C
C...system description -- Code 2
      IF(IDFI(2).GT.0) READ(IU,FM3,END=9,ERR=99) SYSDES
      IF(IDFI(2).EQ.2) READ(IU,FM3,END=9,ERR=99) OPMSG
C
C...ionogram sounding settings (preface) -- Code 3
      IF(IDFI(3).GT.0) READ(IU,FM4,END=9,ERR=99)
     +                      (IPREF(I),I=1,IDFI(3))
C
C...scaled ionogram parameters
      IF(IDFI(4).GT.0) READ(IU,FM5,END=9,ERR=99)
     +                      (SCALED(I),I=1,IDFI(4))
C
C...ARTIST analysis flags
      IF(IDFI(5) .GT.0) READ(IU,FM6,END=9,ERR=99)
     +                      (IAF(I),I=1,IDFI(5))
C
C...Doppler translation table
      IF(IDFI(6).GT.0) READ(IU,FM2,END=9,ERR=99)
     +                      (DTT(I),I=1,IDFI(6))
C
C...O-trace F2 points
C...virtual height
      IF(IDFI(7).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
          READ(IU,FM1,END=9,ERR=99) (OTF(I),I=1,IDFI(7))  ! SAO V4.0
          DO I=1,IDFI(7)
          IOTF(I)=OTF(I)
          ENDDO
        ELSE
          READ(IU,FM1,END=9,ERR=99) (IOTF(I),I=1,IDFI(7)) ! SAO V3.0
                                                          ! AND LOWER
          DO I=1,IDFI(7)
          OTF(I)=IOTF(I)
          ENDDO
        ENDIF
      ENDIF
C
C...true height
      IF(IDFI(8).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
          READ(IU,FM1,END=9,ERR=99) (OTHF(I),I=1,IDFI(8))  ! SAO V4.0
        ELSE
          READ(IU,FM1,END=9,ERR=99) (IOTHF(I),I=1,IDFI(8)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(9).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IOAF(I),I=1,IDFI(9))
C
C...Doppler numbers
      IF(IDFI(10).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IODF(I),I=1,IDFI(10))
C
C...frequency table
      IF(IDFI(11).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTOF(I), I=1,IDFI(11))
C
C...O-trace F1 points
C...virtual height
      IF(IDFI(12).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
         READ(IU,FM1,END=9,ERR=99) (OTF1(I),I=1,IDFI(12))  ! SAO V4.0
         DO I=1,IDFI(12)
            IOTF1(I)=OTF1(I)
         ENDDO
        ELSE
         READ(IU,FM1,END=9,ERR=99) (IOTF1(I),I=1,IDFI(12)) ! SAO V3.0
                                                           ! AND LOWER
         DO I=1,IDFI(12)
             OTF1(I)=IOTF1(I)
         ENDDO
        ENDIF
      ENDIF
C
C...true height
      IF(IDFI(13).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
        READ(IU,FM1,END=9,ERR=99) (OTHF1(I),I=1,IDFI(13))  ! SAO V4.0
        ELSE
        READ(IU,FM1,END=9,ERR=99) (IOTHF1(I),I=1,IDFI(13)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(14).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IOAF1(I),I=1,IDFI(14))
C
C...Doppler number
      IF(IDFI(15).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IODF1(I),I=1,IDFI(15))
C
C...frequency table
      IF(IDFI(16).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTOF1(I), I=1,IDFI(16))
C
C...O-trace E points
C...virtual heights
      IF(IDFI(17).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
          READ(IU,FM1,END=9,ERR=99) (OTE(I),I=1,IDFI(17))  ! SAO V4.0
          DO I=1,IDFI(17)
             IOTE(I)=OTE(I)
          ENDDO
        ELSE
          READ(IU,FM1,END=9,ERR=99) (IOTE(I),I=1,IDFI(17)) ! SAO V3.0
                                                           ! AND LOWER
          DO I=1,IDFI(17)
             OTE(I)=IOTE(I)
          ENDDO
        ENDIF
      ENDIF
C
C...true height
      IF(IDFI(18).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
         READ(IU,FM1,END=9,ERR=99) (OTHE(I),I=1,IDFI(18))  ! SAO V4.0
        ELSE
         READ(IU,FM1,END=9,ERR=99) (IOTHE(I),I=1,IDFI(18)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(19).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IOAE(I),I=1,IDFI(19))
C
C...Doppler numbers
      IF(IDFI(20).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IODE(I),I=1,IDFI(20))
C
C...frequency table
      IF(IDFI(21).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTOE(I), I=1,IDFI(21))
C
C...X-trace F2 points
C...virtual heights
      IF(IDFI(22).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
          READ(IU,FM1,END=9,ERR=99) (XTF(I),I=1,IDFI(22))  ! SAO V4.0
        ELSE
          READ(IU,FM1,END=9,ERR=99) (IXTF(I),I=1,IDFI(22)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(23).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IXAF(I),I=1,IDFI(23))
C
C...Doppler numbers
      IF(IDFI(24).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IXDF(I),I=1,IDFI(24))
C
C...frequency table
      IF(IDFI(25).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTXF(I),I=1,IDFI(25))
C
C...X-trace F1 points
C...virtual heights
      IF(IDFI(26).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
         READ(IU,FM1,END=9,ERR=99) (XTF1(I),I=1,IDFI(26))  ! SAO V4.0
        ELSE
         READ(IU,FM1,END=9,ERR=99) (IXTF1(I),I=1,IDFI(26)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(27).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IXAF1(I),I=1,IDFI(27))
C
C...Doppler numbers
      IF(IDFI(28).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IXDF1(I),I=1,IDFI(28))
C
C...frequency table
      IF(IDFI(29).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTXF1(I),I=1,IDFI(29))
C
C...X-trace E points
C...virtual heights
      IF(IDFI(30).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
          READ(IU,FM1,END=9,ERR=99) (XTE(I),I=1,IDFI(30))  ! SAO V4.0
        ELSE
          READ(IU,FM1,END=9,ERR=99) (IXTE(I),I=1,IDFI(30)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(31).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IXAE(I),I=1,IDFI(31))
C
C...Doppler numbers
      IF(IDFI(32).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IXDE(I),I=1,IDFI(32))
C
C...frequency table
      IF(IDFI(33).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTXE(I),I=1,IDFI(33))
C
C...median amplitude of F echo
      IF(IDFI(34).GT.0) READ(IU,FM6,END=9,ERR=99)
     +                      (MEDF(I),I=1,IDFI(34))
C...median amplitude of E echo
      IF(IDFI(35).GT.0) READ(IU,FM6,END=9,ERR=99)
     +                      (MEDE(I),I=1,IDFI(35))
C...median amplitude of Es echo
      IF(IDFI(36).GT.0) READ(IU,FM6,END=9,ERR=99)
     +                      (MEDES(I),I=1,IDFI(36))
C
C...F2 layer true height parameters
      IF(IDFI(37).GT.0) READ(IU,FM9,END=9,ERR=99)
     +                      (THF2(I),I=1,IDFI(37))
C
C...F1 layer true height parameters
      IF(IDFI(38).GT.0) READ(IU,FM9,END=9,ERR=99)
     +                      (THF1(I),I=1,IDFI(38))
C...E layer true height parameters
      IF(IDFI(39).GT.0) READ(IU,FM9,END=9,ERR=99)
     +                      (THE(I),I=1,IDFI(39))
C
C...valley parameters from Polan and NhPc version 2.01
      IF(IDFI(40).GT.0) THEN
        IF(IDFI(80).LT.2) THEN
        READ(IU,FM9,END=9,ERR=99)  (THVAL(I),I=1,IDFI(40))
        ELSE
        READ(IU,FM12,END=9,ERR=99) (QPCOEF(I),I=1,IDFI(40))
        ENDIF
      ENDIF
C
C...edit flags
C...NOTE: FOR OLD DATA, THIS INCLUDES BOTH THE CHARACTERISTCS FLAG
C         AND THE TRACE+PROFILE FLAG
      IF(IDFI(41).GT.0) READ(IU,FM7,END=9,ERR=99) (IEDF(I),I=1,IDFI(41))
C
C...Valley parameters from NhPc version 3.01 and greater Sept. 1993
      IF(IDFI(42).GT.0) READ(IU,FM9,END=9,ERR=99)
     +                      (THVAL(I),I=1,IDFI(42))
C
C...O-trace sporadic-E
C...virtual heights
      IF(IDFI(43).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
         READ(IU,FM1,END=9,ERR=99) (OTSE(I),I=1,IDFI(43))  ! SAO V4.0
        ELSE
         READ(IU,FM1,END=9,ERR=99) (IOTSE(I),I=1,IDFI(43)) ! SAO V3.0
                                                           ! AND LOWER
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(44).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IOASE(I),I=1,IDFI(44))
C
C...Doppler numbers
      IF(IDFI(45).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IODSE(I),I=1,IDFI(45))
C
C...frequency table
      IF(IDFI(46).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTOSE(I),I=1,IDFI(46))
C
C...O-trace - Auroral E layer
C...virtual heights
      IF(IDFI(47).GT.0) THEN
        IF(IDFI(80).GE.2)THEN
         READ(IU,FM1,END=9,ERR=99) (OTSE(I),I=1,IDFI(47))  ! SAO V4.0
         DO I=1,IDFI(47)
         IOTSE(I)=OTSE(I)
         ENDDO
        ELSE
         READ(IU,FM1,END=9,ERR=99) (IOTSE(I),I=1,IDFI(47)) ! SAO V3.0
                                                           ! AND LOWER
         DO I=1,IDFI(47)
         OTSE(I)=IOTSE(I)
         ENDDO
        ENDIF
      ENDIF
C
C...amplitudes
      IF(IDFI(48).GT.0) READ(IU,FM10,END=9,ERR=99)
     +                      (IOASE(I),I=1,IDFI(48))
C
C...Doppler numbers
      IF(IDFI(49).GT.0) READ(IU,FM7,END=9,ERR=99)
     +                      (IODSE(I),I=1,IDFI(49))
C
C...frequency table
      IF(IDFI(50).GT.0) READ(IU,FM8,END=9,ERR=99)
     +                      (FTOSE(I),I=1,IDFI(50))
C
C...N(h) Tabulation
      IF(IDFI(51).GT.0) THEN
        READ(IU,FM11,END=9,ERR=99) (HTAB(I),I=1,IDFI(51))
        READ(IU,FM11,END=9,ERR=99) (FTAB(I),I=1,IDFI(52))
        READ(IU,FM11,END=9,ERR=99) (NTAB,I=1,IDFI(53)) !! ELSE test on IDFI(53)
      ENDIF

c      print*,IDFI(52),(FTAB(I),I=1,IDFI(52))
c      print*,IDFI(51),(HTAB(I),I=1,IDFI(51))
c      print*,IDFI(53),IDFI(54),IDFI(55),IDFI(56)

C
C...Qualifying Letters
      IF(IDFI(54).GT.0) READ(IU,FM4,END=9,ERR=99) (QL(I),I=1,IDFI(54))
c      IF(IDFI(54).GT.0) print*, (QL(I),I=1,IDFI(54))

C...Descriptive Letters
      IF(IDFI(55).GT.0) READ(IU,FM4,END=9,ERR=99) (DL(I),I=1,IDFI(55))
c      IF(IDFI(55).GT.0) print*, (DL(I),I=1,IDFI(55))

C...Edit Flags - Traces and Profile
      IF(IDFI(56).GT.0) THEN
        READ(IU,FM7,END=9,ERR=99) (IEDFTP(I),I=1,IDFI(56))
c      IF(IDFI(56).GT.0) print*,  (IEDFTP(I),I=1,IDFI(56))
      ENDIF

C
      IREAD=1
      EOF = .FALSE.
      RETURN
            
    9 EOF = .TRUE.
      RETURN
C
   99 CONTINUE
      IERR=1
C
      RETURN
      END
c......................................................................
c
      function z_cheb (a, g, hmax)
c       function to calculate the real height Z corresponding to the
c       parameter G.
c       ref [2] Huang & Reinisch Radio Science 17(4) July/August 1982
c           [3] Reinisch & Huang ........      18(3) May/June 1983
c
c       written by leo mcnamara 5-apr-90
c-----    last modified 17-apr-90
c
      dimension a(*)
c
c       the counter I runs from 0 to iii - 1   ...  we use 1 to iii
cc?     ii = iii 
c
c       eqn 2.6 & 3.21
      s = 0.
      do 20 i = 1,5
20      s = s + a(i) * shifted_cheb(g,i-1)
c
      z_cheb = 0.
      if(g.ge.0.) z_cheb = hmax +sqrt(g) * s
c
      return
      end
c......................................................................
      function shifted_cheb (g,ii)
c       function to calculate the shifted Chebychev poly, using the recursion
c       relation given by Huang & Reinisch Radio Science 17(4) July/August 1982
c
c       written by leo mcnamara 5-apr-90
c-----    last modified 6-apr-90
c
      dimension t(50)
c
      t0 = 1.
      shifted_cheb = t0
      if(ii.eq.0) return
      t(1) = 2. * g - 1.
      if(ii.eq.1)     shifted_cheb = t(1)
      t(2) = 2. * (2. * g - 1.) * t(1) - t0
      if(ii.eq.2)     shifted_cheb = t(2)
c
      if(ii.lt.3) return
c
      do 10 i = 3,ii
      t(i) = 2. * (2. * g - 1.) * t(i-1) - t(i-2)
ccc     print*,' i,t(i) ',i,t(i)
10      shifted_cheb = t(i)
c
      return
      end