c----- Notes:
c      cnps, apch and apfs come from an IBM math pac from the early 70s.
c      A GOOGLE search indicates that versions of CNPS, APCH and APFS 
c      are part of PCSSP. PC Scientific Subroutine Package,

      subroutine cheby_lsf (x,y,w,np,nc,eps,eta,ier,xd,xo,ires,coeff)
c
c      s/r to fit a LSF curve based on Chebyshev polynomials
c
c++++++++++++++++++  Change Log  +++++++++++++++++++++++++++++++++++
c 19-dec-93 Dimension of DATI changed from 300 to 600 (takes 200 points)
c++++++++++++++++++  Change Log  +++++++++++++++++++++++++++++++++++
c      written by leo mcnamara 12-aug-93
c      IER returned 19-aug-93
c      Modified to handle only 1 or 2 points 31-aug-93
c-----        last modified 19-dec-93
c
c      X	the abscissae at which the function is given
c      Y	the values of the function
c      W	the weights associated with each data point
c      NP	number of data points
c      NC	number of polynomials to be used as basis functions
c      EPS	relative tolerance loss of significance
c      ETA	relative tolerance square sum of errors
c      IER	error flag (should be 0)
c      XD,XO	transformation t = xd * x + xo
c      IRES	dimension of calculated LSF which satisfies tolerances
c      COEFF	coefficients of the expansion in terms of Cheb polys
c
c      s/r calls in the IBM Applications Programs functions APCH & APFS
c
      dimension x(*),y(*),w(*),dati(1666),work(100),coeff(*)
c
c      Check for only one point
c      Y(x) = c1 T0(x), and T0(x) = 1
      ires = 1
      coeff(1) = y(1)
c      XD,XO	transformation t = xd * x + xo
      xo = 0.
      xd = 1.
      if(np.eq.1) return
c
c      Check for only 2 points
c      Y(x) = c1 T0(x) + c2 T1(x)
      ires = 2
      slope = (y(2) - y(1)) / (x(2) - x(1))
      coeff(1) = y(1) - slope * x(1)
      coeff(2) = slope
      xo = 0.
      xd = 1.
      if(np.eq.2) return
c
c      set up the array DATI
      do 10 i = 1,np
      dati(i) = x(i)
      dati(i + np) = y(i)
      dati(i + np + np) = w(i)
10      continue
c
      call APCH (dati,np,nc,xd,xo,work,ier)
      if(ier.ne.0) print*,' Error return from APCH ', ier 
      if(ier.ne.0) return
c
      iop = -1	!   choose smaller of N1 & N2
      call APFS (work,nc,ires,iop,eps,eta,ier)
ccc      if(ier.ne.0) print*,' Error return from APFS ', ier 
      if(ier.ne.0) return
      if(ires.eq.0) print*,' IRES = 0 from APCH '
      if(ires.eq.0) stop
c
c      pull out the coefficients from the triangular array
      ii = 0
      ires1 = ires - 1
      do 20 i = 1,ires1
      ii = ii + i
20      continue
      ii = ii + 1
      jj = ii + ires - 1
      ll = 0
      do 30 i = ii,jj
      ll = ll + 1
cc      print*,' I ',i
      coeff(ll) = work(i)
30      continue
c
ccc      print*,(work(i),i=1,20)
      return
      end
c.......................................
      function cnps (c,n,x)
c
c      see IBM Applications Programs page 199
c      Computes the value of an N-term expansion in Chebyshev
c      polynomials with coeff vector C for argument X
c
      dimension c(*)
c
      if(n-2) 3,4,4
3      cnps = c(1)
      return
c
c      initialization
4      arg = x + x
      h1 = 0.
      h0 = 0.
c
      do 5 i = 1,n
      k = n - i
      h2 = h1
      h1 = h0
5      h0 = arg * h1 - h2 + c(k + 1)
      cnps = 0.5 * (c(1) - h2 + h0)
      return
      end
c..........................................................................
      subroutine apch (dati,n,ip,xd,xo,work,ier)
c
c      IBM Applications Programs Pg 264 etc
c
c      typed by leo mcnamara 12-aug-93
c-----        last modified 12-aug-93
c
c      APCH sets up the normal equations of a least squares fit in terms of
c      Chebyshev polynomials for a given discrete function
c
c      DATI	vector of dimension 3*N (or dimension 2*n+1) containing
c      	the given arguments, followed by the function values and
c      	N (repectively 1) weight values.
c      N	number of given points
c      IP	dimension of LSF i.e. number of Chebyshev polynomials
c      	used as fundamental functions. IP should not exceed N.
c      XD	resultant multiplicative constant for linear transformation
c      	of argument range	
c      XO	resultant additive constant for linear transformation
c      	of argument range	
c      WORK	working storage of dimension (ip+1)*(ip+2)/2
c      	On return, WORK contains the symmetric coefficient
c      	matrix of the normal equations in compressed form,
c      	followed immediately by RHS and square sum of function values
c      IER	-1  formal errors in dimension
c      	 0  no errors
c      	+1  coinciding arguments
c
c      REMARKS	    no weights are used if dati(2*n+1).le.0
c      	    Execution of this s/r is to be followed by execution of
c      	    s/r APFS
c
      dimension dati(*),work(*)
c
c      check for formal errors in specified dimensions
      if(n-1) 19,20,1
1      if(ip) 19,19,2
c
c      search smallest and largest argument
2      if(ip - n) 3,3,19
3      xa = dati(1)
      xo = xa
      xe = 0.
      do 7 i = 1,n
      xm = dati(i)
      if(xa - xm) 5,5,4
4      xa = xm
5      if(xo - xm) 6,7,7
6      xo = xm
7      continue
c
c      initialize calculation of normal equations
      xd = xo - xa
      m = (ip * (ip + 1)) / 2
      iend = m + ip + 1
      mt2 = ip + ip
      mt2m = mt2 - 1
c
c      set working storage and RHS to zero
      do 8 i = 1,ip
      j = mt2 - i
      work(j) = 0.
      work(i) = 0.
      k = m + i
8      work(k) = 0.
c
c      check for degenerate argument range
      if(xd) 20,20,9
c
c      calculate constants for reduction of arguments
9      xo = -(xo + xa) / xd
      xd = 2. / xd
      sum = 0.
c
c      start great loop over all given points
      do 15 i = 1,n
      t = dati(i) * xd + xo
      j = i + n
      df = dati(j)
c
c      calculate and store values of Chebyshev polynomials
c      for argument T
      xa = 1.
      xm = t
      if(dati(2*n + 1)) 11,11,10
10      j = j + n
      xa = dati(j)
      xm = t * xa
11      t = t + t
      sum = sum + df * df * xa
      df = df + df
      j = 1
12      k = m + j
      work(k) = work(k) + df * xa
13      work(j) = work(j) + xa
      if(j - mt2m) 14,15,15
14      j = j + 1
      xe = t * xm - xa
      xa = xm
      xm = xe
      if (j - ip) 12,12,13
15      continue
      work(iend) = sum + sum
c
c      calculate matrix of normal equations
      ll = m
      kk = mt2m
      jj = 1
      k = kk
      do 18 j = 1,m
      work(ll) = work(k) + work(jj)
      ll = ll - 1
      if(k - jj) 16,16,17
16      kk = kk - 2
      k = kk
      jj = 1
      go to 18
17      jj = jj + 1
      k = k - 1
18      continue
      ier = 0
      return
c
c      error return in case of formal errors
19      ier = -1
      return
c
c      error return in case of coinciding arguments
20      ier = 1
      return
      end
c.....................................................................
      subroutine apfs (work,ip,ires,iop,eps,eta,ier)
c
c      IBM Applications Programs page 261 etc
c
c      s/r performs symmetric factorization of the normal equations,
c      followed by calculation of the LSF optionally
c
c      WORK	Given symmetric coefficient matrix, stored compressed.
c      	i.e. upper triangular part columnwise. The given RHS
c      	occupies the next IP locations in WORK. The very last
c      	component of WORK contains the square sum of function
c      	values E0. This scheme of storage is produced by s/r APCH.
c      	The given matrix is factored in the form transpose(t)*T 
c      	and the given RHS is divided by transpose(T). The upper
c      	triangular factor T is returned in WORK if IOP = 0.
c      	In case of nonzero IOP, the calculated solutions are
c      	stored in the columns of triangular array WORK of
c      	corresponding dimension, and E0 is replaced by the square
c      	sum of the errors for fit of dimension IRES. The total
c      	dimension of WORK is (IP + 1)*(IP + 2) / 2.
c      IP	number of fundamental functions used for LSF
c      IRES	dimension of calculated LSF
c      	Let N1 & N2 denote the following numbers
c      	N1 = maximal dimension for which no loss of significance
c      	    was indicated during factorization
c      	N2 = smallest dimension for which the square sum of the
c      	    the errors does not exceed TEST = abs(eta * fsq)
c      	Then IRES = min0(ip,n1) if IOP nonnegative
c      	and  IRES = min0(ip,n1,n2) if IOP is negative
c      IOP	Input for selection of operation
c      	IOP = 0	means triangular factorization, division of the RHS
c      		by transpose(T), and calculation of the square sum
c      		of errors is performed only.
c      	IOP = +1,-1 The solution of dimension IRES is also calculated
c      	IOP = +2,-2 All solutions for dimension 1 up to IRES are
c      		    calculated.
c      EPS	Relative tolerance for test on loss of significance
c      	Choose between 1.e-3 and 1.e-6
c      ETA	Relative tolerance for tolerated square sum of errors
c      	Choose between 1.e0 and 1.e-6.
c      IER	-1  nonpositive IP
c      	 0  no loss of significance detected and specified tolerance of
c      	    errors reached
c      	+1  loss of significance detected or specified tolerance
c      	    not reached.
c
c      The absolute tolerance used internally for test on loss of significance
c      is tol = abs(eps*work(1)).
c      The absolute tolerance used internally for the square sum of errors
c      is abs(eta*fsq)
c      IOP > 2 has the same effect as IOP = 2
c      IOP < -2 has the same effect as IOP = -2
c      IRES = 0 means that the absolute value of EPS is not less than one 
c      and/or work(1) is not positive and.or IP is not positive.
c
c      Calculation of the LSF is done using Choleskys square root method for
c      symmetric factorization. The incorporated test on loss of significance
c      means each radicand must be greater than the internal absolute tolerance
c      TOL = abs(eps*work(1)). In case of loss of significance in the above
c      sense, only a subsystem of the normal equations is solved. In case of
c      negative IOP, the triangular factorization is terminated prematurely 
c      either if the square sum of the errors does not exceed eta*fsq or if
c      there is indication for loss of significance.
c
c      dimension dummy variables
      dimension work(*)
      ires = 0
c
c      test of specified dimension
      if(ip) 1,1,2
c
c      error return in case of illegal dimension
1      ier = -1
      return
c
c      initialize factorization process
2      ipiv = 0
      ipp1 = ip + 1
      ier = 1
      ite = ip * ipp1 / 2
      iend = ite + ipp1
      tol = abs(eps*work(1))
      test = abs(eta*work(iend))
c
c      start loop over all rows of WORK
      do 11 i = 1,ip
      ipiv = ipiv + i
      ja = ipiv - ires
      je = ipiv - 1
c
c      form scalr product needed to modify current row elements
      jk = ipiv
      do 9 k = i,ipp1
      sum = 0.
      if(ires) 5,5,3
3      jk = jk - ires
      do 4 j = ja,je
      sum = sum + work(j) * work(jk)
4      jk = jk + 1
5      if(jk - ipiv) 6,6,8
c
c      test for loss of significance
6      sum = work(ipiv) - sum
      if(sum - tol) 12,12,7
7      sum = sqrt(sum)
      work(ipiv) = sum
      piv = 1. / sum
      goto 9
c
c      update off-diagonal elements
8      sum = (work(jk) - sum) * piv
      work(jk) = sum
9      jk =jk + k
c
c      update square sum of errors
      work(iend) = work(iend) - sum * sum
c
c      record address of last pivot element
      ires = ires + 1
      iadr = ipiv
c
c      test for tolerable error if specified
      if(iop) 10,11,11
10      if(work(iend) - test) 13,13,11
11      continue
      if(iop) 12,22,12
c
c      perform back substitution if specified
12      if(iop) 14,23,14
13      ier = 0
14      ipiv = ires
15      if(ipiv) 23,23,16
16      sum = 0.
      ja = ite + ipiv
      jj = iadr
      jk = iadr
      k = ipiv
      do 19 i = 1,ipiv
      work(jk) = (work(ja) - sum) / work(jj)
      if(k - 1) 20,20,17
17      je = jj - 1
      sum = 0.
      do 18 j = k,ipiv
      sum = sum + work(jk) * work(je)
      jk = jk + 1
18      je = je + j
      jk = je - ipiv
      ja = ja - 1
      jj = jj - k
19      k = k - 1
20      if(iop/2) 21,23,21
21      iadr = iadr - ipiv
      ipiv = ipiv - 1
      go to 15
c
c      normal return
22      ier = 0
23      return
      end
c=======================================================================
