c+++++ qualscan V2007.2  -- February 2007 Leo F McNamara

      subroutine write_record_for_quality_figure
     a (khit,kasenum,kflag,kall_POLAN,iPOLANok,NGDCfile,numprob,
     a fof2_pol1,fof2_art,hmf2_pol1,hmf2_art,em3_smoothed,
     a em3_art,em3_rms_def,iaafcon,
     a statid,cyear4,cdayno,chour,cmin)

c----- Writes out the record that is used to assign a quality figure
c----- leo mcnamara 12-april-2005; 23-january-2006; 11-september-2006

c----- Note that qualscan writes some special records directly (f, s or u)

      character*1 khit
      character*3 cdayno
      character*2 chour,cmin
      character*4 cyear4
      character*5 statid
      
      dimension khit(18,9000)

c-----These are the Problems used to define the NGDC Quality Figure
c      for each ionogram
c      PROBLEM 1 ---The Amplitude Quality Figure is too small
c      PROBLEM 2 ---M(3000)F2 comes from the end of the F2 trace
c       PROBLEM 3 ---Real ht is too close to the virtual ht at start of F2
c       PROBLEM 4 ---Too large a difference between POLAN & ARTIST hmF2
c      PROBLEM 5 ---Too long a horizontal section in F2 trace (ES?)
c      PROBLEM 6 ---Could not fit a Cheby LSF polynomial through F2 points
c      PROBLEM 7 ---Too many sudden jumps in F2 trace
c       PROBLEM 8 ---Large errors in POLAN peak fit
c      PROBLEM 9 ---ARTIST value of hmF2 too low
c       PROBLEM 10---Could not fit good or any LSF curve through F1 trace	  
c      PROBLEM 11---Average slope decreasing in last 0.5 MHz
c      PROBLEM 12---Too long a gap in trace above foE or foF1
c      PROBLEM 13---There is a corner in the F2 trace
c      PROBLEM 14---foF2 is very near a prohibited frequency
c      PROBLEM 15---Slope of F2 trace is too low at end
c      PROBLEM 16---fmin is too high to get reliable profile
c      PROBLEM 17---too few points with non-zero amplitudes
c      PROBLEM 18---foF2 too close to model value of foF1

      NGDC_fatal_problems = 0
      if(kasenum.gt.0.and.kflag.eq.1) NGDC_fatal_problems = 1	!! First index is the problem number

      if(NGDC_fatal_problems.eq.1) 
     a write(NGDCfile,1770) statid,cyear4,cdayno,chour,cmin,
     a (khit(k,kasenum),k=1,numprob),      !! problem flags
     a kall_POLAN,iPOLANok,iaafcon,
     a fof2_pol1,fof2_art,hmf2_pol1,hmf2_art,em3_smoothed,
     a em3_art,em3_rms_def
1770      format(1x,a5,1x,a4,1x,a3,1x,a2,a2,1x,18(1x,a1),3i2,10f10.2)

      if(NGDC_fatal_problems.eq.0.)
     a write(NGDCfile,2770) statid,cyear4,cdayno,chour,cmin,
     a kall_POLAN,iPOLANok,iaafcon,
     a fof2_pol1,fof2_art,hmf2_pol1,hmf2_art,em3_smoothed,
     a em3_art,em3_rms_def
2770      format(1x,a5,1x,a4,1x,a3,1x,a2,a2,1x,36x,3i2,10f10.2)	!! 36 = 2 x numprob

      return
      END
c======================================================================

      subroutine NGDC_quality_figures
     a (NGDCfile,NGDCqual,kall_POLAN,iPOLANok,
     a statid,cyear4,cdayno,chour,cmin,ctemp,iononumber,numprob,
     a statlon,
     a kprob1,kprob2,kprob3,kprob4,kprob5,kprob6,kprob7,
     a kprob8,kprob9,kprob10,kprob11,kprob12,kprob13,
     a kprob14,kprob15,kprob16,kprob17,kprob18)
           
c----- s/r to assign a quality figure to each ionogram after all have
c      been processed

c----- leo mcnamara 12-april-2005, 24-august-2005; 11-september-2006

c      If more than one ionogram has been processed, the statistics
c      are calculated

      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      character*1 ctemp
      character*5 statid
      character*50 dummy
      
      dimension ctemp(*)
      dimension iqut(96,4),iqut_total(96),iqtem(4),jsum(4)

      rewind(NGDCfile)

c----- Read in data for each ionogram, & write out quality figure

c 2003 121 0215  1   x             x   x   E   x H
2300      continue
      read(NGDCfile,2305,end=2315) statid,cyear4,cdayno,chour,cmin,
     a (ctemp(j),j=1,numprob),kall_POLAN,iPOLANok,iaafcon,
     a fof2_pol1,fof2_art,hmf2_pol1,hmf2_art,em3_smoothed,
     a em3_art,em3_rms_def
2305      format(1x,a5,1x,a4,1x,a3,1x,2a2,1x,18(1x,a1),3i2,10f10.2)	!! for numprob = 18
      if (cyear4.eq."    ") go to 2315	!! end of quality figures
c      print*," CCTEMP",(ctemp(j),j=1,20)

c----- Error bars for three scaled parms fof2_2, hmf2_q and em3_q
      barf = 99.
      fof2_q = fof2_art
      if(fof2_pol1.ne.0.) THEN
      barf = abs(fof2_pol1 - fof2_art) / 2.
      fof2_q = (fof2_pol1 + fof2_art) / 2.
      ENDIF

      barh = 99.
      hmf2_q = hmf2_art
      if(hmf2_pol1.ne.0.and.hmf2_art.ne.0.) THEN
      barh = abs(hmf2_pol1 - hmf2_art) / 2.
      hmf2_q = (hmf2_pol1 + hmf2_art) / 2.
      ENDIF

      barm = em3_rms_def
      em3 = em3_art	!! this is always supplied
      if(ctemp(2).eq."2") THEN	!! M came from end of trace, or other reasons	
      em3= - em3_art
      ENDIF

      if(em3.gt.0.and.em3_smoothed.gt.0.) THEN
      barm = abs(em3_art - em3_smoothed) / 2.
      em3_q = (em3_art + em3_smoothed) / 2.
      ENDIF

      if(em3.gt.0.and.em3_smoothed.le.0.) THEN
      barm = em3_art * em3_rms_def / 100.
      em3_q = em3_art
      ENDIF

      if(em3.le.0.and.em3_smoothed.le.0.) THEN
      barm = 2. * em3_art * em3_rms_def / 100.
      em3_q = em3_art
      ENDIF

      if(em3.le.0.and.em3_smoothed.gt.0.) THEN
      barm = 2. * em3_art * em3_rms_def / 100.
      em3_q = em3_smoothed	!! use the smoothed value
      ENDIF

c----- Check for peak too low (from c-----ICHECK = 28)
      if(ctemp(9).eq."9") THEN
      barf = 99
      barh = 99
      ENDIF
      
c      print*,fof2_pol1,fof2_art,fof2_q,barf
c      print*,hmf2_pol1,hmf2_art,hmf2_q,barh
c      print*,em3_smoothed,em3_art,em3_q,barm

c----- Assign a quality figure for this ionogram

c      print*,cdayno,chour,cmin
      call assign_quality_figure(ctemp,numprob,iaafcon,iqf)

      if(iqf.eq.-999) THEN	!! was not set
      print*,(ctemp(i),i=1,numprob)
      print*," Quality figure not set for ",cdayno,chour,cmin
      pause
      iqf = 0
      ENDIF

      write(NGDCqual,2310) statid,cyear4,cdayno,chour,cmin,iqf,
     a (ctemp(j),j=1,numprob),iaafcon,
     a fof2_q,barf,hmf2_q,barh,em3_q,barm
c      print*,cyear4,cdayno,chour,cmin,iqf
2310      format(a5,1x,a4,1x,a3,1x,2a2,i2,
     a 2x,18a1,i3,      !! numprob
     a 4x,f4.1," +/- ",f4.1,
     a 3x,f5.1," +/- ",f5.1,4x,f4.2," +/- ",f4.2)

c----- Count the cases vs UT
      WRITE(DUMMY,'(a2)') chour
      READ(DUMMY,'(I2)')  ihour
      kuth = ihour + 1
      iqut(kuth,iqf+1) = iqut(kuth,iqf+1) + 1
      iqut_total(kuth) = iqut_total(kuth) + 1
      go to 2300
2315      continue

c----- Write out quality figure summary if not processing a single ionogram

      if(iononumber.gt.10) THEN

      DO j = 1,4
      jsum(j) = 0
      ENDDO
      write(NGDCqual,2317)
2317      format(//"   % Occurrence of Quality Figures",
     a /3x,"UT",2x,"~LT",3x,"##",4x,"0",4x,"1",4x,"2",4x,"3")
      DO ih = 1,24
      flocalt = ih - 1 + statlon/15.
      ilt = flocalt + 0.5
      if(ilt.ge.24) ilt = ilt - 24
      fff = float(iqut_total(ih))
      DO j = 1,4
      iqtem(j) = -1.
      jsum(j) = jsum(j) + iqut(ih,j)
      if(fff.ne.0.) iqtem(j)=float(iqut(ih,j)) / fff * 100. + 0.5
      ENDDO
      write(NGDCqual,2320) ih-1,ilt,iqut_total(ih),(iqtem(j),j=1,4)
2320      format(7i5)
      ENDDO	!! ih

c----- Normalized counts for 4 bins
      sumj = 0.
      DO j = 1,4
      sumj = sumj + jsum(j)
      ENDDO 
      DO j = 1,4
      jsum(j) = float(jsum(j)) / sumj * 100
      ENDDO
      write(NGDCqual,2321) (jsum(i),i=1,4)
2321      format(/" TOTALS",8x,4i5)

      write(NGDCqual,2325)
2325      format(/" Key:",/" 0	Could not process ionogram",
     a /" 1      High risk / low quality",
     a /" 2      Medium risk",
     a /" 3      Good quality (no problems)")

      ENDIF	!! iononumber.gt.10

c----- write out the counts and key that are used

      write(NGDCfile,2779) 
     a kprob1,kprob2,kprob3,kprob4,kprob5,kprob6,kprob7,
     a kprob8,kprob9,kprob10,kprob11,kprob12,kprob13,
     a kprob14,kprob15,kprob16,kprob17,kprob18

2779  format(//' These are the Problem counts:'//,
     a /' Problem 1 ',i4,' The Amplitude Quality Figure is too small',
     a /' Problem 2 ',i4,' M(3000)F2 comes from end of the F2 trace',
     a /' Problem 3 ',i4,' Real ht too close to hdash at start of F2',
     a /' Problem 4 ',i4,' Too large a diff of POLAN and ARTIST hmF2',
     a /' Problem 5 ',i4,' Too long a horizontal section in F2 trace',
     a /' Problem 6 ',i4,' Could not smooth F2 trace',
     a /' Problem 7 ',i4,' Too many sudden jumps in F2 trace',
     a /' Problem 8 ',i4,' Large errors in POLAN peak fit',
     a /' Problem 9 ',i4,' ARTIST value of hmF2 too low',
     a /' Problem A ',i4,' Could not fit LSF curve through F1 trace',
     a /' Problem B ',i4,' Aver slope decreases in last half of trace',
     a /' Problem C ',i4,' Too long a gap in trace above foE',
     a /' Problem D ',i4,' There is a corner in the F2 trace',
     a /' Problem E ',i4,' foF2 is very near a prohibited frequency',
     a /' Problem F ',i4,' Slope of F2 trace is too low at end',
     a /' Problem G ',i4,' fmin too high to get reliable profile',
     a /' Problem H ',i4,' too few points with non-zero amplitudes',
     a /' Problem I ',i4,' foF2 too close to model value of foF1',
     a //' Note that some ionograms had multiple problems')
     
c-----END of assigning quality figure

      return
      END
c======================================================================

      subroutine assign_quality_figure (chits,nc,iaafcon,iqual_fig)
      
c----- s/r to assign a quality figure to a set of ARTIST ionogram data

c      leo mcnamara 15-april-2005

c---- The question: 
c      Does it appear that the autoscaled trace is an accurate and complete 
c      representation of the leading edge of the actual ionogram?

c----- quality of 1 is checked last. (after 3 then 2)

c      chits		character array of letters/numbers corresponding to 
c      			qualcon problems
c      nc			number of chars used (18 problems)
c      iaafcon		ARTIST analysis flag 10, confidence level
c      IQUAL_FIG	integer 0,1,2 or 3

c      0	Could not process ionogram
c      1	High risk / low quality
c      2	Medium risk
c      3	Good quality (no problems encountered)

c      f f f f f f f f f f f f f f f f f		fof2 value no good, trace probably ES
c      q q q q q q q q q q q q q q q q q q		ARTIST confidence level > 40 (44 or 55)
c      s s s s s s s s s s s s s s s s s		trace too short
c      u u u u u u u u u u u u u u u u u		virtual heights not defined
c     t t t t t t t t t t t t t t t t t      	truncated trace too short

c      1 2 3 4 5 6 7 8 9 A B C D E F G H		valid qualcon problems

      character*1 chits(*),c4

c      print*," QFF",(chits(i),i=1,18)

      iqual_fig = -999	!! default - should never be used
      
c----------------------------------------------------------------------
c-----      check for quality figure = 0	
c      trace too short. fof2 value no good, h'f = 9999, h'F trace probably ES
c      chits(1).eq."q" means that ARTIST confidence level was 44 or 55

      if(chits(1).eq."s".or.chits(1).eq."f".or.chits(1).eq."u".
     a or.chits(1).eq."t".or.chits(1).eq."q") THEN
c      print*," iqual_fig = 0"
      iqual_fig = 0
      return
      ENDIF

c----------------------------------------------------------------------
c-----      check for quality figure = 3	qualcon & ARTIST found no problems

      iall = 1
      DO i = 1,nc
      if(chits(i).ne." ") iall = 0	!! not all blanks
      ENDDO

      if(iall.eq.1.and.iaafcon.le.22) THEN
      iqual_fig = 3
      return
      ENDIF

c----------------------------------------------------------------------
c-----      check for quality figure = 2	medium risk -- could overwrite current value
      if(
     a  chits(2).eq."2".or.       !! M(3000)F2 comes from the end of the F2 trace
     a  chits(3).eq."3".or.      !! Real ht is too close to the virtual ht at start of F2
     a  chits(4).eq."4".or.      !! Too large a difference between POLAN & ARTIST hmF2
     a  chits(5).eq."5".or.       !! Too long a horizontal section in F2 trace (ES?)
     a  chits(7).eq."7".or.      !! Too many sudden jumps in F2 trace
     a  chits(8).eq."8".or.      !! Large errors in POLAN peak fit (probably IpoorFitF2 = 6)
     a  chits(9).eq."9".or.      !! ARTIST value of hmF2 too low
     a  chits(10).eq."A".or.      !! Could not fit good or any LSF curve through F1 trace
     a  chits(12).eq."C".or.      !! Too long a gap between trace segments
     a  chits(13).eq."D".or.      !! There is a corner in the F2 trace
     a  chits(16).eq."G".or.      !! fmin too high relative to fof2
     a  chits(17).eq."H")       !! too few F2 points with non-zero amplitudes 
     a THEN
      iqual_fig = 2
c      print*," iqual_fig = 2"
      ENDIF		!! no, we don't return yet

c----------------------------------------------------------------------
c-----      check for quality figure = 1	high risk -- could overwrite current value
      if(
     a  chits(1).eq."1".or.       !! The Amplitude Quality Figure is too small
     a  chits(6).eq."6".or.       !! Could not smooth F2 trace
     a  chits(11).eq."B".or.      !! Average slope decreasing in last 0.5 MHz	
     a  chits(14).eq."E".or.       !! foF2 is very near a prohibited frequency	
     a  chits(18).eq."I".or.       !! foF2 too close to model value of foF1	
     a  chits(15).eq."F")       !! Slope of F2 trace is too low at end	
     a THEN
      iqual_fig = 1
      ENDIF

c----- Use ARTIST confidence levels      mod 6-july-2007
c      44 and 55 were used earlier to skip the ionogram (Q = 0)

      if(iaafcon.eq.33) iqual_fig = 1	!! ARTIST confidence level

c----- Ugly way to see if QS and A5 agree on bad ionograms - Unit not defined
      I_have_no_probs = 1
      DO i = 1,nc
      if(chits(i).ne." ") I_have_no_probs = 0
      ENDDO
      if(I_have_no_probs.eq.1.and.iaafcon.eq.33) numc=numc+1
      if(I_have_no_probs.eq.0.and.iaafcon.eq.33) numok=numok+1
ccc      if(iaafcon.eq.33)
ccc     a write(77,*) (chits(i),i=1,nc),numok,numc


      return
      END	

c======================================================================
