c+++++ qualscan V2007.2  -- February 2007 Leo F McNamara

c**********************************************************************
c      Some of the subroutines called by QUALSCAN
c**********************************************************************

c      aqf (ioaf,idfi9,delta_f,ampquality)
c      calculate_X_trace
c      correlation
c      dayofmon (iyear,idayno,month,iday)
c      Elayer
c      find_ursi_code
c      flatrace (freq,vht,nv,nflat,fstart,hflat)
c      foeccir
c      fof1OT (zenith,r12,month,dipmod,fof1,chimax)
c      heapsort (n,ra) (Numerical Recipes)
c      ilios(ord,pan,var,n,l,ans)
c      jumpinht (freq,ht,nht,njump,jump,htjump)
c      makeGAIMfile 
c      median (a,n,amed)
c      oblique_propagation    -- NOT for the faint-hearted
c      POLAN.DOC  -  Description and use of the POLynomial ANalysis subroutine:
c      prohibited_frequencies
c      qs_check11
c      qs_check12
c      qs_check16
c      qs_check18
c      qs_check32
c      qs_check33
c      qs_check34
c      qs_check39
c      SAOPLOT
c      set_jumpfail(iflagfile,njump,htjump,virtht,jumpfail)
c      setupPOLANarrays
c      slope_at_end (nval,freq,virt,nslope,slope_end)
c      smoothed_m3000
c      solar
c      spline_fit (spline, splint - Numerical Recipes)
c      starth
c      valley_WandD

c**********************************************************************

      subroutine aqf (ioaf,idfi9,delta_f,ampquality)
c
c      s/r to calculate the Average Relative Amplitude for the echoes in the last "MHz" 
c     of the F2 trace. We are looking for ARTIST traces that have no associated echoes

c---- Uses last half of trace if fewer than 20 points
c
c      Written by leo mcnamara 21-dec-93
c     Extensively modifed 24-oct-01
c-----Last modified 11-january-2006 for short traces
c
c      IOAF                amplitude of F2 trace (2-digit numbers)
c      IDFI9                number of F2 data points IDFI(9)

c      ampquality          % relative amplitude (default = 0)
c
      integer*2 ioaf
      dimension ioaf(*)

      ampquality = -999.

c----- If trace is longer than 2 MHz
      if(delta_f.lt.0.1) nva = 20    !! 20 x 0.05 = 1
      if(delta_f.ge.0.1) nva = 10    !! 10 x 0.10 = 1

c----- If trace is shorter than 2 MHz, use top half of trace
      if(idfi9.le.20) nva = idfi9/2

c-----Average amplitude for the whole F2 trace
      sumamp = 0.
      ii = 0
      DO i = 1,idfi9
       IF (ioaf(i).gt.0) THEN  !! do not include zeros
        ii = ii + 1
        sumamp = sumamp + ioaf(i)
       ENDIF
      ENDDO
      if(ii.eq.0) RETURN
      averamp = sumamp / float(ii)
c
c-----Average for last nva data points - include zero amplitudes
      averampNVA = 0.
      nv = idfi9 - nva
      if(nv.lt.1) print*," AAQF",idfi9,delta_f,nva,nv
c      if(nv.lt.1) pause
      if(nv.lt.1) go to 40 !! F2 trace too short
      sumamp = 0.
      ii = 0
      DO i = nv,idfi9
      ii = ii + 1
      sumamp = sumamp + ioaf(i)
      ENDDO
      averampNVA = 0.
      if(ii.gt.0) averampNVA = sumamp / float(ii)
c
40    continue    !! Arrive here if F2 trace too short
c
c-----Normalized amplitude
      ampquality = averampNVA/averamp * 100.
c
      return
      end

c**********************************************************************

      subroutine calculate_X_trace
     a(fftab,hhtab,idfi51,scaled,hmF2_ART,freq_int,dip,fb,
     a gyro_ht,freq_iono,virt,realht,numpts,iray)

c----- s/r to calculate an ionogram trace, given an ARTIST profile
c----- written for either polarization
c----- uses the ARTIST tabulated profile

c----- leo mcnamara 27-march-2007

c      fftab        tabulated profile
c      hhtab        tabulated profile
c      idfi51        # tabulated points
c      scaled        ARTIST parms
c      hmF2        hmF2
c      freq_int    freq spacing for scaled trace    
c      dip            dip angle in degrees (from SAO)
c      fb            gyrofreq (from SAO)
c      gyro_ht        gyro_ht_ART = 300

c      freq_iono    freqs at which iono trace is calculated
c      virt        trace virtual heights
c      realht        trace real heights
c      numpts        number of trace points

c      iray = 1 for ordinary ray, -1 for Extraordinary ray

c----- Returned from VI_ionogram_QualScan
c      ierr        = 0 for OK. Set to 1 if any dfndh = 0 (vertical profile)
c      Set to 2 if problems in reflecting lamina


      dimension freq_iono(*),virt(*),realht(*),scaled(*)
      dimension fftab(*),hhtab(*)
      
c      dimension fn(900),coeffs(20),ffnn(900),hhtt(900)
      dimension pfn(900),hfn(900),Y2(2000),ftab(900),htab(900)

      foe_art  = scaled(9)
      fof2_ART = scaled(1)
      shf2_art = scaled(40)

c----- Tabular profile htab,ftab,idfi(51) and 52

c----- Fit a spline through the profile, and find plasma freqs at each km
      yp1 = 1.e31
      ypn = 1.e31

c----- Some tables have been found to have identical entries, so
c      we first get rid of these (which SPLINE hates)
      kk = 0
      ftab(1) = fftab(1)
      htab(1) = hhtab(1)
      DO i = 2,idfi51
      if(fftab(i).eq.fftab(i-1).and.hhtab(i).eq.hhtab(i-1)) go to 10
      kk = kk + 1
      ftab(kk) = fftab(i)
      htab(kk) = hhtab(i)
c      print*,kk,ftab(kk),htab(kk)
10      continue
      ENDDO
      idfi51z = kk

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      call SPLINE(htab,ftab,idfi51z,YP1,YPN,Y2)
c      print*," Returned from SPLINE 51"
      ihtab1 = htab(1) + 0.5
      ihfof2 = hmF2_art + 0.5
      numht = 0
c      print*,ihtab1,ihfof2,hmF2_art
      DO i = ihtab1,ihfof2
      hi = i
      call SPLINT(htab,ftab,Y2,idfi51z,hi,fni)
      if(fni.le.foF2_ART) THEN
      numht = numht + 1
      pfn(numht) = fni
      hfn(numht) = hi
c      print*,i,hi,fni
      ENDIF
      ENDDO

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- Calculate the X trace

      fof1_art = scaled(2)
      if(scaled(2).lt.0.or.scaled(2).gt.10.) foF1_ART = 0.
      foE_art = scaled(9)
      if(scaled(9).lt.0.or.scaled(9).gt.10.) foE_ART = 0.
      fmin = (foE_ART + foF2_ART) / 2.
cc      if(foF1_ART.gt.0.) fmin = (foF1_ART + foF2_ART) / 2.
      if(foF1_ART.gt.0.) fmin = foF1_ART + 0.1
      fstep = freq_int 

c      print*," PPPP",foe_ART,fof2_ART,fmin

      call VI_ionogram_QualScan 
     a (iray,fmin,fof2_ART,fstep,pfn,hfn,numht,dip,
     a fb,gyro_ht,foE_ART,foF1_ART,freq_iono,virt,realht,numpts,ierr)

      return
      END

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine VI_ionogram_QualScan 
     a (iray,fmin,fof2,fstep,fn,ht,numht,dip,
     a fbz,gyro_ht,foE,foF1,freq_iono,virt,realht,numfreq,ierror)

cxxxx For more exact GAIM ionograms, we could use the Cheby coeffs for the
c      peak to get dfn/dh. Currently we use a constant value for the slab.
c      However, the effort does not seem worthwhile

c-----s/r to calculate a vertical incidence ionogram corresponding to
c      a given fn(h) profile. Calculation is a numerical integration of
c      (MU' - 1)dh.

c----- leo mcnamara 19-august-2004

c      iray = 1 for ordinary ray, -1 for Extraordinary ray
c      fmin    virtual height calculations start at fmin (defined in calling prog)
c      fstep    calculate h' at fmin(fstep)foF2 (nominally)
c      fn        array of plasma frequencies
c      ht        array of heights corresponding to fn
c      numht    number of points in fn & ht arrays
c      dip        dip angle in degrees
c      fbz        gyrofrequency at the ground
c      gyro_ht        height at which gyrofreq is to be specified

c      freq_iono    array of freqs at which h' is calculated
c      virt        array of virtual heights
c      realht        array of real heights of reflection
c      numfreq        number of frequencies at which h' is calculated
c      ierror        = 0 for OK. Set to 1 if any dfndh = 0 (vertical profile)
c              Set to 2 if problems in reflecting lamina

      dimension fn(*),ht(*),freq_iono(*),virt(*),realht(*)


c      print*,fmin,fof2,fstep,numht,dip,fbz,gyro_ht,foE,foF1
c      DO i = 1,numht
c      print*,i,ht(i),fn(i)
c      ENDDO

      ierror = 0
      numfreq = 0
      npoly = 4    !! npoly = order of interpolation
      ng = 4        !! ng-point guassian quadrature for each slab
      fstepp = fstep
      fb = fbz    !! mod 29-march-2007

c----- GIND already initialized in QualScan
c-----Initialize the group refractive index s/r, GIND
c      dip = abs(dip)
c      trash = gind (fbz,-dip)
c      scale GIND to the height 250 km (i.e. GIND will use the gryofreq
c      at 250 km)
c      trash = gind (0.,gyro_ht)
      
c-----Calculate the ionogram for freqs > f_min

      freq = fmin - fstepp
      DO ifreq = 1,400
      if(freq.gt.0.9*fof2) fstepp = fstep/2.!! smaller step size near foF2 05-august-2005
      freq = freq + fstepp
      fr = freq    !! plasma fn at reflection for O ray

c      X ray        X ray    X ray    X ray    X ray    X ray    X ray    X ray
c      Plasma freq at which reflection occurs
      if(iray.eq.-1.and.freq.le.fbz) THEN    !! dummy calculations
      fr = fbz + 0.2
       ENDIF
      IF(iray.eq.-1.and.freq.gt.fbz) THEN
      fr = sqrt(freq*(freq-fbz))
      ENDIF

c-----Ignore freqs above foF2-fstep (keep clear of tip of profile to avoid trouble)
c      if(fr.ge.fof2 - 0.5*fstepp)    go to 50
      if(fr.ge.fof2 - 0.01)    go to 50    !! 18-march-2007

      numfreq = numfreq + 1

c-----Find the last slab penetrated
      DO i = 1,numht
c      print*,fr,fn(i)
      if(fn(i).gE.fr) THEN    !! gt changed to gE 04-august-2005
      lastpene = i - 1    !! last slab penetrated starts at fn(lastpene)
      go to 6
      ENDIF    !! fn(i).gt.fr
      ENDDO
6      continue
      if(lastpene.eq.0) THEN
      print*," LastPene=0",i,fn(i),fr
      go to 49    !! bypass thsi freq
      ENDIF
c      print*," LASTPENE",ifreq,lastpene,fr,fn(lastpene),fn(lastpene+1)

c-----Find the height at which this freq is reflected      (fn = fr)
      call height_at_fn(npoly,fn,ht,numht,foe,fof1,fof2,fr,hr)
c      print*," HR+",fr,hr,ht(lastpene),ht(lastpene+1),foe,fof1,fof2
c      pause

      if(hr.eq.ht(lastpene)) lastpene = lastpene - 1    !! special case (rare)

c-----Contributions from the penetrated slabs
c      fn(lastpene+1) is the plasma freq at the bottom of the reflecting slab

      virtht = 0
      delay_pene = 0
      delay_refl = 0
      lastpenem1 = lastpene-1

      DO i = 1,lastpenem1

      h1 = ht(i)
      h2 = ht(i+1)
      if(h2.eq.h1) stop "h2=h1"
      dfndh = (fn(i+1) - fn(i)) / (h2 - h1)

      if(dfndh.eq.0) THEN    !! error return for vertical profile
      print*," Equal FN",i,lastpene,fn(i+1),fn(i)
      ierror = 1
      return
      ENDIF
      
      dhdfn = 1. / dfndh

      call slab_delay     !! delay in penetrated lamina
     a(freq,fn,ht,numht,foe,fof1,fof2,h1,h2,dhdfn,iray,fb,ng,npoly,delh)
      delay_pene = delay_pene + delh
cc      delh_pene = delh

      ENDDO    !! i = 1,lastpenem1
      
c-----Contribution from reflecting slab/lamina
      h1 = ht(lastpene)
      h2 = hr

c-----Error return - should not be used - see hr.eq.ht(lastpene) above
      if(h2.eq.h1) THEN
      ierror = 2
      return
      ENDIF
      
      dfndh = (fr - fn(i)) / (h2 - h1)
      if(dfndh.eq.0) THEN    !! error return
      print*," fr - fn(i)",i,lastpene,fn(i+1),fn(i),fr
c      pause
      ierror = 1
      return
      ENDIF

      dhdfn = 1. / dfndh
      call slab_delay     !! delay in reflecting lamina
     a(freq,fn,ht,numht,foe,fof1,fof2,h1,h2,dhdfn,iray,fb,ng,npoly,delh)
      delay_refl = delay_refl + delh
      delh_refl = delh

      virtht = delay_pene + delay_refl + hr    !! total delay

c      print*," Total delay",delay_pene,delay_refl,virtht,freq,fr,hr
c      print*, "DFNDH ",dfndh,fr,fn(i),h2,h1

      
      freq_iono(numfreq) = freq
      virt(numfreq) = virtht
      realht(numfreq) = hr

c      write(06,13) freq,hr,virtht    !! to save ionogram
c      write(33,13) freq,hr,virtht    !! to save ionogram    DEBUG33

13    format(" Ionogram",8f10.3)

49    continue

      ENDDO    !! ifreq
      
50    continue    !! fr.ge.fof2

      return
      END

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine height_at_fn
     a      (npoly,fn,ht,numht,foe,fof1,fof2,fnz,hr)
     
c      s/r to find the height in a GAIM profile corresponding to a given fn

c      leo mcnamara 7-feb-2003

c      Find the height at which this freq is reflected    (fn = fr)
c      Since the profile is possibly double-functioned, we interpolate
c      in either the "F1" layer or the F2 layer.
c      There are 3 cases

      dimension ht(1),fn(1)    !! full PRISM profile
      dimension hh(800),ff(800)    !! interpolation arrays

      hr = ht(1)
      if(fnz.eq.0.) return

c      print*," foe,fof1,fof2,fnz",foe,fof1,fof2,fnz

c      print*," NUMHTTT height_at_fn_GAIM",numht,fr,foe,fof1,fof2

      fr = fnz    !! 19-august-2004 -- how did it ever work before???

      if(fr.lt.foe) THEN    !! case 0: simple E layer
c      Set up the interpolation array
      kint = 0
      DO i = 1,numht
      kint = kint + 1
      ff(kint) = fn(i)
      hh(kint) = ht(i)
c      print*," height_at_fn .. fr.lt.foe",kint,ff(kint),hh(kint)
      ENDDO
c      print*,"ILIOS1"
c      print*,(ff(i),hh(i),i=1,kint)
c      print*,"FNZ",fnz
      call ilios (hh,ff,fnz,kint,npoly,ans)

      hr = ans    !! reflection height
      ENDIF    !! fr.lt.foe
cc      print*,"fr.lt.foe",hr

      if(fr.gt.foe.and.fof1.eq.0) THEN    !! case 1: simple F2 layer
c      Set up the interpolation array
      kint = 0
      DO i = 1,numht
c      if(fr.gt.fn(i).and.fr.lt.fof2) THEN
c      if(fn(i).gt.foe.and.fr.lt.fof2) THEN    !! MODIFIED 05-august-2005
      if(fn(i).gt.foe.and.fn(i).lt.fof2) THEN    !! MODIFIED 08-march-2007
      fni = fn(i)
      fnnumht = fn(numht)
      kint = kint + 1
      ff(kint) = fn(i)
      hh(kint) = ht(i)
cc      print*," Case 1",kint,ff(kint),hh(kint),fr,foe,fni,fnnumht
      ENDIF
      ENDDO
c      print*,"ILIOS2"
      call ilios (hh,ff,fr,kint,npoly,ans)
c      print*,"ILIOS2",foe,fr,ans
      hr = ans    !! reflection height
      ENDIF    !! fof1.eq.0

      if(fof1.gt.foe.and.fr.lt.fof1.and.fr.gt.foe) THEN    !! case 2: in F1 layer
c      Set up the interpolation array
      kint = 0
      DO i = 1,numht
      if(fn(i).gt.foe.and.fn(i).lt.fof1) THEN
      kint = kint + 1
      ff(kint) = fn(i)
      hh(kint) = ht(i)
c      print*," Case 2",kint,ff(kint),hh(kint),fr,foe
      ENDIF
      ENDDO
c      print*,"ILIOS3"
      call ilios (hh,ff,fr,kint,npoly,ans)
      hr = ans    !! reflection height
      ENDIF    !! fof1.eq.0

      if(fof1.ne.0.and.fr.gt.fof1.and.fr.lt.fof2) THEN    !! case 3: in the F2 layer
c      Set up the interpolation array
       kint = 0
      DO i = 1,numht
      if(fn(i).gt.fof1.and.fn(i).lt.fof2) THEN
      kint = kint + 1
      ff(kint) = fn(i)
      hh(kint) = ht(i)
c      print*," Case 3",kint,ff(kint),hh(kint),fr,foe
      ENDIF
      ENDDO
c      print*,"ILIOS4"
      call ilios (hh,ff,fr,kint,npoly,ans)
      hr = ans    !! relection height
      ENDIF

c      print*,"HEIGHT_AT_FNZ",foe,fof1,fof2,fnz,hr

      return
      end

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine slab_delay 
     a(freq,fn,h,numh,foe,fof1,fof2,h1,h2,dhdfn,iray,fb,ng,npoly,delay)

c      s/r to calculate the group delay through a slab of ionosphere

c      written by leo mcnamara 2 feb 2003

c      s/r handles both penetration and reflection

c      We start with INT MU' dh. There are 3 changes of variable:
c      1. from h to fn, since MU' is a function of fn, not h
c      This gives INT MU' (dh/dfn) dfn, which has a singularity at the peak
c      2. From fn to t = sqrt[1-(fn/fr)**2)] - fr is plasma freq at reflection
c      Integration limits are 0 at the base of the ionosphere, and 1 
c      at reflection. In general t1 to t2 (from fn1 to fn2) (from h1 to h2)
c      3. Gaussian quadrature is for dx from -1 to +1. We have t1 to t2.
c      So change to t = (b-a)/2 * x + (b+a)/2    ( we have a = t1, b=t2)
c      See Abrahowitcz & Stegun formulas 25.4.29, 25.4.30

c      fr is the plasma freq at reflection
c      fr = fn for O ray
c      fr**2 = fn(fn-fb) for X ray

c      freq    operating frequency
c      fn        PRISM profile plasma freqs
c      h        PRISM profile altitudes
c      numh    number of points on PRISM profile 
c      h1        height at bottom of slab
c      h2        height at top of slab (penetration) or at reflection
c      dhdfn    gradient of h wrt fn (assumed constant across a slab)
c      iray = 1 for O ray; = -1 for X ray; 0 for no field
c      fb        gyrofrequency - adjusted to 250 km by GIND 250 km
c      ng        order of Gaussian quadrature (e.g. 4 for 1 km slabs)
c      npoly    Interpolation in PRISM profile uses npoly-order poly interp

      dimension qp(16),qw(16)    !! up to 16-point quadrature
      dimension fn(1),h(1)    !! h(fn) profile
c
      delay = 0.

      fr = freq
c      Plasma freq at which reflection occurs
      if(iray.eq.-1.and.freq.le.fb) THEN    !! dummy calculations
      fr = fb + 0.2
       ENDIF
      IF(iray.eq.-1.and.freq.gt.fb) THEN
      fr = sqrt(freq*(freq-fb))
      ENDIF

c      print*,"FR",fr
c      Guassian quadrature points and weight
      call QUADPW (NG,QP,QW)

c      Find the fn at the limits h1 and h2
      call ilios (fn,h,h1,numh,npoly,fn1)    !! poly interp
      call ilios (fn,h,h2,numh,npoly,fn2)
c      print*,"h1,h2,fn1,fn2,fr",h1,h2,fn1,fn2,fr
c      pause

c      Integration limits in terms of t
      if(fn1.gt.fr) PRINT*,"Reflection cannot occur in this layer"
      t1 = sqrt (1.-(fn1/fr)**2)
      x = fn2 / fr
      if(x.gt.1.0) THEN
      x = 1.    !! reflection x = 1, t = 0
      ENDIF
      t2 = sqrt (1.- x*x)

c      Integral is from a to b. Gauss is for -1 to +1
c      t = (b-a)/2 *x + (b+a)/2
      bma = (t2 - t1) / 2.
      bpa = (t2 + t1) / 2.
c      print*,t1,t2
      delay = 0. 

      DO i = 1,ng

      tee = bma * qp(i) + bpa
c      print*,"TEE",tee,bma,bpa
      fng = fr * sqrt(1.-tee*tee)

      if(iray.eq.1) tem = gind (freq,tee)      !! GIND gives MU' - 1
      if(iray.eq.-1) tem = gind (-freq,tee)

      term = dhdfn * tem / fng * tee
      delay = delay + term * qw(i)
      ENDDO    !! end of qudrature summation

      delay = delay * fr * fr    !! mod 26-june-2006
      delay = delay * bma
      delay = -delay    !! minus comes with change of variable

      return
      end

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE QUADPW (NG,QP,QW)
C     S/R TO SELECT THE GAUSSIAN QUADRATURE POINTS (QP) AND
C     WEIGHTS (QW) CORRESPONDING TO THE GIVEN ORDER OF INTEGRATION
C     NG.   NG MUST BE 4,8,12 OR 16.
C     CREATED BY LEO MCNAMARA 10-SEP-82
C.....LAST MODIFIED (LFM) 13-SEP-82
C
      DIMENSION QP4(4),QW4(4),QP8(8),QW8(8),QP12(12),QW12(12),
     A QP16(16),QW16(16),QP(NG),QW(NG)
C
C#####POINTS AND WEIGHTS FOR A 4-POINT GUASSIAN QUADRATURE
      DATA QP4 /
     A -0.86113 63116, -0.33998 10436, 0.33998 10436,
     A  0.86113 63116/
      DATA QW4 /
     A 0.34785 48451, 0.65214 51549, 0.65214 51549,
     A 0.34785 48451/
C##### POINTS AND WEIGHTS FOR AN 8-POINT GUASSIAN QUADRATURE
      DATA QP8 /
     A -0.96028 98565, -0.79666 64774, -0.52553 24099, -0.18343 46425,
     A  0.18343 46425,  0.52553 24099,  0.79666 64774,  0.96028 98565/
      DATA QW8 /
     A 0.10122 85363, 0.22238 10345, 0.31370 66458, 0.36268 37834,
     A 0.36268 37834, 0.31370 66458, 0.22238 10345, 0.10122 85363/
C
C#####POINTS & WEIGHTS FOR A 12-POINT QUADRATURE
      DATA QP12 /
     A -0.98156 06342, -0.90411 72564, -0.76990 26742, -0.58731 79543,
     A -0.36783 14990, -0.12523 34085,  0.12523 34085,  0.36783 14990,
     A  0.58731 79543,  0.76990 26742,  0.90411 72564,  0.98156 06342/
      DATA QW12 /
     A 0.04717 53364, 0.10693 93260, 0.16007 83285, 0.20316 74267,
     A 0.23349 25365, 0.24914 70458, 0.24914 70458, 0.23349 25365,
     A 0.20316 74267, 0.16007 83285, 0.10693 93260, 0.04717 53364/
C
C#####POINTS & WEIGHTS FOR A 16-POINT QUADRATURE
      DATA QP16 /
     A -0.98940 09350, -0.94457 50231, -0.86563 12024, -0.75540 44084,
     A -0.61787 62444, -0.45801 67777, -0.28160 35508, -0.09501 25098,
     A  0.09501 25098,  0.28160 35508,  0.45801 67777,  0.61787 62444,
     A  0.75540 44084,  0.86563 12024,  0.94457 50231,  0.98940 09350/
      DATA QW16 /
     A 0.02715 24594, 0.06225 35239, 0.09515 85117, 0.12462 89713,
     A 0.14959 59888, 0.16915 65194, 0.18260 34150, 0.18945 06105,
     A 0.18945 06105, 0.18260 34150, 0.16915 65194, 0.14959 59888,
     A 0.12462 89713, 0.09515 85117, 0.06225 35239, 0.02715 24594/
C
      IF(NG.NE.4) GO TO 20
      DO 10 I=1,NG
      QP(I)=QP4(I)
10    QW(I)=QW4(I)
      RETURN
20    IF(NG.NE.8) GO TO 40
      DO 30 I=1,NG
      QP(I)=QP8(I)
30    QW(I)=QW8(I)
      RETURN
40    IF(NG.NE.12) GO TO 60
      DO 50 I=1,NG
      QP(I)=QP12(I)
50    QW(I)=QW12(I)
      RETURN
60    IF(NG.EQ.16) GO TO 65
      WRITE(6,66) NG
66    FORMAT(//10X,' NG MUST BE 4,8,12 OR 16. IT IS =',I3)
      STOP 66
65    DO 70 I=1,NG
      QP(I)=QP16(I)
70    QW(I)=QW16(I)
      RETURN
      END
      
c**********************************************************************

      subroutine correlation
     a  (x,y,n,slope,fintercept,xbar,ybar,correl_coeff)
c
c       s/r to calculate the slope & intercept of the linear regression line
c       (SLOPE, FINTERCEPT), the average values (XBAR, YBAR) and the correlation
c       coefficient for two associated series (X, Y).
c
c       written by leo mcnamara 13-nov-89
c-----last modified 8-nov-2001 to cover horizontal and vertical straight
c     lines correctly

c      check performed with test_LSF.for 14-oct-2003
c      S/R is CORRECT!!!

c
c     x     x values
c     y     y values
c     n     number of (x,y) pairs
c     slope slope of the LSF straight line
c     fintercept  intercept of the LSF straight line
c     xbar  average value of x
c     ybar  average value of y
c     correl_coeff      correlation coefficient
c
      dimension x(*),y(*)
c
      sx = 0.
      sy = 0.
      sxx = 0.
      syy = 0.
      sxy = 0.
      fn = n
c
      DO i = 1,n
      sx = sx + x(i)
      sy = sy + y(i)
      sxx = sxx + x(i)**2
      syy = syy + y(i)**2
      sxy = sxy + x(i) * y(i)
      ENDDO
c
      xbar = sx / fn
      ybar = sy / fn
      IF (sxx/fn.le.xbar**2) THEN    !! vertical straight line 
       s_xx = 0.
      ELSE
       s_xx = sqrt (sxx / fn - xbar**2)
      ENDIF
c
      IF (syy/fn.le.ybar**2) THEN    !! horizontal straight line
       s_yy = 0.
      ELSE
       s_yy = sqrt (syy / fn - ybar**2)
      ENDIF
      s_xy = sxy / fn  - xbar * ybar
c
c       regression line
      slope = 0.     !! horizontal line
      fintercept = 0.
      if(s_yy.ne.0.) THEN
      slope = s_xy / s_xx **2
      fintercept = ybar - slope * xbar
      ENDIF
c
      slope = 9999. !! vertical line
      fintercept = 9999.
      IF (s_xx.ne.0.) THEN
       slope = s_xy / s_xx **2
       fintercept = ybar - slope * xbar
      ENDIF
c

c       correlation coefficient
      correl_coeff = 1.0
      if(s_xx * s_yy.ne.0.) correl_coeff = s_xy / s_xx / s_yy
c
      return
      end    
      
c**********************************************************************

      subroutine dayofmon (iyear,idayno,month,iday)
c       s/r to return the month and day corresponding to a day of year
c
c       written by leo mcnamara 5-sep-93
c-----    last modified 2-nov-2001
c
      dimension ids(13),idsleap(13)   !!  day no. of start of month
      data ids     /1,32,60,91,121,152,182,213,244,274,305,335,366/
      data idsleap /1,32,61,92,122,153,183,214,245,275,306,336,367/
c
c       Check for leap year. 2000 was a leap year (Windows 2000)
      if(mod(iyear,4).eq.0) go to 20
c
c       Not a leap year
      DO i = 1,12
       IF (idayno.ge.ids(i).and.idayno.lt.ids(i+1)) THEN
        month = i
        iday = idayno - ids(i) + 1
        return
       ENDIF
      ENDDO
cx      print*,iyear,idayno,month,iday
      stop ' DISASTER in DAYofMON'
c
c       A leap year
20    continue
      DO i = 1,12
       IF (idayno.ge.idsleap(i).and.idayno.lt.idsleap(i+1)) THEN
        month = i
        iday = idayno - idsleap(i) + 1
        return
       ENDIF
      ENDDO
      stop ' DISASTER in DAYofMON'
c
      end
      
c**********************************************************************

      subroutine Elayer
     a (flux12,r12,lyear,lmon,lday,lh,lm,flat,flon,reflon,
     a  chi,chi_noon,foe,yme,hme)
c
c      s/r to calculate the E-layer parameters foe, yme & hme
c
c      written by leo mcnamara 10-sep-86
c      modified to return chi & chi_noon 13-sep-88
c-----      last modified 14-sep-88
c
c      uses the ccir method of calculating foe (s/r foeccir),
c      Titheridge's experimental model for hme, and mcnamara's h(0.5)
c      model to find yme.
c
      pi = 3.141592654
      dtr = pi / 180.
c
c      first find the solar zenith angle at noon, zen12
      call solar (lyear,lmon,lday,12, 0,flat,flon,reflon,dec,zen12,r,s)
      chi_noon = zen12
c
c      find the zenith angle 0.05 hours earlier, to account for the
c      sluggishness of the ionosphere in calculating foE
      kh = lh
      km = lm
      kd = 3
      if(km.lt.kd) kh = kh - 1
      if(kh.lt.0) kh = kh + 24
      km = km - kd
      if(km.lt.0) km = km + 60
      call solar (lyear,lmon,lday,kh,km,flat,flon,reflon,dec,zensl,r,s)
c
c      Becker's values of ymf2 at noon & midnight (comparison purposes)
cccc      call becker (flux12,zen12,ymnoon,ymmid)
ccc      print*,' Beckers values ymF2  Noon & midnight ', ymnoon,ymmid
c
c      calculate zenith angle at given local time, & the times of sunrise/set
      call solar 
     a   (lyear,lmon,lday,lh,lm,flat,flon,reflon,dec,zen,rise,set)
      chi = zen
      hourlt = lh + float(lm)/60.
      call foeccir 
     a       (flux12,r12,hourlt,zen,zen12,zensl,rise,set,flat,FOE)
ccc      type*,' E layer - hourlt,dec,zen,rise,set,foe',
ccc     a                    hourlt,dec,zen,rise,set,foe
c
c      Titheridge formula for hme - upper limit set to 120km for
c      large zenith angles
      hme = 120.
      if(zen.ge.77.10) go to 20
      ang = zen * dtr
      hme = 108. + 8. * alog (1./cos(ang))
20    continue
c
c      find yme by making quasi-parabola pass through the 0.5MHz model
c      given by mcnamara (JATP 40,543-547,1979)
c      set yme to MSIS83 value for zenith angles > 77.1
c      temp120 is the MSIS temperature at 120km (gives yme = 120)
      temp120 = 370.
      yme = 2.0 * 0.027 * temp120
      if(zen.ge.77.1) return
c      JATP eqn 5 - change season for southern hemisphere
      mon = lmon
      if(flat.lt.0.) mon = mon + 6
      if(mon.gt.12) mon = mon - 12
      s = (float(mon) - 0.5)/12.*2.*pi
      hh = 91.5 - 5.6*cos(zen*dtr) - 1.4*cos(s) -0.027*r12
c
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c      force the value of HH to exceed 90km - PATCH of the worst kind !!
c      patch REMOVED 17-aug-87
ccc      if(hh.lt.90.) hh = 90.
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c
c      layer must pass through (hh,0.5)
      rz = 6370.
      rme = rz + hme
      r = rz + hh
      fr = 0.5 / foe
      fac = sqrt(1.-fr*fr)
      fac = fac / (rme/r - 1.) + 1.
      yme = rme / fac
c
c      type*,' E layer - hourlt,dec,zen,rise,set,foe,yme,hme,h(0.5)',
c     a                    hourlt,dec,zen,rise,set,foe,yme,hme,hh
      return
      end

c**********************************************************************

      subroutine find_ursi_code
     a (sysdes,num_ursi,ursi_code,ursi_number,statid,ierror)
     
c----- s/r to search the ARTIST sysdes and derive the URSI code

c      leo mcnamara 21-april-2004

c----- There are several possibilities
c      1. There is a \ followed by the URSI code
c      2. The first 7 chars are DGS-256
c      3. The first 5 chars are DPS-4 or DPS-1

c----- For cases 2 and 3, the next three chars give the station number, from
c      which we can deduce the code, using the two ursi arrays

      character*120 Sysdes
      character*5 ursi_code,statid
      character*4 ctemp4
      character*3 ursi_number,ctemp3
      
      dimension ursi_code(*), ursi_number(*)
      
c----- See if we have case 1
      ik = 0
      DO i = 1,119
      if(sysdes(i:i).eq."/") ik = i    !! look for unique char (/)
      ENDDO

c----- Check for missing URSI ID
      j = ik+1
      if(ik.gt.0.and.sysdes(j:j).eq.",".or.sysdes(j:j).eq." ") go to 10

      if(ik.ne.0) THEN
      statid = sysdes((ik+1):(ik+5))
      ierror = 0
      RETURN
      ENDIF

10      continue

c----- See if we have case 2
      ctemp4 = sysdes(1:4)
      if(ctemp4.eq."DGS-") THEN    !! Digisonde 256
      ctemp3 = sysdes(9:11)
      ENDIF
      
c----- See if we have case 3
      ctemp4 = sysdes(1:4)
      if(ctemp4.eq."DPS-") THEN    !! DPS-4
      ctemp3 = sysdes(7:9)
      ENDIF

c----- Retrieve the URSI code
      DO i = 1,num_ursi
      iii = i
      if(ctemp3.eq.ursi_number(i)) go to 20
      ENDDO
      ierror = 1
      RETURN

20    continue
      statid = ursi_code(iii)
      ierror = 0
      
      RETURN
      END
          
c**********************************************************************

      subroutine flatrace (freq,vht,nv,nflat,fstart,hflat)
c
c      s/r to search for the longest flat section in a scaled h' trace
c
c      FREQ    frequency array
c      VHT    array of virtual heights
c      NV    size of this array
c
c      NFLAT    largest number of points in a flat part of the trace VHT
c      FSTART    starting freq of longest flat section
c      HFLAT    the height of the longest flat part
c
c      written by leo mcnamara 21-oct-93
c-----      last modified 21-oct-93
c
      dimension freq(*),vht(*),temh(666),f1(666),ht(666),length(666)
c
      DO i = 1,200
       temh(i) = 0.
      ENDDO
c
c      Save any repeated heights in TEMH
      nvm1 = nv - 1
      DO i = 2,nvm1
       if(vht(i).eq.vht(i-1).or.vht(i).eq.vht(i+1)) temh(i) = vht(i)
      ENDDO
c
c      Scan TEMH for sections of constant height - up to 50 sections
c
      kk = 0        !!    number of flat sections
      do 50 ii = 2,nv
      if(temh(ii).gt.0.and.temh(ii).ne.temh(ii-1)) THEN
c      We are at the start of a flat section
      kk = kk + 1
      f1(kk) = freq(ii)
      ht(kk) = temh(ii)
      ELSE
      go to 50
      ENDIF
c      Find length of flat section
      k = 0        !!    number of points in this section
      do 40 i = ii,nvm1
      if(temh(i).eq.temh(i+1)) THEN
      k = k + 1
      ELSE
      go to 45
      ENDIF
40    continue
45    continue
      length(kk) = k + 1
c
50    continue
c
c      Longest flat section
      maxk = 1
      DO k = 2,kk
      if(length(k).gt.length(maxk)) maxk = k
      ENDDO
      fstart = f1(maxk)
      nflat = length(maxk)
      hflat = ht(maxk)
c
      return
      end
      
c**********************************************************************

      subroutine foeccir
     a  (flux12, r12, hourlt, zenith, chinoon, chisl, sunrise, 
     a   sunset, geoglat, foe)
c
c      s/r to calculate foe, using the CCIR method (see Supplement to 
c      Report 252-2)
c
c      written by leo mcnamara 2-sep-86
c-----      last modified 5-sep-90
c
c.............
c      flux12        solar 10cm flux - 12-month smoothed
c      r12        sunspot number - 12-month smoothed
c      hourlt        local time hour
c      zenith        solar zenith angle - degrees
c      chinoon        solar zenith angle at noon - degrees
c      chisl        solar zenith angle 0.05 hours before hourlt
c      sunrise        local time of sunrise (zenith angle = 90)
c      sunset        local time of sunset
c      geoglat        north geographic latitude - degrees
c
c      foe        foe
c.............
c
      real lambda
      lambda = abs(geoglat)
      dtr = 3.141592654 / 180.
c
c      minimum value of foe (Wakai, 1971)
      foemin = (0.017 * (1. + 0.0098 * r12) **2 ) ** 0.25
c
c      return with this value if sun never rises
      foe = foemin
      if(sunrise.lt.-90.and.sunset.lt.-90.) return
c
c      the exponent "m" and the terms A and B
      coslam = cos (lambda*dtr)
      if(lambda.ge.32.0) go to 20
      em = -1.93 + 1.92 * coslam
      a = 23.
      b = 116.
      go to 30
20      em = 0.11 - 0.49 * coslam
      a = 92.
      b = 35.
30    continue
c
c      the terms chi_dash  and p 
      chid = zenith
      if(lambda.gt.23.) chid = chisl
      p = 1.20
      if(lambda.le.12.0) p = 1.31
c
c      the term "d"
      if(chid.gt.73.0) go to 40
      d = (cos(chid*dtr)) ** p
      go to 70
40    if(chid.ge.90.0) go to 50
      delchid = 6.27e-13 * (chid - 50.) ** 8
      d = ( cos( (chid - delchid)*dtr ) ) ** p
      go to 70
50    t = hourlt
      t1 = sunrise
      t2 = sunset
      if(abs(sunrise).gt.24.or.abs(sunset).gt.24)
     a  print*,' Sunrise & sunset ', sunrise,sunset
c      allow for sun never rising - sunset = -99  ..mod 23-aug-90
      d = 0.
      if(t2.le.0.) go to 70    
c      we may accidentally get here during the day, so set up 
c      suitable value for D
      delchid = 6.27e-13 * (chid - 50.) ** 8
      con = cos( (chid - delchid)*dtr ) 
      d = 0
      if(con.ge.0.) d = con ** p
      if(t.ge.0.0.and.t.lt.t1)  d = (0.077)**p * exp(-1.68*(t1-t))
      if(t.ge.t2.and.t.lt.24.0) d = (0.077)**p * exp(-1.01*(t-t2))
70    continue
c
c      now back to equation 6 for foe ..
      termc = a + b * coslam
      termd = d
      brak = cos (chinoon * dtr) 
      termb = 0.
      if(brak.gt.0.) termb = brak  ** em
      terma = 1.0 + 0.0094 * (flux12 - 66.) 
      term = terma * termb * termc * termd
cccc      type*,' FOE LAYERS a,b,c,d', terma,termb,termc,termd
      foe = 0.
      if(term.gt.0.) foe = term ** 0.25
c
c      keep foe above the nighttime minimum value
      if(foe.lt.foemin) foe = foemin
c
      return
      end

c**********************************************************************

      subroutine fof1OT (zenith,r12,month,dipmod,fof1,chimax)
c
c      s/r to evaluate foF1 and the value of solar zenith angle at
c      which foF1 becomes zero
c
c      see "The numerical representation of the critical frequency of
c      the F1 region of the ionosphere", U.S. Dept of Commerce Office of
c      Telecommunications report 73-22 by R.K.Rosich & W.B.Jones, 1973.
c
c      copied by leo mcnamara 2-sep-86
c-----      last modified 2-sep-86
c
c...............................
c      zenith        solar zenith angle in degrees
c      r12        sunspot number - 12-month smoothed
c      month        month of year (1 to 12)
c      dipmod        Rawer modified dip in degrees
c
c      foF1        foF1 in MHz
c      chimax        the solar zenith angle at which foF1 becomes zero
c              foF1 is set = -1. when zenith > chimax
c..............................
c
      dimension c(3),s(3),abc(3),uvw(3),abcr(3),de(2),xy(2),
     a            der(2),ff(3,7),gg(3,7),hh(2,7),jj(2,7)
      real jj
c
      data d2r / 0.01745329252 /
c
      data ff / +2.731571774e+00, 2.922599724e+00, -1.424253677e+00, 
     a            -1.938569077e-02, 2.747632206e-02, -1.253989254e-02,
     a            -5.502616589e-02, 3.816418193e-01, -3.291256471e-01,
     a             3.168712348e-02,-1.925121761e-01,  1.391033503e-01,
     a            -9.132491683e-02, 3.368776479e-01, -2.537423654e-01,
     a            6 * 0.0 /
c
      data gg /  3.216896233e-03, 5.813409815e-03,  4.272985193e-03,
     a             4.333407272e-04,-3.570802646e-03,  2.355164794e-03,  
     a            -1.967480443e-04, 6.104587828e-04,  8.772529229e-04,
     a            -2.010965922e-04, 6.342852489e-04,  1.125034531e-03,
     a            -1.153490288e-04, 7.715714758e-04, -3.805614657e-03,
     a            -5.060336623e-04, 1.318911492e-03, -6.265522583e-04,
     a            -1.693499682e-04, 2.377225732e-03, -2.339569077e-03 /
c
      data hh /  9.529308825e+01,-4.457102130e+01,  1.875351482e+00,
     a            -1.727308857e+00, 3.416911504e+00, -5.716319625e+00,
     a            -7.431739835e-01,-2.072159697e+00, -1.509294719e+00,
     a             7.673016681e+00, 4*0.0 /
c
      data jj /  3.206102780e-02,-1.110502726e-01, -1.556266677e-02,
     a             3.168785858e-02,-2.272330798e-02,  2.166970987e-02,
     a            -7.044257463e-05, 3.703201399e-03, -1.130401518e-02,
     a             3.074219500e-02, 0.0,              1.232533191e-02,
     a             0.0,            -3.996897503e-02 /
c
c      put angles in radians & evaluate trig functions of month number   
      chi = zenith * d2r
      rmmd = dipmod * d2r
      chicos = cos(chi)
      tm = (30.0 * (month-1) ) * d2r
      c(1) = cos (tm)
      c(2) = cos (2.*tm)
      c(3) = cos (3.*tm)
      s(1) = sin (tm)
      s(2) = sin (2.*tm)
      s(3) = sin (3.*tm)
c
c      evaluate FF and GG Fourier coefficients, giving abc & uvw for a
c      given month and all running average sunsspot numbers (median foF1)
c
      do 10 i = 1,3
      abc(i) = ff(i,1)
      uvw(i) = gg(i,1)
      do 10 j = 1,3
      abc(i) = abc(i) + ff(i,2*j) * s(j) + ff(i,2*j+1) * c(j)
10      uvw(i) = uvw(i) + gg(i,2*j) * s(j) + gg(i,2*j+1) * c(j)
c
c      evaluate HH and JJ Fourier coefficients, giving de & xy for a given
c      month and all running average sunspot numbers (maximum solar zenith
c      angle)
c
      do 12 k = 1,2
      de(k) = hh(k,1)
      xy(k) = jj(k,1)
      do 12 l = 1,3
      de(k) = de(k) + hh(k,2*l) * s(l) + hh(k,2*l+1) * c(l)
12    xy(k) = xy(k) + jj(k,2*l) * s(l) + jj(k,2*l+1) * c(l)
c
c      do sum for coefficients in final expressions for fof1 and chimax
c
      do 14 m = 1,3
      if(m.eq.3) go to 14
c
c      evaluate de & xy to give der for one month and r12 (chi maximum)
      der(m) = de(m) + xy(m) * r12
c
c      evaluate abc & uvw to give abcr for one month and r12 (median foF1)
14    abcr(m) = abc(m) + uvw(m) * r12
c
c      evaluate final expression for foF1 and chimax, and test zenith
c      vs. chimax
c
      fof1 = abcr(1) + abcr(2) * chicos + abcr(3) * chicos * chicos
      chimax = der(1) + der(2) * cos(rmmd)
      if(zenith.gt.chimax) fof1 = -1.0
c
      return
      end

c**********************************************************************

      subroutine heapsort (n,ra)
c      s/r to sort an array RA of length N into ascending numerical order
c      RA is OVERWRITTEN by the sorted arrangement
c
c      see Numerical Recipes by Press et al, pg 231
c
c      typed in by Leo McNamara 2-feb-90
c-----        last modified 2-feb-90
c
      dimension ra(n)
      l = n / 2 + 1
      ir = n
10    continue
      if(l.gt.1) then
          l = l - 1
          rra = ra(l)
      else
          rra = ra(ir)
          ra(ir) = ra(1)
          ir = ir - 1
          if(ir.eq.1) then
          ra(1) = rra
          return
          endif
      endif
      i = l
       j = l + l
20     if(j.le.ir) then
          if(j.lt.ir) then
          if(ra(j).lt.ra(j+1)) j = j + 1
          endif
          if(rra.lt.ra(j)) then
          ra(i) = ra(j)
          i = j
          j = j + j
      else
          j = ir + 1
      endif
      go to 20
      endif
      ra(i) = rra
      go to 10
      end
c**********************************************************************

      SUBROUTINE ILIOS(ORD,PAN,VAR,N,L,ANS)
c
c      This is a FUDGED version, with an error return appropriate
c      to use of the s/r with QUALCON (ANS = 0.)
c
c       lagrangian polynomial interpolation
c       N data points; L th order polynomial
c       ord = y; pan = x; var = xbar; ans = y(xbar)
c       pan must be in increasing order
c       problems if the dynamic range of ord is too great (take logs)
c       or if two values of pan are too close together
c
      DIMENSION ORD(N),PAN(N),ARRAY(8,8),ROW(8),R(8),P(8)
      I=1       
 4801 IF(PAN(I)-VAR)   5801,4800,4800
 5801 I=I+1     
      IF(I-N)   4801,4801,4806
 4800 IF(PAN(I)-VAR)   4444,5800,4802
 4444 STOP 4444 
 5800 ANS=ORD(I)
      RETURN    
 4802 I=I-L/2   
      IF(I-1)   4803,4803,4804
 4803 I=1       
      GO TO 4805
 4804 IF(I-N+L-1)   4805,4806,4806
 4806 I=N-L+1   
C     CORRECT STARTING POSITION IN ALL CASES
 4805 II=I+L-1  
      K=1       
      DO 4807 J=I,II
      R(K)=ORD(J)
      P(K)=PAN(J)
ccc     type*,' ILIOS  j,k,r(k),p(k) ',  j,k,r(k),p(k) 
 4807 K=K+1     
 4818 DO 4808 I=1,L
      DO 4808 J=1,L
 4808 ARRAY(I,J)=P(I)-P(J)
      DO 4809 J=1,L
 4809 ARRAY(J,J)=VAR-P(J)
C     ALL OF ARRAY SET UP
      DIAG=1.0  
      ANS=0.0   
      DO 4810 I=1,L
      ROW(I)=1.0
      DO 4811 J=1,L
 4811 ROW(I)=ROW(I)*ARRAY(I,J)
c
c       Error return if two points are equal
      IF(row(i).eq.0.) THEN
      ans = 0.
      RETURN
      ENDIF
c
      DIAG=DIAG*ARRAY(I,I)
      ROW(I)=R(I)/ROW(I)
 4810 ANS=ANS+ROW(I)
      ANS=ANS*DIAG
c
      RETURN    
      END             

c**********************************************************************

      subroutine jumpinht (freq,ht,nht,njump,jump,htjump)
c
c      s/r to look for a sudden jump in the scaled F2 trace, as when
c
c      1. The scaled trace switches from the leading to the trailing edge
c      2. The scaled trace jumps from nEs to F2
c
c      written by leo mcnamara 11-mar-94
c-----      last modified 14-mar-94
c
c      A jump is defined as a sudden doubling or greater of the slope,
c      provided the height step is greater than 4*5 km (the resolution)
c      The check stops at fof2 - 0.5 MHz (where the slope can increase
c      very rapidly)
c
c      FREQ    freqs of scaled trace
c      HT        virtual heights
c      NHT        number of points in trace        
c      NJUMP    the number of jumps detected
c      JUMP    the location of the freq at which jump starts
c      HTJUMP    the absolute height step h'(i+1) - h'(i)
c
      dimension freq(nht),ht(nht),jump(*),htjump(*)
c
      fof2 = freq(nht)
      nhtm1 = nht - 1
      jj = 0
      del = abs(ht(2) - ht(1))
      slope1 = (ht(2) - ht(1)) / (freq(2) - freq(1))
c
      DO i = 2,nhtm1
       IF (ht(i).GT.0.) THEN    !!    in case h'=0 at start of trace
        if(slope1.lt.10.) slope1 = 10.    !!    may be a flat trace
        del = abs(ht(i+1) - ht(i))
        slope2 = (ht(i+1) - ht(i)) / (freq(i+1) - freq(i))
        ratio = abs(slope2 / slope1)
        diff = fof2 - freq(i+1)
        IF (ratio.gt.2.0.and.del.gt.20.0.and.diff.gt.0.5) THEN
         jj = jj + 1
         jump(jj) = i    !!    beginning of the jump
         htjump(jj) = ht(i+1) - ht(i)
        ENDIF
        slope1 = slope2
       ENDIF
      ENDDO
c
      njump = jj
c
      return
      end

c**********************************************************************

      subroutine makeGAIMfile
     a(version,igaim,IwantProfiles,devf_pol,devh_pol,chits,numprob,
     a statID,statlat,statlon,cyear,cdayno,chour,cmin,csec,iaafcon,
     a idfi,scaled,freq_e,freq_f1,freq_f2,the,thval,thf1,thf2,
     a fof2_pol,hmf2_pol,sh_pol,fnpol,realpol,freq_int,slope_end,
     a fn_art,fn_pol,fn_gmf,fn_error_bar,polan_fn_errorbars,
     a IwantGAIMalts)

     
c----- s/r to set up the ARTIST and POLAN profiles for every integer km 
c      from the first GMF altitude up to one scale height above hmF2_ART
c      and hmF2_POL, as well as the profiles and error bars for every 
c      GMF altitude up to the lower value of hmF2 + shF2.

c----- leo mcnamara 16-january-2006

c----- The profile used is the average of the ARTIST and POLAN profiles
c----- The error bars come from the difference between the ARTIST and POLAN 
c      profiles.
c----- The errors are forced to be at least as large as the POLAN error
c      bars, polan_fn_errorbars
c----- The profile is the average of the ARTIST & POLAN profiles
c----- The peak parms are the POLAN values with its sigs as error bars

c      version        QualScan version
c      igaim        LU for GAIM EDP2 profile
c      IwantProfiles    1 or 2 for POLAN 1 or 2 (>90 means there is no POLAN profile)
c      devf_pol    standard dev of POLAN peak fit
c      devh_pol    standard dev of POLAN peak fit
c      chits        the char IDs for the problems (or reason why no profile)
c      numprob        number of problems currently used
c      statID        5-char URSI ID
c      statlat        latitude of ionosonde
c      statlon        east longitude of ionosonde
c      cyear,cdayno,chour,cmin,csec    epoch
c      iaafcon        ARTIST confidence figure
c      idfi        ARTIST counters
c      scaled        ARTIST scaled parms
c      freq_e        ARTIST E-layer freqs
c      freq_f1        ARTIST E-layer freqs
c      freq_f2        ARTIST E-layer freqs
c      the            ARTIST E layer true heights
c      thval        ARTIST valley parms
c      thf1        ARTIST F1 layer true heights
c      thf2        ARTIST F2 layer true heights
c      fof2_pol    POLAN foF2 (either POLAN1 or POLAN2)
c      hmf2_pol    POLAN foF2 (either POLAN1 or POLAN2)
c      sh_pol        POLAN shF2 (either POLAN1 or POLAN2)
c      fnpol        POLAN plasma freqs (profile)
c      realpol        POLAN altitudes (profile)
c      freq_int    Digisonde sounding freq interval
c      slope_end    slope at end of scaled ARTIST trace (s/r slope_at_end)
c      fn_art        ARTIST 1 km profile
c      fn_pol        POLAN 1 km profile
c      fn_gmf        Average 1 km profile fn profile
c      fn_error_bar    error bars for 1 km profile
c      polan_fn_errorbars    The max spread in the 4 POLAN profiles 
c      IwantGAIMalts = 1 to use only GAIM altitudes; = 0 to save profiles at every integer km

      integer*2 idfi
      character*17 version  !06Octo9
      character*4 cyear
      character*3 cdayno
      character*2 chour,cmin,csec,cart,cpol
      character*5 statid
      character*1 chits
c
c----- Dummy variable arrays    
      dimension idfi(*),freq_e(*),freq_f1(*),freq_f2(*),
     a      the(*),thval(*),thf1(*),thf2(*),scaled(*),chits(*)
      dimension fnpol(*),realpol(*),polan_fn_errorbars(*)
      dimension fn_ART(*),fn_POL(*),fn_GMF(*),fn_error_bar(*)
c
c----- Local arrays      
      dimension coeffs(20),temp_ht(4),temp_fn(4)
      dimension fn(900),realht(900)
      dimension alt(83),ialtgaim(1380)
      dimension fnj(2),htj(2)

!! GMF altitudes
      data alt / 92, 96, 100, 104, 108, 112, 116, 120, 124,
     a  128, 132, 136, 140, 144, 148, 152, 156, 160, 164, 168, 172,
     a  176, 180, 200, 220, 240, 260, 280, 300, 320, 340, 360, 380,
     a  400, 420, 440, 460, 480, 500, 520, 540, 560, 580, 600, 620,
     a  640, 660, 680, 700, 720, 740, 760, 780, 800, 820, 840, 860,
     a  880, 900, 920, 940, 960, 980, 1000, 1020, 1040, 1060, 1080,
     a  1100, 1120, 1140, 1160, 1180, 1200, 1220, 1240, 1260, 1280,
     a  1300, 1320, 1340, 1360, 1380 /

      data ksetupgaim / 1 /
      data ialtgaim / 1380*0 /
      save alt,ksetupgaim,ialtgaim
c
c---- Initialize local arrays.   TWB
      do i=1,20
         coeffs(i)=0
      enddo
      do i=1,4
         temp_ht(i)=0.0
         temp_fn(i)=0.0
      enddo
      do i=1,900
         fn(i)=0.0
         realht(i)=0.0
      enddo
      do i=1,2
         fnj(i)=0.0
         htj(i)=0.0
      enddo 
      numalt = 83

c----- Write GAIM file headers only
      if(IwantProfiles.ge.90) THEN
         write(igaim,200)
         print*," No profile for GAIM file ",cdayno,chour,cmin
         print*, IwantProfiles, numprob,(chits(k),k=1,numprob)
cdb        pause
         badval = -99.
         write(igaim,201) badval,badval,badval,badval,badval,
     a statlat,statlon,iqual_fig,
     a statID,cyear,cdayno,chour,cmin,csec,
     a (chits(k),k=1,numprob),version      !! not in standard EDP2 file
         return
      ENDIF

c----- Set each km from alt(1) to alt(numalt) as being a GAIM altitude
      IF(ksetupgaim.eq.1) THEN
         do i = 1,numalt
            ialt = alt(i) + 0.5
            ialtgaim(ialt) = 1
         ENDDO
         ksetupgaim = 0
      ENDIF
        
      igaimht1 = alt(1) + 0.5    !! first GAIM altitude
      foe_art  = scaled(9)
      fof2_ART = scaled(1)
      shf2_art = scaled(40)
      shf2_pol = sh_pol
      IDFI17 = IDFI(17)

c---- Calculate the quality figure.
c      The quality figure is also calculated after all of the ionograms
c      have been processed, but we want it NOW.
      call assign_quality_figure (chits,numprob,iaafcon,iqual_fig)
cdb      print*,"CHITS",numprob,(chits(i),i=1,numprob)      
c----- Overwrite blank CHIT to indicate presence of no problem
      DO i = 1,numprob
         if(chits(i).eq." ") chits(i) = "."
      ENDDO

c----- Write out header
      write(igaim,200)
200   format(4x,"hmf2",4x,"fof2",4x,"conf",9x,"nmf2",9x,"conf",
     a 5x,"lat",5x,"lon")

c----- The peak parms and error bars etc Use POLAN profile peak
      ed_pol = (fof2_pol/0.00898)**2 * 1.e6
      conk = 0.00898**2
      zfn = devf_pol
      deln = (2.*fof2_pol*zfn + zfn**2)/conk   !! fn**2 = kN; kDn = 2 f Df + Df**2
      ed_err = deln * 1.e6

      write(igaim,201) hmf2_pol,fof2_pol,devf_pol,ed_pol,ed_err,
     a statlat,statlon,iqual_fig,
     a statID,cyear,cdayno,chour,cmin,csec,
     a (chits(k),k=1,numprob),version      !! not in standard EDP2 file
201   format(3f8.2,2e13.4,2f8.2," AQI=",i1,1x,a5,"_",a4,a3,3a2,
     a " Problem Flags",18a1,1x,a17)       !06Oct09

c----- Write out second header
      write(igaim,202)
202   format(2x,"height",4x,"freq",2x,"f_conf",6x,"density",
     a 1x,"density_conf","  ARTIST","   POLAN", "     ART","   POL",
     a  "   ART","   POL","   ART","   POL"," Slope", " POLAN 1/2")

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- ARTIST profile

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c      print*," ART 3 heights",the(3),thf1(3),thf2(3)
c      print*,"idfi(17)",idfi(17),foe_ART

c-----The ARTIST E profile
      kk = 0
      if(idfi(17).le.0) go to 198     !! no E-layer points
      DO i = 1,idfi(17)
c      print*," E freqqs",i,freq_e(i)
      ENDDO
c      print*," ART E Peak",scaled(9),the(3)
      f_start = the(1)
      f_end = the(2)
      if(f_start.eq.f_end) go to 198     !! only 1 E-layer point
      hme = the(3)
      DO i = 1,5
         coeffs(i) = the(i+4)
      ENDDO
      do 196 i = 1,9999
         plasmafreq = freq_e(1) + (i - 1) * freq_int
         if(plasmafreq.gt.f_end) go to 197
         kk = kk + 1
         gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
         realht(kk) = z_cheb (coeffs,gg,hme)
         fn(kk) = plasmafreq
c        print*," ART PROF E",kk,realht(kk),fn(kk)
196   continue
197   continue
c----- Find plasma freqs at each km
      ihtE = hme + 0.1     !! last integer km below hmE; 0.1 in case hme is "exact" km
c      print*," IHTE",ihte
      intord = 4
      if(kk.lt.4) intord = 2
      DO i = igaimht1,ihtE
      hite = i
      call ilios(fn,realht,hite,kk,intord,ffn)    !! polynomial interpolation
      fn_art(i) = ffn    !! 1 km profile    !! accidentally commented out - fixed 27-march-2007
c      print*," Interp ART E",hite,ffn
      ENDDO
      kkE = kk

c----- Standard valley,  thval(1) wide,  thval(2) deep
      kkV = 0
      IF(idfi(42).ne.0) THEN    !! we have an E-F1 valley
c      Take a rectangular valley to start with    
      ihtv1 = ihte + 1    !! first integer km above hmE
c      print*," ART VALLEY",thval(1),thval(2),hme
c      Top of valley
      valtop = hme + thval(1)
      ivaltop = valtop    !! last integer km in valley
      DO i = ihtv1,ivaltop
      fn_art(i) = foE_ART - thval(2)
c      print*," ART PROF Vall",i,fn_art(i)
      ENDDO
      kkV = ivaltop - ihtv1 + 1    !! # of points in valley
      ENDIF    !! idfi(42).ne.0

c----- Titheridge's standard valley uses a topside E layer with scale height
c      equal to 1.4 lower E scale height
c      We assume the E-layer parabola is symmetric, and replace valley freqs
c      by parabolic values if these are higher than the valley fn
      ivaltop2 = ihtv1 + (ivaltop - ihtv1) / 2
c      print*,ihtv1,ivaltop2
      DO i = ihtv1,ivaltop2
      hite = i    
      parabval = foe_ART * (1. - ((hite - hme)/scaled(16))**2)
c      if(parabval.gt.fn_art(i)) print*," PARAB E",fn_art(i),parabval
      if(parabval.gt.fn_art(i)) fn_art(i) = parabval
      ENDDO

c      print*,ihte,ihtv1,ivaltop
c      pause

198   continue    !! bypass for no valid E data

c-----The ARTIST F1 profile

      kk = 0
      if(idfi(12).le.1) go to 298    !! no F1-layer points    !! note the 1 (bitter experience)
      DO i = 1,idfi(12)
c      print*," F1 freqqs",i,freq_f1(i),f_start,f_end
      ENDDO
      f_start = thf1(1)
      f_end = thf1(2)
      if(f_start.eq.f_end) go to 298    !! only 1 F1-layer point
      hmf1= thf1(3)
c      print*," HMF111",idfi(12),hmf1
      DO i = 1,5
      coeffs(i) = thf1(i+4)
      ENDDO
      do 296 i = 1,9999
      plasmafreq = freq_f1(1) + (i - 1) * freq_int
      if(plasmafreq.gt.f_end) go to 297
      kk = kk + 1
      gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
      realht(kk) = z_cheb (coeffs,gg,hmf1)
      fn(kk) = plasmafreq
c      print*," ART PROF F1",kk,realht(kk),fn(kk)
296   continue
297   continue
c----- Find plasma freqs at each km
      ihtF1= hmf1    !! last integer km below hmf1
      ihf1_1 = realht(1) + 0.5    !! first integer km in F1 layer
      intord = 4
c      print*,"ihf1_1,ihtF1 ",ihf1_1,ihtF1
      if(kk.lt.4) intord = 2
      DO i = ihf1_1,ihtF1
      hite = i
      call ilios(fn,realht,hite,kk,intord,ffn)
      fn_art(i) = ffn    !! 1 km profile
c      print*," ART F1 points",i,ffn

      ENDDO
298   kkF1 = kk    !! bypass for no valid F1 data

c-----The ARTIST F2 profile
      kk = 0
      if(idfi(7).le.0) go to 398    !! no F2-layer points
      DO i = 1,idfi(7)
c      print*," F2 freqqs",i,freq_f2(i)
      ENDDO
      f_start = thf2(1)
      f_end = thf2(2)
      if(f_start.eq.f_end) go to 398    !! only 1 F2-layer point
      hmf2 = thf2(3)
      hmf2_ART = thf2(3)
      DO i = 1,5
      coeffs(i) = thf2(i+4)
      ENDDO
      do 396 i = 1,9999
      plasmafreq = freq_f2(1) + (i - 1) * freq_int
      if(plasmafreq.gt.f_end) go to 397
      kk = kk + 1
      gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
      realht(kk) = z_cheb (coeffs,gg,hmf2)
      fn(kk) = plasmafreq
c      print*," ART PROF F2",kk,realht(kk),fn(kk)
396   continue
397   continue
c----- Find plasma freqs at each km
      ihtF2= hmf2    !! last integer km below hmf2
      ihf2_1 = realht(1) + 0.5    !! first integer km in F2 layer
      DO i = ihf2_1,ihtF2
      hite = i
      call ilios(fn,realht,hite,kk,4,ffn)
      fn_art(i) = ffn    !! 1 km profile
c      print*," ART F2 points",i,ffn
      ENDDO
      kkF2 = kk

c----- ARTIST topside profile up to one scale height
      itop1 = ihtF2 + 1    !! first integer km in topside
      itop2 = hmf2 + shf2_art    !! last integer km in topside
      hmf2_ART = hmf2
      DO i = itop1,itop2
      hite = i
      call chapman_peak (fof2_ART,hmf2_ART,shf2_ART,hite,fn_chap)
      fn_art(i) = fn_chap
c      print*," ART TOP ",hite,fn_chap
      ENDDO

      DO i = igaimht1,itop2
c      if(fn_art(i).ne.0.) print*," ARTIST proff",i,fn_art(i),cdayno
      ENDDO

398   continue    !! bypass for no valid F2 data

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- POLAN profile

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- POLAN profile is (realpol,fnpol)

c      DO i = 1,200
c         if(fnpol(i).gt.0) print*," POL PRO given",i,realpol(i),fnpol(i)
c      ENDDO
      ihtvalpol2 = 0    !! in case no E

      if(idfi17.gt.0.) THEN    !! check for E layer (check on foe_ART is NBG)

      call retrieve_pol_EF_parms     !! retrieve the valley shape (UAG-93, Figure 6)
     a (realpol,fnpol,foe_pol,hmE_pol,fnb_POL,htb_POL,fnc_POL,htc_POL,
     a fnd_POL,htd_POL)

c       print*,foe_pol,hmE_pol,fnb_POL,htb_POL,fnc_POL,htc_POL,
c     a fnd_POL,htd_POL

c-----The POLAN E profile
      kk = 0
      do i = 3,999
         if(realpol(i).gt.hme_pol) go to 111
         kk = kk + 1
         realht(kk) = realpol(i)
         fn(kk) = fnpol(i)
c      print*, " POLAN PROF E",kk,realht(kk),fn(kk)
      ENDDO
111   continue

c----- Find plasma freqs at each km
      jhtE = hme    !! last integer km below hmE
      khte = realpol(3)    !! first point in E profile
      igg = igaimht1
      if(igg.lt.khte) igg = khte
      DO i = igg,jhtE
         hite = i
         call ilios(fn,realht,hite,kk,4,ffn)
         fn_pol(i) = ffn    !! 1 km profile
c      print*," POLAN E profile",i,ffn
      ENDDO
      kkE_pol = kk

c-----The POLAN valley - assume linear from A to B and C to D. Constant B to C
      ihtvalpol1 = hmE_POL
      ihtvalpol1 = ihtvalpol1 + 1    !! first integer km in valley
      ihtvalpol2 = htd_POL
      ihtvalpol2 = ihtvalpol2 + 1    !! last integer km in valley
      temp_ht(1) = hmE_POL
      temp_ht(2) = htb_POL
      temp_ht(3) = htc_POL
      temp_ht(4) = htd_POL
      temp_fn(1) = foE_POL
      temp_fn(2) = fnb_POL
      temp_fn(3) = fnc_POL
      temp_fn(4) = fnd_POL
      kk = 0
      DO i = ihtvalpol1,ihtvalpol2
      kk = kk + 1
      var = i
      call ilios(temp_fn,temp_ht,var,4,2,ans)    !! linear interpolation
      fn_pol(i) = ans
c      print*," POLAN Valley PROF",i,ans
      ENDDO
      kkV_pol = kk

      ENDIF    !! idfi17.gt.0.

c      DO i = 1,100
c      print*," ABCWXYZ",fnpol(i),realpol(i)
c      ENDDO
c      pause
c      
c-----The POLAN F layer - qualscan joins the F1 and F2 points into a single trace
      iflayer = ihtvalpol2 + 1    !! first integer km above valley
      htdpol = iflayer
      IF (idfi17.le.0.) THEN    !! no E layer
         htdpol = realpol(3)    !! starting point of F1/F2; Third point is fminF
         iflayer = htdpol    !! no E layer
      ENDIF
      jflayer = hmf2_POL
      kk = 0
c
c----- MODIFIED 30-july-2009. Loop now start at 3 (not 1), to avoid bad
c      values of altitude at second point (effective electron distribution)
      DO i = 3,700  !! 700 = DIM of realpol
         hite = realpol(i)
         if(hite.le.0.or.hite.gt.hmf2_POL) go to 1132    !!  (to break out of loop)
         IF (hite.gt.htdpol.and.hite.le.hmf2_POL) THEN
            kk = kk + 1
            realht(kk) = realpol(i)
            fn(kk) = fnpol(i)
c      print*," POLAN PROF F1-F2",i,kk,realpol(i),fnpol(i)!!,kk,realht(kk),fn(kk)
         ENDIF
      ENDDO
1132  continue
c
c----- Modified 30-july-2009  - code removed
c----- For daytime ionograms, there could be a gap in the trace between foE
c      and fminF. Once upon a time, this gap was filled by a linear interpolation
c      between the two points. As of 30-july-2009, such gaps are ignored, rather
c      than make something up. The plasma frequencies for the intervening altitude
c      range will be left at the default values of -99.
c----- Find plasma freqs at each km [F layer]
      kflayer = realht(1)
      iiflayer = kflayer
      DO i = iiflayer,jflayer
         hite = i
         call ilios(fn,realht,hite,kk,4,ffn)
         fn_pol(i) = ffn      !! 1 km profile
c     print*," POLAN F layer",i,fn_pol(i)
      ENDDO
      kkF_pol = kk
c----- POLAN topside profile up to one scale height
      polsh = abs(shF2_POL)      !! may be flagged negative as uncertain
      jtop1 = jflayer + 1
      jtop2 = hmF2_POL + polsh
      DO i = jtop1,jtop2
         hite = i
         call chapman_peak (fof2_POL,hmf2_POL,polsh,hite,fn_chap)
         fn_pol(i) = fn_chap
      ENDDO
cdb      DO i = igaimht1,jtop2
cdb         if(fn_pol(i).ne.0.) print*," POLAN proff",i,fn_pol(i)
cdb      ENDDO
cdb      pause "XXXXX"
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- Fill in any gaps in ARTIST heights

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- If there is a separate F1 trace, fill in gap between ivaltop & ihf1_1
      if(idfi(12).ne.0.and.idfi17.gt.0.) THEN
      ihv1 = ivaltop + 1
c      ihv2 = ihf2_1 - 1
      ihv2 = ihf1_1 - 1
c      print*,ihv1,ihv2
      DO i = ihv1,ihv2
      hite = i
      temp_fn(1) = fn_art(ivaltop)
      temp_fn(2) = fn_art(ihf1_1)
      temp_ht(1) = ivaltop
      temp_ht(2) = ihf1_1
      call ilios (temp_fn,temp_ht,hite,2,2,ans)
      fn_art(i) = ans
c      print*," FILLER E-F1", i,ans
      ENDDO
      ENDIF

c----- If there is no separate F1 trace, fill in gap between ivaltop & ihf2_1
      if(idfi(12).le.0.and.idfi17.gt.0.) THEN
      ihv1 = ivaltop + 1
      ihv2 = ihf2_1 - 1
c      print*,ihv1,ihv2
      DO i = ihv1,ihv2
      hite = i
      temp_fn(1) = fn_art(ivaltop)
      temp_fn(2) = fn_art(ihf2_1)
      temp_ht(1) = ivaltop
      temp_ht(2) = ihf2_1
      call ilios (temp_fn,temp_ht,hite,2,2,ans)
      fn_art(i) = ans
c      print*," FILLER E-F2", i,ans
      ENDDO
      ENDIF

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- Set any stupid POLAN values of fn to zero
      DO i = igaimht1,jtop2
      if(fn_pol(i).gt.20.) fn_pol(i) = 0.
      if(fn_pol(i).lt. 0.) fn_pol(i) = 0.
      ENDDO

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c----- Calculate the error bars & write file

c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


c      print*,ihf1_1,ihtf1
c      pause

      iFirstWrite = 1    !! to write foF2 & hmF2 on first line of profile
      DO i = 1,600
      if(fn_art(i).gt.0.or.fn_pol(i).gt.0.) THEN
      fnerrbar = abs(fn_art(i)-fn_pol(i))/2.
      fnn = (fn_art(i)+fn_pol(i))/2.
      fnn_ART = fn_ART(i)
      fnn_POL = fn_POL(i)
      ed_ART = (fnn_ART/0.00898)**2 * 1.e6
      ed_POL = (fnn_POL/0.00898)**2 * 1.e6
      ed = (ed_ART + ed_POL)/2.
      ed_err = abs(ed_ART - ed_POL)/2.
      fnerrbar = abs(fnerrbar)
      fnn_min = polan_fn_errorbars(i) / 2.    !! minimum error based on 4 POLAN profiles

      IF (fnerrbar.lt.fnn_min) THEN
c      print*," THRESHH", i,fnn,fnerrbar,fnn_min
         IF (fnerrbar.ne.0.) THEN
            ed_err = ed_err * (fnn_min/fnerrbar)**2    !! increase ed error in proportion
         ENDIF
         fnerrbar = fnn_min
      ENDIF
      IF (fn_art(i).eq.0.or.fn_pol(i).eq.0.) THEN
         fnn = -99.
         fnerrbar = -99.
         ed = -99.
         ed_err = -99.
      ENDIF

      if(ialtgaim(i).eq.1.or.IwantGAIMalts.eq.0) THEN    !! use GAIM grid only / 1km grid

c---- Flag the ARTIST and POLAN values as E, V, F, T (E region, valley, F region, Topside)
      cart = "c "
      cpol = "c "
      if(foe_ART.gt.0.) THEN
         if(i.ge.igaimht1.and.i.le.ihte) cart = "E "
         if(i.ge.ihtv1.and.i.le.ivaltop) cart = "V "
         if(i.ge.igaimht1.and.i.le.jhte) cpol = "E "
         if(i.ge.ihtvalpol1.and.i.le.ihtvalpol2) cpol = "V "
      ENDIF
      if(i.ge.ihf1_1.and.i.le.ihtf1) cart = "F1"
      if(i.ge.ihf2_1.and.i.le.ihtf2) cart = "F2"
      if(i.ge.itop1.and.i.le.itop2) cart = "T "
      if(i.ge.iflayer.and.i.le.jflayer) cpol = "F "
      if(i.ge.jtop1.and.i.le.jtop2) cpol = "T "
      if(iFirstWrite.eq.1) THEN    !! write foF2 & hmF2 on first line of profile
         write(igaim,60) float(i),fnn,fnerrbar,ed,ed_err,
     a    fnn_ART,cart,fnn_POL,cpol,foF2_art,foF2_pol,hmf2_art,hmf2_pol,
     a    shf2_art,shf2_pol,slope_end,IwantProfiles 
60    format(3f8.2,2e13.4,2(f6.2,a3),7f6.1,i9)
      iFirstWrite = 0
      go to 33
      ENDIF
      if(iFirstWrite.eq.0) THEN
         write(igaim,60) float(i),fnn,fnerrbar,ed,ed_err,
     a fnn_ART,cart,fnn_POL,cpol
      ENDIF
33    continue

      ENDIF    !! ialtgaim(i).eq.1.and.IwantGAIMalts.eq.1

      ENDIF    !! fn_art(i).gt.0.or.fn_pol(i).gt.0.
      ENDDO    !! 1,600

      return

      end
c======================================================================
      subroutine retrieve_pol_EF_parms 
     a (realpol,fnpol,foe_pol,hmE_pol,fnb_POL,htb_POL,fnc_POL,htc_POL,
     a fnd_POL,htd_POL)
      
c----- s/r to retrieve the POLAN valley shape from the POLAN profile
c      See Figure 6 of UAG-93 (page 29) for definition of points A, B, C & D
      
c----- leo mcnamara 16-january-2006

      dimension realpol(*),fnpol(*)
      
c----- Find first local maximum in fn (must be below 150 km)

      DO i = 1,100
c      print*," POL PRO",i,realpol(i),fnpol(i)
      ENDDO

      DO i = 4,100    !! ignore points 1 & 2 (model)
      if(fnpol(i).lt.fnpol(i-1)) THEN
      imax = i - 1
      go to 20
      ENDIF
      ENDDO
      print*," Problem #1 in S/R retrieve_pol_EF_parms"
c      pause
20    continue
      if(realpol(imax).gt.200.) THEN
      print*," Problem #2 in S/R retrieve_pol_EF_parms"
      print*,imax,realpol(imax)
c      pause
      ENDIF
      
c----- Point A is fnpol(imax),realpolt(imax)

      foe_POL = fnpol(imax)
      hme_POL = realpol(imax)
      fnb_POL = fnpol(imax+2)
      htb_POL = realpol(imax+2)
      fnc_POL = fnpol(imax+3)
      htc_POL = realpol(imax+3)
      fnd_POL = fnpol(imax+4)
      htd_POL = realpol(imax+4)    
      
      return
      END
c======================================================================
      subroutine chapman_peak (fof2,hmf2,shf2,alt,fn_chap)

c----- leo mcnamara 18-oct-2004

      enmax = (fof2 / 0.00898)  **2  !! per c.c.

      z = (alt - hmf2 ) / shf2
      fac = (1. - z - exp(-z) ) /2.
      chap = enmax * exp(fac)
      
      fn_chap = 0.00898 * sqrt(chap)    !! MHz

c      print*," CHAP s/r",fof2,hmf2,shf2,alt,fn_chap

      return
      END
c**********************************************************************

      subroutine median (a,n,amed)
c      s/r to derive the median value AMED of an array A of N values
c      written by leo mcnamara 2-feb-90
c-----        last modified 2-feb-90
c
c      uses the s/r HEAPSORT, courtesy of "Numerical Recipes"
c
      dimension a(n)
c
c      return if no data
      amed = 999.
      if(n.le.0) return
      amed = a(1)
      if(n.eq.1) return
c
c      sort the array A IN PLACE & OVERWRITING, using HEAPSORT
c
ccc      print*, (a(i),i=1,n)
      call heapsort (n,a)
ccc      print*, (a(i),i=1,n)
c
c      now find the median value
      mid1 = n/2 
      mid2 = mid1 + 1
      if(n.ne.2*mid1) mid1 = mid2
ccc      print*,' mid1,mid2 ',mid1,mid2
      amed = 0.5 * (a(mid1) + a(mid2))
ccc      print*,' Median ',amed
c
      return
      end
      
c**********************************************************************

      subroutine oblique_propagation 
     a      (freq,virtht,numpts,thf2,iob_unit,filename,em3_art,fof2_art)
      
c----- leo mcnamara 21-may-2004

c----- 29-april-2008 em3_art,fof2_art added to c/s

c-----      s/r to find the elevation angles for a fixed circuit 
c      centered on the ionosonde site

c----- INPUT
c      freq        ARTIST F2 freqs
c      virtht        SMOOTHED ARTIST F2 virtual heights (changed to smoothed 16-feb-06)
c      numpts        number of ARTIST F2 points
c      iob_unit    output unit for plot file for this ionogram
c      filename    in directory /elev_ang/

c-----      The oblique frequency fob and the equivalent vertical frequency fv
c      are related by fob = fv * k secPHI, where k is the empirical "k factor"
c      and PHI is the angle at the reflection point, assuming mirror
c      reflection at the virtual height corresponding to fv.

c      O = center of Earth
c      T = transmitter
c      P = point of reflection [mirror height = h'(fv)]

c      delta = elevation angle
c      theta = half-angle at center of Earth
c      phi = angle TPR 

      character*60 filename
c      integer*2 numpts

      dimension freq(*),virtht(*),thf2(*)
      dimension elev(300),freqob(300),group(300),tem(300),circ(20)
      dimension coeffs(10),realht(300),fv_real(300)
      dimension wieder_d(9),wieder_k(9)

c-----      The variation of k wrt range, as given by Wieder (actually
c      Newbern Smith's theoretical curve) - JGR 60(4) 395, 1955.
      data wieder_d / 0.,  500.,1000.,1500.,2000.,2500.,3000.,3500.,
     a      4000. /
      data wieder_k / 1.,1.004,1.020,1.042,1.066,1.090,1.115,1.146,
     a      1.185 /
      
      close(iob_unit)
      open(unit=iob_unit,file=filename)

      pi = atan(1.0) * 4.
      rz = 6371.2    !! radius of the Earth
      numcirc = 6    !! Lest anyone uses too long a circuit
      
c-----      Loop on circuit length
c      DO len = 2,6,2    !!1,numcirc
c      circlength = 500.* len
      DO len = 1,1    !!1,numcirc
      circlength = 1735.    !! London-Rome
      circ(len) = circlength
c      write(iob_unit,3476)

      fack = wieder_k(len+1)    !! obliquity factor
      theta = circlength/2./rz
      sintheta = sin(theta)
      costheta = cos(theta)
      
c-----Loop on equivalent vertical frequency (ARTIST freqs)

      DO ifreq = 1,numpts

      fv = freq(ifreq)
      ifv = 10. * fv + 0.5    !! Integer equivalent vertical frequency
      vht = virtht(ifreq)        !! Virtual height for this frequency

c----- Real height of reflection is the true height at fv
c----- Real heights using Cheby coeffs
      f_start = thf2(1)
      f_end = thf2(2)
      hmf2 = thf2(3)
      DO i = 1,5
      coeffs(i) = thf2(i+4)
      ENDDO
      gg = alog(fv/f_end) / alog(f_start/f_end)
      hr= z_cheb (coeffs,gg,hmf2)
      realht(ifreq) = hr

c      print*," OBLLLL",fv,f_start,f_end,hr,hmF2

c-----      Find the elevation angle, given theta and h'
      delta = 0.
      TP2 = rz**2 + (rz+vht)**2 - 2*rz*(rz+vht)*costheta
      TP = sqrt(TP2)    !! TP = half the mirror group path
      sinTPO = rz * sintheta/TP
      TPO = 0
      if(sinTPO.lt.1.) TPO = asin(sinTPO)
      delta = (pi - theta - TPO) - pi/2.
      delta_deg = delta * 180. / pi
      elev(ifreq) = delta_deg

c----      Oblique frequency
      cosphi = cos(TPO)
      secphi = 0.
      fob = 0.
      if(cosphi.ne.0.) THEN
      secphi = 1. / cosphi
      fob = fv * fack * secphi    !! OVERWRITTEN by Davies [1965] eqn 4.11
      ENDIF
      freqob(ifreq) = fob

c----- Davies [1965] eqn 4.11
      brack = rz * cos(delta) / (rz + realht(ifreq))
      bb = 1. - brack**2
      f411 = freq(ifreq)/sqrt(bb)

c      print*,"f_oblique",circlength,fob,f411
c      write(66,*) "f_oblique",circlength,fob,f411 

c----- modified 28-april-2008 (same as used in HFNowcast)
      fob = f411
      freqob(ifreq) = f411    !! Use Davies in place of fv k sec[phi]

c-----      Group path
      group(ifreq) = 2. * TP

      if(ifreq.eq.1) THEN
      write(iob_unit,3476) ifreq,fv,vht,delta_deg,fob,2*TP,
     a em3_art,fof2_art
3476      format(i5,7f8.2)
      ENDIF

      if(ifreq.gt.1) THEN
      write(iob_unit,3476) ifreq,fv,vht,delta_deg,fob,2*TP
      ENDIF

c======================================================================
c======================================================================

c      fv_real would agree with fv if the Weider curve applied exactly
c      to this ionogram. We just calculate fv_real, but do nothing with it.
c      fv_real depends on delta, which used fv, so the correct solution 
c      would be iterative. An alternative is to use a better Wieder curve.

c======================================================================
c======================================================================

      cosdelta = cos(delta)
      ff = rz * cosdelta / (rz + hr)
      if(ff.gt.1.) print*,"SQRT ERR", hr,cosdelta,ff
c      if(ff.gt.1.) pause
      ff = sqrt(1. - ff**2)
      fv_real(ifreq) = fob * ff

      ENDDO    !! ifreq

      DO i = 1,numpts
      tem(i) = freqob(i)    !! used for second sort
c      write(6,66) freq(i),virtht(i),elev(i),freqob(i),fv_real(i)
c      write(iob_unit,66) freq(i),virtht(i),elev(i),freqob(i)
66    format(10f10.2)
      ENDDO

c======================================================================
c======================================================================

c-----      Sort the data into increasing order of fob [not necessary for plots]
      call sort2 (numpts,freqob,elev)
      call sort2 (numpts,tem,group)
      DO i = 1,numpts
c      write(6,66) freqob(i),elev(i)
      if(i.eq.1) THEN
c      write(iob_unit,166) freqob(i),elev(i),group(i),filename
166      format(/3f10.2,1x,a60)
      ENDIF
c      if(i.gt.1) write(iob_unit,66) freqob(i),elev(i),group(i)
      ENDDO

      ENDDO    !! len
      
      END

c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++      
      SUBROUTINE SORT2(N,RA,RB)      !! Press et al page 241 - sorts both RA & RB
      DIMENSION RA(N),RB(N)
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          RRA=RA(L)
          RRB=RB(L)
        ELSE
          RRA=RA(IR)
          RRB=RB(IR)
          RA(IR)=RA(1)
          RB(IR)=RB(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            RA(1)=RRA
            RB(1)=RRB
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(RA(J).LT.RA(J+1))J=J+1
          ENDIF
          IF(RRA.LT.RA(J))THEN
            RA(I)=RA(J)
            RB(I)=RB(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        RA(I)=RRA
        RB(I)=RRB
      GO TO 10
      END      

c**********************************************************************

c                                              Nov'87/ Mar'88/ Feb'93/ (Nov'95)
c
c      POLAN.DOC  -  Description and use of the POLynomial ANalysis subroutine:
c
c         POLAN  (N, FV,HT, QQ, FB,DIP,   START, AMODE, VALLEY, LIST).
c
c   for the calculation of real-height profiles from sweep-frequency ionograms.
c
c        This outline is for Versions after September 1986.  It supplements the
c   information contained in the 194-page report "Ionogram analysis with the
c   generalised program POLAN",  obtainable as report UAG-93 from:-
c        NOAA/NGDC,  325 Broadway,   E/GC2, Dept. 884
c        Boulder,    Colorado 80303, USA.   Phone (303)497-6761.
c
c        If problems arise run one data set with  LIST = 3, and mail all output
c   to:  J.E. Titheridge,  Physics Dept.,  University of Auckland,  New Zealand.
c        or (better) use EMAIL, to:  J.TITHERIDGE@AUCKLAND.AC.NZ
c
c   NOTE: Profile summaries, and any debug information, are written to unit 2.
c       Changes made in recent versions of POLAN are summarised in section F. below.
c
c   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
c   A. >>>>>>>>>>>>>>>>>>>>>>>> DATA ARRAYS <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
c
c     Polan is called with frequency, height data in the arrays  FV, HT.
c           The dimension of these arrays (NDIM) must be greater than
c           30 + the number of data points in the arrays.
c       --> For the first call to POLAN, you must set  N = NDIM.
c     QQ is an output array, used primarily with single-polynomial calculations.
c           For 3 layers and 8-term polynomials, the dimension of QQ must exceed 50.
c           POLAN will not write over a value of -1. in QQ, so setting the last 
c           element of QQ equal to -1.0 will prevent any overflow.  
c           The data returned in QQ is described in D.2 below.
c       --> If this data is not required, use Dimension QQ(2) and set QQ(1)= -1.
c
c     In the data arrays, intermediate layers are terminated by a scaled (or 
c           zero) value for the critical frequency, with:-
c               h' = 0.0  for a Chapman peak and normal valley,
c               h' = 10.0 for a peak  with no following valley,
c               h' = negative and less than 50 is used to set the valley
c                        constants, for this valley only, as described below.
c               h' = negative (equal to minus the scaled virtual height)
c                        for a cusp-type discontinuity only.
c           Note that profiles are normally continuous across a cusp point,
c           so h' is scaled normally.  (Or preferably scale points either side
c           of a cusp, and not at the cusp itself;  see JATP 44,657,1982.)
c
c     The o-ray FC (scaled or zero) may be followed by an x-ray value (-FCX).
c    
c     The final layer is terminated by at least 2 null points, with  h = f = 0.
c     Data can be terminated without a peak by using a final frequency of -1.0.
c
c        Data for the extraordinary ray, if any, precedes the o-ray data for
c     each layer.  This is because x-ray data is used only to calculate the
c     (start or valley) corrections to be made at the beginning of the
c     calculation for that layer.  x-ray data are distinguished by using -f.
c
c        The format for input data is best seen by study of the examples in the 
c    test file POLRUN.DAT.
c   ----------------------------------------------------------------------------
c
c   B.  INPUT PARAMETERS,  in the call to POLAN.
c    
c      FB  gives the gyrofrequency at the ground in MHz, for an inverse cube 
c   variation.   If you have only the gyrofrequency  FH  at a height  h km,
c   the ground value is obtained from    FB = FH * (1. + h/6371.2)**3.
c      To use a gyrofrequency (FH, say) which is independent of height, 
c   set  FB = - FH.
c                          ------------------------
c
c      DIP  is the magnetic dip angle  IN  DEGREES.   Use of a negative value
c   for  DIP suppresses the physical checks which are normally applied to the
c   calculated profile,  so that the result obtained is the best mathematical
c   (but possibly non-physical) fit to the virtual-height data.  
c     [Some physically based equations are still included in start and valley 
c   calculations, unless AMODE is negative.]
c                          ------------------------
c
c      START normally gives a model height at 0.5 MHz.  Typical values are:    
c   noon   sunset-2/rise+2hr   set/rise    set+1hr   set+2   set+4 to rise-1    
c   85km    88km(E layer)    90(E)/80(F)   100 km    130 km     150 km. 
c    
c      A preferred procedure is to calculate model values of START from the 
c   equations (10) to (13) given in J. Atmosph. Terr. Phys. 48, 435-446, 1986.
c
c      Use of START = 0.0 makes some allowance for underlying ionisation based 
c   on a limited extrapolation of the first few virtual heights.
c
c      With initial x-ray data, START is taken to give the gyrofrequency height
c   for underlying ionisation calculations; the values listed above are still
c   suitable for this purpose.  The x-ray data is used to calculate a slab start
c   correction from 0.3*fmin  (adding points at 0.3, 0.6 and 0.8 *fmin). 
c
c   [Alternative procedures can be obtained using non-standard values of START:-
c      START between 0. and 44.  defines the plasma frequency for a model start.
c      Start = -1.0   uses a direct start, from the first scaled point.
c      Start < -1.0   for x-starts to use a polynomial from (-Start -1.0) MHz. ]
c                          ------------------------
c
c   THE final three parameters - AMODE, VALLEY and LIST, are zero for most work.
c
c      AMODE  sets the type of analysis, as listed below.   Zero uses mode 6.
c        Use Amode+10. for 12-point integrals, for high accuracy at large dip
c        angles (this is done automatically, at  DIP > 60, when Amode=0).
c      For denser (e.g. digital) data, with more than 30 points in one layer,
c        use a higher-order mode.  Thus AMODE = 9. gives maximum detail,  or
c        AMODE= 95. gives single-polynomials with 5, 9 terms for the E, F2 layers.
c
c     Values of Amode greater than 29.0 are used to specify the number of
c        polynomial constants to be used to describe each ionospheric layer.
c        e.g. 80.  uses an 8-term real height polynomial for each separate layer.
c             85.  uses 8 terms for the final layer and 5 terms for lower layers.
c             853. uses 8 terms for the last, 3 terms for the first, and 5 terms
c                  for any intermediate layer.
c
c        Setting AMODE negative causes physical relations to be omitted from the
c   start and valley calculations. 
c                          ------------------------
c
c      VALLEY= 0.0 or 1.0  uses a valley width equal to the initial default
c   value of twice the local scale height.  The initial default depth is 0.05
c   MHz.  The calculated depth is scaled according to (calculated width)**2. 
c
c        Alternative solutions may be obtained as follows:
c
c     VALLEY = 10.0  gives a monotonic (no valley) analysis.
c     Valley =  5.0  gives a maximum valley (upper reasonable limit) analysis.
c     Valley =  0.1 to 5.0  multiplies the standard valley width by this factor.
c     Valley = -.01 to -.99 uses  -2.0 * Valley  as the initial depth
c                                (instead of the default value of 0.05 MHz).
c     Valley = -1.0  iterates both valley depth and width for best fit, with 
c                 x-ray data.  (-1.D iterates from an initial depth of 0.D MHz).
c     Valley = -2.0 to -50. specifies a fixed valley width of 2*int(-Valley) km.
c                           Any decimal part D specifies a depth of 2*D in MHz.
c                          ------------------------
c
c     LIST = 0   prints results for the start, peak and valley regions only.
c            1   adds one line of output showing the frequency range and the
c                polynomial coefficients calculated at each step.
c            2, 3   add additional output.
c            4 to 9 show the data used at each step, and the calculated
c                   polynomial coefficients:
c               5   shows each set of simult equations, in the call to SOLVE;
c               6/7/8/9 give detail in the start/reduction/peak/valley steps.
c
c            LIST negative  suppresses most trace output below the first peak.
c            LIST= -10 suppresses all output, even the normal layer summaries.
c   ---------------------------------------------------------------------------- 
c
c   C.  OUTPUT PARAMETERS,  returned by POLAN.
c    
c     The arrays  FV, HT contain the calculated frequencies and real heights.
c
c     N  gives the number of calculated real-height data points.
c
c     The peak of the last layer is at  FC = fv(N-3),  Hmax = ht(N-3).
c     A point at (N-4) is added, on the fitted Chapman-layer peak; this and the
c            points above the peak permit accurate 2nd-difference interpolation.
c     Points at  N-2, N-1 and  N  in the output arrays are extrapolated heights
c            at  0.35, 0.85 and 1.5  scale heights above the peak (calculated from
c                the Chapman expression with a scale height gradient of 0.1).
c
c     fv(N+1)  gives the standard error of the last critical frequency, in MHz.
c     ht(N+1)  gives the standard error of the last peak height  Hmax,  in km.
c     fv(N+2)  gives the slab thickness, in km.   This is equal to the 
c                sub-peak electron content divided by the peak density.
c     ht(N+2)  gives the scale height SH of the last peak, in km.
c                A negative value of SH shows that a model value was used for
c                the scale height, to limit an unreasonable peak extrapolation.
c
c     QQ returns the real-height coefficients, for single-polynomial calculations,
c                as described under D.2 below.  For overlapping polynomial modes,
c                coefficients are returned for the last polynomial in each layer.
c   ---------------------------------------------------------------------------- 
c
c   D.  MODES OF ANALYSIS.
c
c   D.1 THE TEN STANDARD MODES
c
c       MODE is obtained from the input parameter AMODE, modified to the range 
c       1 to 10, and is used to select the type of analysis as summarised below.
c       All Modes include an estimated start correction,  a Chapman-layer peak,
c       and a model valley between layers.
c
c   MODE=1.- The Linear-Lamination analysis.
c        2.- A Parabolic-Lamination analysis, matching end gradients  ( = Paul).
c        3.- Overlapping Cubics, with no spurious oscillations (JATP 1982 p657).
c        4.- Fourth Order Overlapping Polynomials   (Radio Science 1967, p1169).
c        5.- Fifth Order Least-Squares fit to 6 points  (4 virtual + 2 real).
c        6.- Sixth Order Least-Squares fit to 8 points  (5 virtual + 3 real).
c        7.- Sixth Order fit to 7 virtual +3 real heights; calculates 2 new hts.
c        8.- Sixth Order fit to 8 virtual +4 real heights; calculates 2 new hts.
c        9.- Seventh Order fit to 13 virtual + 6 real hts; calculates 3 new hts.
c        10. A Single Polynomial,  fitting  2*sqrt(NV)  terms to  NV heights.
c            A maximum of 90 (=MAXB-9) points can be included in one polynomial.
c
c      The basic parameters which define the type of analysis depend on the
c   parameter MODE, and are obtained from the arrays given below.  
c      NT is the number of terms used in the polynomial representation of each 
c   real-height segment.
c      NV is the number of virtual heights which are fitted in this step.
c      NR is the number of previously-calculated real heights which are fitted
c   (in addition to the origin FA, HA).  A negative value of NR indicates that
c   one of the fitted real heights is below the origin.   If  NT = NV + NR  we
c   get an exact fit to the data, and if  NT < NV + NR  the calculated profile
c   segment is a least-squares fit. 
c
c      NH is the number of new real heights to be calculated.  
c      'First step' values are used at the beginning of an analysis, or when
c   starting on a new layer, when no real heights are known above the starting
c   point.  In this case the number of known real heights is zero, and the
c   tabulated values of NR define the position of the origin (counting backwards
c   from the last calculated real height) for the following step. 
c
c          |-------- First step --------|    |------- Following steps --------|
c   MODE=  1, 2  3  4  5  6  7  8   9  10    1  2  3  4   5   6   7   8   9  10 
c    NT =  1, 2, 3, 4, 4, 5, 6, 6,  7, 73,   1, 2, 3, 4,  5,  6,  6,  6,  7, 73
c    NV =  1, 2, 3, 4, 5, 7, 8,10, 12, 90,   1, 1, 2, 3,  4,  5,  7,  8, 13, 90
c    NR =  0, 0, 0, 1, 1, 2, 2, 3,  5,  2,   0,-1,-1, 1, -2, -3, -3, -4, -6, -3
c    NH =  1, 1, 2, 3, 3, 4, 5, 6,  8, 28,   1, 1, 1, 1,  1,  1,  2,  2,  3, 28
c
c
c   D.2 SINGLE-POLYNOMIAL MODES
c
c     These use a defined number of real-height coefficients for each layer, 
c   and return all profile parameters in the array QQ.  The order of the 
c   analysis is set by the parameter AMODE, as follows.
c
c   AMODE = 10L,  where L is an integer in the range 3 to 14, uses a single
c                 polynomial with L terms to describe each ionospheric layer.
c   AMODE = 10L+M   uses  L terms for the final layer, and M for earlier layers.
c   AMODE = 100L+10M+F is L terms for Last, M for Middle and  F for First layer
c                                                (M and F must be less than 10).
c
c      QQ returns the real-height parameters which describe the profile, for
c   single-polynomial modes of analysis (unless QQ(1) was set equal to -1.0 by
c   the calling program).  (For normal [overlapping polynomial] runs, QQ returns
c   the coefficients for the last polynomial, and the peak, in each layer.)
c
c      The returned value of QQ(1) gives the total number of stored values (numq).
c   Starting at QQ(2), the parameters returned for each layer are:
c        FA, HA,  nq,  q1, q2, .. qn,  devn,   FP, FC, Hmax, and SH.
c
c      nq is the number of polynomial coefficients (q1 to qn) used for this layer.
c   This is normally equal to the number of coefficients requested in AMODE.
c      HA is the true height at FA, after any start or valley adjustments, so the 
c   real-height profile is 
c                 h  =  HA + q1.(f-FA) + q2.(f-FA)^2 + ... qn.(f-FA)^nq.
c
c      devn is the rms deviation (in km) of the fit to the virtual height data.
c
c      FC, Hmax and SH are the constants which define the Chapman-layer peak;
c   this joins the polynomial section at the frequency FP (close to the second to
c   highest scaled frequency for the layer, but limited to 0.9FM < FP < 0.97FC).
c
c      For a 2nd (or 3rd) layer,  FA, HA give the new real-height origin at the 
c   top of the valley region.   Thus FA is equal to the previous FC,  and the
c   valley width is   W = HA - Hmax  in km.   The valley depth (D, in MHz) can be
c   obtained from the width using equations (14) of the report UAG-93, which give
c        D = 0.008 W**2/(20 + W) MHz,  followed by   D = D.FC/(D + FC).
c
c      The end point of the data in QQ is verified by a value  QQ(numq+1) = -99.
c   for a normal exit, and  -98. for an error (or no-peak) exit.
c   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
c
c   E.   PROCESSING - Outline of the REAL-HEIGHT ANALYSIS LOOP within POLAN.
c
c   E.1  THE OVERALL PROCEDURE, FOR ONE CYCLE OF THE CALCULATION
c
c        Analysis can proceed with any number of scaled virtual heights (even
c   1 height and no critical frequency) for each layer.  If the number of data
c   points NV is less than the number of polynomial terms NT (as specified by 
c   AMODE), NT is automatically decreased.
c
c   -    Calculate one polynomial, with NT terms, from the point  FA = fv(K),
c   HA = ht(K)  to fit the next NV virtual and NR real heights.  (The fitted 
c   real heights include one point below HA, if NR is negative.)   
c        The real-height origin (FA,HA) is at K = KR, in the data arrays FV, HT;
c   the corresponding virtual height is at K = KV. 
c
c   -    With x-ray data (-ve frequencies), at the start or after a peak,
c   recalculate HA to include the correction for underlying or valley ionisation. 
c
c   -    Calculate a further NH real heights, and set KR = KR + NH; KV = KV + NH.
c                                                                              
c   -    Repeat this loop, calculating successive overlapping real-height
c   sections, until a critical frequency (or end-of-layer) is found in the range
c   KV +1  to  KV +NV +1.   Then calculate real heights at the remaining scaled 
c   frequencies and determine a least-squares Chapman-layer peak. 
c                     - - - - - - - - - - - - - - - - - -
c
c   E.2  INDIVIDUAL STEPS WITHIN EACH CYCLE,
c        numbered according to the corresponding section in the program POLAN.
c
c   SECTION 2.2  Count initial x-rays.  Check frequency sequencing.
c                Check for cusp, peak, or end of data.
c    Set NF = number of o-rays 
c          (= NV, if sufficient points exist before a following peak);
c        NX = number of x-rays;      MV = NF+NX.
c        FM = fv(mf) = the top frequency used in this step.
c        FCC= FC or 0.1 for a peak,  = -.1 for a cusp (gradient discontinuity)
c                           at FM,   = 0.0 otherwise.
c
c   SECTION 2.3  Subtract the group retardation due to the last calculated
c                   real-height section.
c        This modifies all the virtual heights at f > FA  (where FA = fv(KR)),
c        and increases the index LK (which gives the point up to which the
c        group retardation has been removed) to KR.
c
c   SECTION 3.  Set up equations for the next profile step.
c
c             Check for the occurrence of a valley; if this is required, set
c        the valley flag HVAL and set initial values for the width and depth.
c
c             Set up equations in the matrix B.   For start calculations using 
c        x-ray data, or for any valley calculations, add suitably weighted
c        equations specifying desired physical properties of the solution.
c
c   SECTION 4.  Solve the set of simultaneous equations in the array B.
c
c             Check that the solution satisfies basic physical constraints.
c        If it does not, obtain a new least-squares solution with the limiting 
c        constraints imposed (in the subroutine ADJUST).
c
c             For an x-start or valley calculation, iterate the solution as
c        required to ensure the use of a correct gyrofrequency height, and 
c        the correct relation between depth and width of the valley.
c             For an o-ray valley, loop once to adjust the valley depth.
c
c   SECTION 5.  Calculate and store the real heights.
c
c             Set KRM as the index for the highest calculated real height.
c
c   SECTION 6.  Least-squares fitting of a Chapman layer peak.
c
c             Calculate the critical frequency and the scale height of a
c        layer peak, by an iterative fit to the real-height gradients at the 
c        last few calculated points  (as in Radio Science 20, 247, 1985).
c             Determine the height of the peak by fitting the peak shape to a 
c        weighted mean of the last few calculated real heights.  Adjust the
c        last real height to agree closely with the Chapman peak (Sept'86).
c        Add an interpolated point between the 'last' height and the peak(2'93).
c
c   SECTION 7.  Go to section 2, to restart for a new layer.
c
c        If there are no further data:-   add one point half-way to the peak;
c   extrapolate 3 points for the topside ionosphere (assuming a Chapman layer
c   with a scale height gradient of 0.1 km/km);  store constants relating to
c   the last layer peak;  and return.
c
c   ================================================================================
c   F.  POLAN changes made in recent versions.
c
c   F.2>>>>>>>>>>>>>>>>>>>>> CHANGES  February 1993 (marked 2'93): <<<<<<<<<<<<<<<<<
c
c   Deleted NDIM from call.  First call must have N = NDIM (or ndim is set to 100).
c   Added extra output point below peak, and spaced those above for best interpoln.
c   Reduced # points over which Chapman peak is fitted, for single polynomials.
c   Imposed lower limit on profile curvature at top point, before peak fit.
c   Mode 10 to NT= 2.*sqrt(NV), so 20/40/60 data -> Nt= 9/13/14 (prev NV>18->NT=15)
c
c   NOTE: I now use ! for comments; you may need to change this for your compiler.
c
c   F.1>>>>>>>>>>>>>>>>>>>>> CHANGES  September 1986 <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
c
c   (a)  Addition of the parameters  NDIM  and  QQ  in the call to POLAN.
c        Use of NDIM makes it unnecessary to reset N (to the dimension of the
c   input arrays) on each call.
c
c        QQ returns the coefficients for single-polynomial representations.  
c   It is now a required parameter in the call to POLAN,  but is not used if
c   (initially) QQ(1) = -1.   (Previous use of QQ returned 1 less coefficient 
c   than described in section D.2, since the count nq was taken to include
c   the constant HA).  For normal (overlapping polynomial) runs, QQ returns the
c   coefficients for the last polynomial, and the peak, in each layer.
c
c   (b)  Use of a negative scale height, to indicate use of a model value rather
c   than one derived from the data, is restricted to the output listing (and the
c   output array QQ).  In some previous versions, -SH was accidentally carried
c   over to later stages creating numerous problems. 
c
c   (c)  The default analysis (obtained at AMODE = 0.0) has been changed from
c   Mode 5 to Mode 6.  Experience has shown some benefits and no problems with
c   the higher modes, particularly since the change (d) below which gives good
c   results even when the scaled frequency interval varies considerably. 
c
c   (d)  Weighting of different points in the least-squares calculation has
c   been made proportional to the scaled frequency interval.  This stops smooth
c   sections of the profile, where fewer points may have been scaled, from
c   getting too low a weight.  It reduces spurious fluctuations in high order
c   modes to well below the levels described in J. Atmosph. Terr. Phys. 44,
c   657-669, 1982. 
c
c   (e)  The START model has been revised to the procedure described in J.
c   Atmosph. Terr. Phys. 48, 435-446, 1986. 
c
c   (f)  Minor improvements have been made in several steps of the calculation. 
c   Programs will now run at DIP = 0.  Calculations proceed normally with 2 or
c   more data points for each layer;  even a layer with only one point (with
c   or without FC) is handled.
c
c   (g)  Descriptive comments have been extracted from the listing of POLAN.FOR,
c   into this file POLAN.TXT  (or POLAN.DOC from Nov=95).
c
c**********************************************************************

      subroutine prohibited_frequencies 
     a (statid,iprohibit,prohibited,numprohibited)

c----- s/r to determine the prohibited frequencies for selected stations
c----- leo mcnamara 30-august-2005, 12-january-2006

      character*5 statid,ctem5
      dimension prohibited(*)
      
c----- We just read all lines of the file and pick the ones we want

      numprohibited = 0
      read(iprohibit,*) ctem5    !! skip over the header

10      continue
      read(iprohibit,*,end=20) ctem5,freqlow,frequp
      
      if(ctem5.eq.statID) THEN
      numprohibited = numprohibited + 1
      prohibited(numprohibited) = freqlow    !! we use just the lower edge of the band
      ENDIF
      
      go to 10
20    continue

c      DO i = 1,numprohibited
c      print*,i,"  ",statid,prohibited(i)
c      ENDDO
c      pause
      
      return
      
      END
      
c**********************************************************************

c      This s/r checks the scaled F2 h' trace for smoothness. 
c      If it is OK,we should be able to fit a LSF curve through the points
c      Work in terms of 1/h' to avoid infinite slope at foF2
c      Fit is weighted by the echo amplitudes.

c----- leo mcnamara 31-march-2005
c----- modifed 22-august-2005 to derive FOM_F2fit

c----- Modified 22-march-2009 to avoid crash with Ionogram for JI91J 2004 239 2100

c      qs_check11 returns once a non-zero value of ipoorfitF2 is set.

      subroutine qs_check11
     a (cyear4,cdayno,chour,cmin,
     a ftof,otf,ioaf,idfi7,idfi12,idfi17,nchebF2,eps,eta,rms_fac,
     a virtht_max,diffitmax,percslope_min,x,y,z,diffit,cheb,
     a too_high_slope,iwantusefuls,iusefuls,
     a iflagfile,f2_smoothed,FOM_F2fit,ier_cheby,ipoorfitF2,icheck11)
     
c      ftof                F2 frequencies
c      otf                    F2 virtual heights
c      ioaf                F2 echo amplitudes (used as weights)
c      idfi7    idfi(7)        # of F2 points
c      idfi12    idfi(12)    # of F1 points
c      idfi17    idfi(17)    # of E points
c      nchebF1                maximum order of fitted Cheby
c      eps                    relative tolerance loss of significance in Cheby fit
c      eta                    relative tolerance square sum of errors in Cheby fit
c      rms_fac                [defined in config file]
c      virtht_max            fit is checked up to this height [defined in config file]
c      diffitmax.            max allowable error in Cheby LSF for F2 [defined in config file]
c      percslope_min        LSF is straight line if slopes of 2 halves of LSF differ by < percslope_min
c                          also allows for a fit that does not have sufficient overall curvature
c      x                    temp array
c      y                    temp array
c      z                    temp array
c      diffit                temp array - abs diff between original and fitted
c      cheb                chebycheb coeffs for the fit
c      too_high_slope        max value for smoothed delta h' Usually very steep slope at end.
c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      f2_smoothed            smoothed F2 trace points
c      FOM_F2fit            figure of merit of F2 fit (ratio of zero amps to non-zero)
c      ier_cheby            error flag for cheby fit - should be 0
c      ipoorfitF2            0 for good fit
c                          1 if no fit possible
c                          2 if fit made but values negative
c                          3 there are points too far off the fitted curve
c                          4 if the fit collapsed to a straight line
c                          5 slope at end of fit is decreasing
c                          6 slope at very end is too high (for POLAN)
c      icheck11            0 if no errors, else 1 (details given by ipoorfitF2)

      dimension ftof(*),otf(*),x(*),y(*),z(*),f2_smoothed(*),cheb(*)
      dimension diffit(*)
      integer*2 ioaf(*)
      dimension zamp(500) 
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      
      ICHECK = 11      !! and PROBLEM(6)
      icheck11 = 0
      ipoorfitF2 = 0
      num_za = 0    !! number of freqs with zero amplitude
      num_nza = 0    !! number of freqs with non-zero amplitud
      FOM_F2fit = 0.

c----- Modified 22-march-2009 to avoid crash with Ionogram for JI91J 2004 239 2100
c----- A4 scaled the second hop. All h' > 1000
      if(otf(1).gt.600.) THEN
      icheck11 = 1    !! error
      ipoorfitF2 = 1
      RETURN
      ENDIF
c----- Modified 22-march-2009 to avoid crash with Ionogram for JI91J 2004 239 2100

c-----Set up arrays for LSF fit
      kk = 0
      DO i = 1,idfi7
       kk = kk + 1
       x(kk) = ftof(i) !!   frequency
       y(kk) = otf(i)  !!   virtual height
       y(kk) = 1. / y(kk)
       z(kk) = ioaf(i) !!   LSF weight = amplitude
       if(ioaf(i).eq.0) num_za = num_za + 1
       if(ioaf(i).ne.0) num_nza = num_nza + 1
       f2_smoothed(kk) = 0.    !! initialize in case no fit
      ENDDO

      FOM_F2fit = float(num_nza) / float(num_za + num_nza) * 100.

c----- ARTIST 5 puts amplitudes to 200 for prohibited freq bands
c      This is too high, so set them to zero first MOD 10-july-2007
      DO i = 2,kk
c      if(z(i).ge.200.) write(88,*) x(i),z(i)
      if(z(i).gt.111.) z(i) = 0.
      ENDDO

c-----Replace zero amplitudes by the previous non-zero value
c      Otherwise, the traces with no amplitudes near the end will be
c      rejected because of a poor fit.
      DO i = 2,kk
      if(z(i).eq.0.) z(i) = z(i-1)
      ENDDO

c-----Find the LSF curve through the points, provided kk > 3
      if(kk.le.3) THEN
      ier = 1
      ipoorfitF2 = 1
      icheck11 = 1    !! error
      return
      ENDIF

      ncheb = kk - 2
      if(ncheb.gt.nchebF2) ncheb = nchebF2  !!  number of terms in Chebyshev fit
      epsi = eps * 0.1
      etai = eta * 0.1
c     We relax the required accuracy of the fit as required.
      DO ierror = 1,4 
       epsi = epsi * 10.
       etai = etai * 10.
       call cheby_lsf (x,y,z,kk,ncheb,epsi,etai,ier,xd,xo,ires,cheb)
       if(ier.eq.0) go to 99   !! Fit has been successful
      ENDDO
      if(iwantusefuls.eq.1) write(iusefuls,7040)
7040  format(/5x,'No LSF Chebyshev fit possible for F2 h" trace')
      ipoorfitF2 = 1
      icheck11 = 1    !! error
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  x(i),f2_smoothed(i),rmsdiff,diffit(i),float(ipoorfitF2)
      return   !! this ionogram has a problem

c-----Evaluate the fit and compare with original values
99    continue
      rmsdiff = 0.
      krms = 0
      DO i = 1,kk
       yinv = 1. / y(i)
       xxx = xd * x(i) + xo
       fit = cnps (cheb,ires,xxx)
       fit = 1. / fit
       if(fit.lt.0.) THEN
       ipoorfitF2 = 2    !! physically impossible -- ERROR
       icheck11 = 1    !! error
       if(iwantusefuls.eq.1) write(iusefuls,7000) i,x(i),y(i),fit
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  x(i),f2_smoothed(i),rmsdiff,diffit(i),float(ipoorfitF2)
       return
       ENDIF    !! fit.lt.0.
       f2_smoothed(i) = fit    !! smoothed F2 trace
       if(yinv.le.virtht_max) THEN  !! limit check to virtht_max
       diffit(i) = abs(yinv - fit)
       rmsdiff = rmsdiff + diffit(i)**2
       krms = krms + 1
       ENDIF
       if(iwantusefuls.eq.1) write(iusefuls,7000) i,x(i),yinv,
     a      fit,ioaf(i)
7000   format(5x,'LSF Chebyshev to F2 h"',i5,3f10.3,i5)
      ENDDO

      rmsdiff = sqrt(rmsdiff / float(krms))
      if(iwantusefuls.eq.1) write(iusefuls,7005) x(1),x(kk),rmsdiff
7005  format(/5x,'RMS km error in LSF Chebyshev F2 fit from',f5.1,
     a ' to',f6.2,'  is',f6.1)

9002  format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)

c-----Look for points that lie more than rms_fac * rmsdiff from the
c      fitted smooth curve
      DO i = 1,kk-2    !! ignore bad fits to last 2 points 
      if(f2_smoothed(i).le.virtht_max) THEN  !! limit check to virtht_max
      if(diffit(i).gt.rms_fac*rmsdiff.or.diffit(i).gt.diffitmax) THEN  
           ipoorfitF2 = 3
       icheck11 = 1    !! error    
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  x(i),f2_smoothed(i),rmsdiff,diffit(i),float(ipoorfitF2)
      return
      ENDIF
      ENDIF    !! y(i).le.virtht_max
      ENDDO

c121   continue      !! kk.le.3 OR this ionogram has a problem

c---- Check that SMOOTHED F2 trace is not a straight line
      mid1 = 1
      mid2 = idfi7

      if(mod(idfi7,2).eq.1) THEN    !! odd
      mid3 = (idfi7 + 1)/2
      mid4 = (idfi7 + 1)/2
      ENDIF
      if(mod(idfi7,2).eq.0) THEN    !! even
      mid3 = idfi7/2
      mid4 = (idfi7 + 2)/2
      ENDIF
c      print*,idfi12,mid1,mid3,mid4,mid2

      diff1 = ftof(mid3) - ftof(mid1)
      diff2 = ftof(mid2) - ftof(mid4) 
      slop1 = (f2_smoothed(mid3) - f2_smoothed(mid1))/diff1
      slop2 = (f2_smoothed(mid2) - f2_smoothed(mid4))/diff2
      slop1a = abs(slop1)
      slop2a = abs(slop2)
      diffslope = abs(slop1a-slop2a)
      averslope = (slop1a+slop2a) / 2.
      percslope = 9999.
      if(averslope.ne.0.)    percslope = diffslope/averslope*100.
       
      IsStraightLine = 0
c-----Slopes normally equal to 5 or 6 figs whem LSF degenerates to a straight line
c      We allow for almost equal (might protect against short linear segments)
      if(percslope.lt.percslope_min.or.averslope.eq.0.) IsStraightLine=1

c----- See if the slope changes sign
      if(slop2.ne.0.) THEN
      if(slop1/slop2.lt.0.) IsStraightLine = 0
      ENDIF
      if(slop2.eq.0) IsStraightLine = 1    !! horizontal line

      if(IsStraightLine.eq.1) THEN
      ipoorfitF2 = 4
      icheck11 = 1    !! error
      if(IwantUsefuls.eq.1) THEN
      write(iusefuls,6788) slop1,slop2,percslope,percslope_min,averslope
6788  format(5x,"** F2 fit degenerated to a straight line",5f10.4)
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,slop1,
     a slop2,float(ipoorfitF2)
      return
      ENDIF
      ENDIF

c-----Check that slope of SMOOTHED trace increases at the end
      slop1 = f2_smoothed(idfi7)-f2_smoothed(idfi7-1)
      slop2 = f2_smoothed(idfi7-1)-f2_smoothed(idfi7-2)
      if(slop1.lt.slop2.or.slop1.le.0.or.slop2.le.0.) THEN
      ipoorfitF2 = 5
      icheck11 = 1    !! error
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,slop1,
     a slop2,float(ipoorfitF2)
      return
      ENDIF

c-----Check that slope of SMOOTHED trace at the end is not ridiculously high
c      The high slope may come legitimately from the scaled trace, but
c      POLAN crashes with a SQRT error if the slope is "too high"

      deltaf = ftof(idfi7) - ftof(idfi7 - 1)
      if(slop1.gt.too_high_slope * deltaf) THEN
      ipoorfitF2 = 6
      icheck11 = 1    !! error
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     aslop1,too_high_slope * deltaf,float(ipoorfitF2)
      return
      ENDIF
c-----
      ier_cheby = ier
c      if(ipoorfitF2.ne.0) icheck11 = 1    !! error
      if(ier_cheby.ne.0) icheck11 = 1    !! error

      return
      
      END

c**********************************************************************

c      This s/r checks for discontinuities in the F2 trace (sudden doubling
c      of the slope).

c----- leo mcnamara 5-april-2005

      subroutine qs_check12
     a (cyear4,cdayno,chour,cmin,ftof,otf,idfi7,
     a x,y,jumpht,htjump,iwantusefuls,iusefuls,iflagfile,
     a jumpfail,icheck12)

c-----Start at half-way point of F2 trace (greatest effect on F2 peak)
c     Last check (jumpfail = 4) is for complete F2 trace
c-----Note: Look at the code for JUMPINHT. The 20 km is currently hard-coded,
c     so KJ should always be greater than or equal to 1.
c      20 = 4*5 km (the resolution)

c      ftof                F2 frequencies
c      otf                    F2 virtual heights
c      idfi7    idfi(7)        # of F2 points
c      x                    temp array
c      y                    temp array
c      z                    temp array
c      jumpht                temp array
c      htjump                temp array
c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      jumpfail = 1        2 or more jumps > 20 km
c      jumpfail = 2        any 3 or more jumps
c      jumpfail = 3        jump changes sign
c      jumpfail = 4        one jump > 100 km (complete F2 trace)
c      icheck12            0 if no errors, else 1 (details given by jumpfail)

      dimension ftof(*),otf(*),x(*),y(*),jumpht(*),htjump(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      
      ICHECK = 12    !! and PROBLEM(7)                
      icheck12 = 0

      kj = 0      !! number of points to search along
      diffdelta = ftof(idfi7) - ftof(1)
      deltajump = 0.5 * diffdelta    !! Start at half-way point of F2 trace
      freqstartjump = ftof(1) + deltajump
      DO i = 1,idfi7
       IF (ftof(i).ge.freqstartjump) THEN
       kj = kj + 1
       x(kj) = ftof(i)    !! freq
       y(kj) = otf(i)        !! h'
       ENDIF
          ENDDO

c----- Look for a sudden jump in the scaled F2 trace
      call jumpinht (x,y,kj,njump,jumpht,htjump)

      IF (njump.gt.0) THEN      !! Flag the ionogram if too jumpy

      call set_jumpfail(idfi7,njump,htjump,y,jumpfail)

       IF (jumpfail.ne.0) THEN
        DO j = 1,njump
        write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  ftof(1),ftof(idfi7),x(jumpht(j)),htjump(j),float(jumpfail)
9002   format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)
        icheck12 = 1
            ENDDO
       ENDIF    ! jumpfail.ne.0

      ENDIF !!    end of njump.gt.0 loop
      
      RETURN
      
      END

c**********************************************************************

c      This s/r checks if M3000 value comes from one end of the F2 trace

c----- leo mcnamara 5-april-2005

      subroutine qs_check16
     a (cyear4,cdayno,chour,cmin,ftof,otf,idfi7,fof2_art,tan_freq_min,
     a x,iflagfile,tangent_freq,iflagpaul,icheck16)

c      ftof                F2 frequencies
c      otf                    F2 virtual heights
c      ioaf                F2 echo amplitudes (used as weights)
c      idfi7    idfi(7)        # of F2 points
c      x                    temp array
c      fof2_art            ARTIST scaled value of foF2
c      tan_freq_min        tangent freq must exceed tan_freq_min * foF2

c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      tangent_freq    basically the freq at which the MUF slider is 
c                      tangential to F2 trace

c      iflagpaul = 1    M(3000)F2 comes from end of trace
c      iflagpaul = 2    M(3000)F2 comes from start of trace
c      iflagpaul = 3    tangent freq / foF2 < tan_freq_min

c      icheck16            0 if no errors, else 1 (details given by ipaulfit)

      dimension ftof(*),otf(*),x(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      
      ICHECK = 16    !! and PROBLEM(2)                
      icheck16 = 0

c-----Calculate M(3000)F2 using Adolph Paul's method (ARTIST also uses this)
      icheck16 = 0
      kk = 0
      DO i = 1,idfi7
       IF (ftof(i).le.fof2_art) THEN
        hprime = otf(i)
        IF (hprime.gt.0.) THEN
        paulfac = (67.654 - 0.0149 * hprime ) / sqrt(hprime)
        kk = kk + 1
        x(kk) = ftof(i) * paulfac
c      print*," qs_check166",kk,ftof(i),x(kk)
        ENDIF    !! hprime.gt.0.
       ENDIF      !! ftof(i).le.fof2_art
      ENDDO
c-----Find the freq at which the maximum occurs
      paulmax = x(1)
      maxp = 1
      DO i = 2,kk
       if(x(i).gt.paulmax) maxp = i
       if(x(i).gt.paulmax) paulmax = x(i)
c      print*," i,qs_check166",i,x(i),maxp,paulmax
      ENDDO

c-----Save the value of the tangency frequency
      tangent_freq = ftof(maxp)
      
      iflagpaul = 0
      if(maxp.ge.kk) iflagpaul = 1    !!   Unreliable value
      if(maxp.eq. 1) iflagpaul = 2    !!   Unreliable value
      
c----- Check NTF if foF2 is high enough - mod 8-july-2007
      if(fof2_ART.ge.5.) THEN
      if(tangent_freq.lt.tan_freq_min*fof2_ART) iflagpaul = 3
c      print*,"qs_check166",tangent_freq,tan_freq_min,fof2_ART
      ENDIF
      

      IF(iflagpaul.ne.0) THEN
       icheck16 = 1
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a tangent_freq,fof2_ART,float(iflagpaul)
9002   format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)
      ENDIF    !! iflagpaul.ne.0

      return
      END

c**********************************************************************

c----- This s/r scans for flat sections in the F2 trace (probably Es)
c      Look for flat sections longer than fr_max_flat * length of F2 trace

c----- leo mcnamara 5-april-2005

      subroutine qs_check18
     a (cyear4,cdayno,chour,cmin,ftof,otf,idfi7,fr_max_flat,
     a iwantusefuls,iusefuls,iflagfile,flat_trace_rat,icheck18)

c      ftof                F2 frequencies
c      otf                    F2 virtual heights
c      idfi7    idfi(7)        # of F2 points
c      fof2_art            ARTIST scaled value of foF2
c      fr_max_flat    !!        fractional allowed length of F2 flat section PROBLEM 5

c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      flat_trace_rat        fraction of F2 trace that is flat
c      icheck18            0 if no errors, else 1 (details given by ipaulfit)

      dimension ftof(*),otf(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin

      ICHECK = 18        !! also PROBLEM = 5
      icheck18 = 0

      if(idfi7.gt.0) THEN !! need a scaled F2

      call flatrace (ftof,otf,idfi7,nflat,fstart,hflat)
      
      freq_int = ftof(2) - ftof(1)
      f2_length = ftof(idfi7) - ftof(1)
      flat_trace = nflat * freq_int
      flat_trace_rat = flat_trace / f2_length
      
      if(flat_trace_rat.gt.fr_max_flat) THEN

      icheck18 = 1
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a      float(nflat),fstart,hflat,flat_trace_rat,fr_max_flat
9002  format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)
     
      if(iwantusefuls.eq.1) THEN
      if(flat_trace_rat.gt.fr_max_flat) 
     a write(iusefuls,8025) nflat,fstart,hflat
      ENDIF
8025  format(/5x,'Too long a flat section in trace ',
     a      /5x,i3,' points, starting at',f5.1,5x,'Height=',f6.1)
       
      ENDIF    !! nflat.gt.max_flat

      ENDIF  !! idfi7.gt.0 arrive here if no scaled F2
      
      return
      
      END
      
c**********************************************************************

c      This s/r checks the scaled F1 h' trace for smoothness. 
c      If it is OK,we should be able to fit a LSF curve through the points
c      Work in terms of 1/h' to follow slope at foF1
c      Fit is weighted by the echo amplitudes.
c      Last two points heavily weighted

c----- leo mcnamara 1-april-2005
c----- modifed 22-august-2005 to derive FOM_F1fit

      subroutine qs_check32
     a (cyear4,cdayno,chour,cmin,
     a ftof1,otf1,ioaf1,idfi12,nchebF1,eps,eta,rms_fac,
     a diffitmax,percslope_min,x,y,z,diffit,cheb,iwantusefuls,iusefuls,
     a iflagfile,f1_smoothed,FOM_F1fit,ier_cheby,ipoorfitF1,icheck32)
     
c      ftof1                F1 frequencies
c      otf1                F1 virtual heights
c      ioaf1                F1 echo amplitudes (uesd as weights)
c      idfi12    idfi(12)    # of F1 points
c      nchebF1                maximum order of fitted Cheby
c      eps                    relative tolerance loss of significance in Cheby fit
c      eta                    relative tolerance square sum of errors in Cheby fit
c      rms_fac                [defined in config file]
c      diffitmax.            max allowable error in Cheby LSF
c      percslope_min        LSF is straight line if slopes of 2 halves of LSF differ by < percslope_min
c      x                    temp array
c      y                    temp array
c      z                    temp array
c      diffit                temp array - abs diff between original and fitted
c      cheb                chebycheb coeffs for the fit
c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      f1_smoothed            smoothed F1 trace points
c      FOM_F1                figure of merit for F1 fit
c      ier_cheby            error flag for cheby fit - should be 0
c      ipoorfitF1            0 for good fit
c                          1 if no fit possible (icluding case with just one point in fit)
c                          2 if fit made but values negative
c                          3 there are points too far off the fitted curve
c                          4 if the fit collapsed to a straight line
c                         5 if the fit is convex upwards
c      icheck32            0 if no errors, else 1 (details given by ipoorfitF1)

      dimension ftof1(*),otf1(*),x(*),y(*),z(*),f1_smoothed(*),cheb(*)
      dimension diffit(*)
      integer*2 ioaf1(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      
      ICHECK = 32      !! and PROBLEM(10)
      icheck32 = 0
      ipoorfitF1 = 0
      num_za = 0    !! number of freqs with zero amplitude
      num_nza = 0    !! number of freqs with non-zero amplitud
      FOM_F1fit = 0.    !! figure of merit for the F1 fit

c-----Set up arrays for LSF fit

      averamp = 0.    !! average of non-zero values
      kk = 0
      DO i = 1,idfi12
      if(ioaf1(i).ne.0) THEN
      kk = kk + 1
      averamp = averamp + ioaf1(i)
      ENDIF
      ENDDO
      if(kk.ne.0) averamp = averamp / float(kk) !! mod 8-april-2007

      kk = 0
      DO i = 1,idfi12
       kk = kk + 1
       x(kk) = ftof1(i) !!   frequency
       y(kk) = otf1(i)  !!   virtual height
       y(kk) = 1. / y(kk)
       z(kk) = ioaf1(i) !!   LSF weight = amplitude
       if(ioaf1(i).eq.0) num_za = num_za + 1
       if(ioaf1(i).ne.0) num_nza = num_nza + 1
       f1_smoothed(kk) = 0.    !! initialize in case no fit
      ENDDO

      FOM_F1fit = float(num_nza) / float(num_za + num_nza) * 100.

c-----Weight the last two points heavily, so as to follow scaled trace (100 works)
c-----Replace zero amplitudes by the average non-zero value
c      Otherwise, the traces with no amplitudes near the end will be
c      rejected because of a poor fit.
      if(z(kk-1).eq.0.) z(kk-1) = averamp
      if(z(kk  ).eq.0.) z(kk  ) = averamp
      z(kk-1) = z(kk-1) * 100
      z(kk) = z(kk) * 100

c-----Find the LSF curve through the points, provided kk > 3
      if(averamp.eq.0.) go to 121    !! no amplitudes for the F1 points    MOD 8-april-2007
      if(kk.lt.3) go to 121   !! this ionogram has a problem
      ncheb = kk - 2
      if(ncheb.gt.nchebF1) ncheb = nchebF1  !!  number of terms in Chebyshev fit
      epsi = eps * 0.1
      etai = eta * 0.1
c     We relax the required accuracy of the fit as required.
      DO ierror = 1,4 
       epsi = epsi * 10.
       etai = etai * 10.
       call cheby_lsf (x,y,z,kk,ncheb,epsi,etai,ier,xd,xo,ires,cheb)
       if(ier.eq.0) go to 99   !! Fit has been successful
      ENDDO
      if(iwantusefuls.eq.1) write(iusefuls,7040)
7040  format(/5x,'No LSF Chebyshev fit possible for F1 h" trace')
      ipoorfitF1 = 1
      go to 121   !! this ionogram has a problem

c-----Evaluate the fit and compare with original values
99    continue
      rmsdiff = 0.
      krms = 0
      DO i = 1,kk
       yinv = y(i)
       yinv = 1. / y(i)
       xxx = xd * x(i) + xo
       fit = cnps (cheb,ires,xxx)
       fit = 1. / fit
       if(fit.lt.0.) THEN
       ipoorfitF1 = 2    !! physically impossible -- ERROR
       if(iwantusefuls.eq.1) write(iusefuls,7000) i,x(i),y(i),fit
       go to 121
       ENDIF    !! fit.lt.0.
       f1_smoothed(i) = fit    !! smoothed F1 trace
       diffit(i) = abs(yinv - fit)
       rmsdiff = rmsdiff + diffit(i)**2
       krms = krms + 1
       if(iwantusefuls.eq.1) write(iusefuls,7000) i,x(i),yinv,fit,z(i)
7000   format(5x,'LSF Chebyshev to F1 h"',i5,4f10.3)
      ENDDO
      rmsdiff = sqrt(rmsdiff / float(krms))

      if(iwantusefuls.eq.1) write(iusefuls,7005) x(1),x(kk),rmsdiff
7005  format(/5x,'RMS km error in LSF Chebyshev F1 fit from',f5.1,
     a ' to',f6.2,'  is',f6.1)

c-----Look for points that lie more than rms_fac * rmsdiff from the
c      fitted smooth curve
      DO i = 1,kk 
      if(diffit(i).gt.rms_fac*rmsdiff.or.diffit(i).gt.diffitmax) THEN  
           ipoorfitF1 = 3    
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  x(i),1./y(i),rmsdiff,diffit(i)
9002  format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)
      ENDIF
      ENDDO

121   continue

c---- Check that SMOOTHED F1 trace is not a straight line
c      mid = idfi12/2
      mid1 = 1
      mid2 = idfi12

      if(mod(idfi12,2).eq.1) THEN    !! odd
      mid3 = (idfi12 + 1)/2
      mid4 = (idfi12 + 1)/2
      ENDIF
      if(mod(idfi12,2).eq.0) THEN    !! even
      mid3 = idfi12/2
      mid4 = (idfi12 + 2)/2
      ENDIF
c      print*,idfi12,mid1,mid3,mid4,mid2

      diff1 = ftof1(mid3) - ftof1(mid1)
      diff2 = ftof1(mid2) - ftof1(mid4) 
      slop1 = (f1_smoothed(mid3) - f1_smoothed(mid1))/diff1
      slop2 = (f1_smoothed(mid2) - f1_smoothed(mid4))/diff2
      slop1a = abs(slop1)
      slop2a = abs(slop2)
      diffslope = abs(slop1a-slop2a)
      averslope = (slop1a+slop2a) / 2.
      percslope = 9999.
      if(averslope.ne.0.)    percslope = diffslope/averslope*100.
       
      IsStraightLine = 0
c-----Slopes normally equal to 5 or 6 figs whem LSF degenerates to a straight line
c      We allow for almost equal (might protect against short linear segments)
      if(percslope.lt.percslope_min.or.averslope.eq.0.) IsStraightLine=1

c----- See if the slope changes sign
      if(slop2.ne.0.) THEN
      if(slop1/slop2.lt.0.) IsStraightLine = 0
      ENDIF
      if(slop2.eq.0) IsStraightLine = 1    !! horizontal line

      if(IsStraightLine.eq.1) THEN
      ipoorfitF1 = 4
      if(IwantUsefuls.eq.1) THEN
      write(iusefuls,6789) slop1,slop2,percslope,percslope_min,averslope
6789  format(5x,"** F1 fit degenerated to a straight line",5f10.4)
      ENDIF
      ENDIF

c----- Check that fitted curve is not convex upwards
      midpoint = kk/2
      hmid = f1_smoothed(midpoint)
      hmb = f1_smoothed(1)
      hma = f1_smoothed(2*midpoint)
      averab = (hmb + hma) / 2.
      if(averab.lt.hmid) ipoorfitF1 = 5

c-----
123   continue
      if(ipoorfitF1.ne.0) THEN
      ICHECK = 32    !! to identify the cause
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a float(ipoorfitF1)
      ENDIF !! ipoorfitF1.ne.0

      ier_cheby = ier
      if(ipoorfitF1.ne.0) icheck32 = 1    !! error
      if(ier_cheby.ne.0) icheck32 = 1        !! error
      
      return
      
      END

c**********************************************************************

c      This s/r checks for too long a gap between foE and fminF2,
c     between foE and fminF1, or between foF1 and fminF2. 
c      Large gaps cause POLAN to die quietly.
c      They also lead to suspect N(h) profiles

c----- leo mcnamara 5-april-2005

      subroutine qs_check33
     a (cyear4,cdayno,chour,cmin,
     a ftof,ftof1,ftoe,idfi7,idfi12,idfi17,gapmax,iflagfile,
     a gapmaxobs,icheck33)
     
c      ftof                F2 frequencies
c      ftof1                F1 frequencies
c      ftoe                E frequencies
c      idfi7    idfi(7)        # of F2 points
c      idfi12    idfi(12)    # of F1 points
c      idfi17    idfi(17)    # of E points
c      gapmax                allowable freq gap after foE or foF1 PROBLEM 12
c      iflagfile            unit number for flags.dat

c----- returned
c      gapmaxobs            largest gap between traces 
c      icheck33            0 if no errors, else 1

      dimension ftof(*),ftof1(*),ftoe(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin

      ICHECK = 33      !! also PROBLEM(12)
      icheck33 = 0      

      gapmaxobs = 0. 
c
      e_last = 0.
      f1_first = 0.
      f1_last = 0.
      f2_first = 0.

      if(idfi17.ne.0) e_last = ftoe(idfi17)
      if(idfi12.ne.0) f1_first = ftof1(1)
      if(idfi12.ne.0) f1_last = ftof1(idfi12)
      if(idfi7.ne.0) f2_first = ftof(1)

      diff_ef1 = 0.
      if(e_last.ne.0.and.f1_first.ne.0) diff_ef1 = f1_first - e_last
      if(diff_ef1.gt.gapmax) icheck33 = 1       !! set the flag
      
      diff_ef2 = 0.
      if(e_last.ne.0.and.f2_first.ne.0.and.f1_first.eq.0) 
     a diff_ef2 = f2_first - e_last
      if(diff_ef2.gt.gapmax) icheck33 = 1       !! set the flag
      
      diff_f1f2 = 0.
      if(f1_last.ne.0.and.f2_first.ne.0) diff_f1f2 = f2_first - f1_last
      if(diff_f1f2.gt.gapmax) icheck33 = 1       !! set the flag
      
c-----Save the biggest gap to use for defining as problem
      if(diff_ef1.gt.diff_ef2.and.diff_ef1.gt.diff_f1f2) 
     a   gapmaxobs = diff_ef1
      if(diff_ef2.gt.diff_ef1.and.diff_ef2.gt.diff_f1f2) 
     a   gapmaxobs = diff_ef2
      if(diff_f1f2.gt.diff_ef1.and.diff_f1f2.gt.diff_ef2) 
     a   gapmaxobs = diff_f1f2

      IF(gapmaxobs.gt.gapmax)  THEN
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,diff_ef1,
     a diff_ef2,diff_f1f2
9002  format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)
      icheck33 = 1

      ENDIF      !! gapmaxobs.gt.gapmax

      return
      END
      
c**********************************************************************

c-----This s/r check for a corner in the F2 trace
c     Corners cause POLAN to die quietly. A corner is where a flat trace 
c     suddenly jumps up almost vertically. Typically, the jump is from the 
c     2Es echo to the F2 trace. For each point, we find the slope for the 
c     last few points, and the slope for the next few points. Sudden jumps 
c     in the slope denote a corner.

c----- leo mcnamara 5-april-2005

      subroutine qs_check34
     a (cyear4,cdayno,chour,cmin,ftof,otf,idfi7,slopechange,
     a iflagfile,icheck34)

c      ftof                F2 frequencies
c      otf                    F2 virtual heights
c      idfi7    idfi(7)        # of F2 points
c      slopechange            change in slope that defines a corner PROBLEM 13
c      iflagfile            unit number for flags.dat

c----- returned
c      icheck34            0 if no errors (corners)

      dimension ftof(*),otf(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin

      ICHECK = 34      !! also PROBLEM(13)

      delta_f = ftof(2) - ftof(1)    !! scaled frequency spacing
      idfi7m = idfi7 - 2
      kk = 1
      DO i = 3,idfi7m  !! we find the average slope for three points
       diff1 = otf(i-1) - otf(i-2)
       diff2 = otf(i) - otf(i-1)
       diff = 0.5 * (diff2 - diff1)
       back = diff
       diff1 = otf(i+2) - otf(i+1)
       diff2 = otf(i+1) - otf(i)
       diff = 0.5 * (diff2 - diff1)
       forward = diff
       change = (forward - back)/delta_f 
       
       IF (change.gt.slopechange.and.back.gt.0.) THEN 
       icheck34 = 1
       back = back/delta_f 
       forward = forward/delta_f 
       write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,ftof(i),
     a back,forward
9002   format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,10f8.2)

       ENDIF      !! change.gt.slopechange.and.back.gt.0.
      ENDDO          !! loop on frequency

      return
      END

c**********************************************************************

c      This s/r counts the non-zero echo amplitudes for three sections of the
c      F2 trace. The trace is divided into three equal sections, giving three
c      counts of non-zero amplitudes, isum1,isum2,isum3

c      ICHECK39 is set to 1 if too many echoes have zero amplitude, indicating
c      that the scaled trace did not follow the echoes.

c----- leo mcnamara 24-august-2005

      subroutine qs_check39
     a (cyear4,cdayno,chour,cmin,ioaf,idfi9,otf,idfi7,
     a iwantusefuls,iusefuls,iflagfile,iamp_flag,icheck39)

c      otf                    F2 virtual heights
c      idfi7    idfi(7)        # of F2 points
c      ioaf                echo amplitude
c      idfi9                number of amplitudes
c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      iamp_flag        1, 2, 3 or 4

c      icheck39            0 if no problems, else 1 
c                          (details given by iamp_flag)

      integer*2  ioaf(*)
      dimension otf(*)
      
      character*4 cyear4
      character*3 cdayno
      character*2 chour,cmin
      
      fac1 = 1.0 / 3.0    !! used in comparisons(2/3 of aver)
      fac2 = 0.5            !! used in comparisons
      
      ICHECK = 39    !! and PROBLEM(17)                
      icheck39 = 0

c----- Ignore trace beyond any point with h'=9999 (espeially for Kwajalein)
      nidfi9 = 0
      DO i = 1,idfi7
      if(otf(i).lt.9000.) THEN
      nidfi9 = nidfi9 + 1
      ENDIF
      ENDDO

c----- Count the non-zero amplitudes for three sections of F2 trace
      n13 = nidfi9 / 3
      na1s = 1    !! s = start
      na1e = n13    !! e = end
      na2s = na1e
      na2e = n13 + n13
      na3s = na2e
      na3e = nidfi9
      
      sum1 = 0.
      sum2 = 0.
      sum3 = 0.
      fa1 = 0.
      fa2 = 0.
      fa3 = 0.
      
      DO i = na1s,na1e
      if(ioaf(i).ne.0) sum1 = sum1 + 1
      fa1 = fa1 + 1
      ENDDO
      DO i = na2s,na2e
      if(ioaf(i).ne.0) sum2 = sum2 + 1
      fa2 = fa2 + 1
      ENDDO
      DO i = na3s,na3e
      if(ioaf(i).ne.0) sum3 = sum3 + 1
      fa3 = fa3 + 1
      ENDDO
      
      if(fa1.ne.0.) isum1 = sum1 / fa1 * 100.
      if(fa2.ne.0.) isum2 = sum2 / fa2 * 100.
      if(fa3.ne.0.) isum3 = sum3 / fa3 * 100.

c      print*,idfi9,idfi7,nidfi9
c      print*,na1s,na1e,na2s,na2e,na3s,na3e
c      print*,fa1,fa2,fa3

c----- Set the various flags if necessary

      iamp_flag = 0
      
c----- One sum is much smaller than the other two 
      summax = isum1
      if(isum2.gt.summax) summax = isum2
      if(isum3.gt.summax) summax = isum3
      if(isum1.lt.fac2*summax) iamp_flag = 1    !! first 1/3 of trace
      if(isum2.lt.fac2*summax) iamp_flag = 2
      if(isum3.lt.fac2*summax) iamp_flag = 3
      
c----- Middle part of trace has too low a # of non-zero amplitudes      
      if(sum2.lt.(sum1+sum3)*fac1) iamp_flag = 4

c----- Mod 8-july-2007
c----- Set flag is the 3 sums do not exceed some min (usually 100+100+100)\
      if(nidfi9.lt.30) percreq = 67.
      if(nidfi9.ge.30) percreq = 83.
      if(isum1+isum2+isum3.le.percreq) iamp_flag = 5    !! too few echoes
      
c----- Set the Check
      if(iamp_flag.ge.1) icheck39 = 1

      if(icheck39.eq.1) THEN
      write(iflagfile,9002) icheck,cyear4,cdayno,chour,cmin,
     a  isum1,isum2,isum3,iamp_flag,summax
9002  format(6x,' ICHECK ',i3,a5,a4,a3,a2,2x,4i5,f8.0)
      ENDIF !! icheck39.eq.1

      if(iwantusefuls.eq.1) write(iusefuls,7000) isum1,isum2,isum3,
     a iamp_flag
c      write(6,7000) isum1,isum2,isum3,iamp_flag,icheck39
7000  format(5x,'F2 Amplitude FOMs',8i5)

      
      RETURN
      
      END

c**********************************************************************
      subroutine SAOPLOT
     a      (iartpro,ipolpro,sysdes,idfi,scaled,freq_e,freq_f1,freq_f2,
     a       the,thf1,thf2,ote,otf1,otf2,iyear,idayno,ihour,imin,
     a       fof2_pol,hmf2_pol,fn,virtht,realht,freq_int,em3_smoothed,    !! em3_smoothed added 23-oct-07
     a     kall_polan,numpolpro,polplot_fr,polplot_hp,polplot_hr)
c
c      Prog to reconstruct either the ARTIST or POLAN FN(h) profile
c      for plotting purposes.
c      The output is a list of fn, h & h' groups (+ descriptor)
c
c     Written by leo mcnamara 11-oct-93
c      Modified 1/19/06 to save smoothed trace for POLAN

c----- MOD 25-march-2009 Calculate profile half width (POLAN)

      Character*120 Sysdes
      integer*2 idfi
      dimension idfi(*),freq_e(*),freq_f1(*),freq_f2(*),
     a the(*),thf1(*),thf2(*),ote(*),otf1(*),otf2(*),scaled(*),
     a fn(*),realht(*),virtht(*)
      dimension polplot_fr(*),polplot_hp(*),polplot_hr(*)
      dimension hdash(666),coeffs(20)

c----- MOD 25-march-2009 Calculate profile half width
      dimension xx(500),yy(500)
      common/POLAN_halfwidth/polHalfWidth
c----- MOD 25-march-2009 Calculate profile half width

      data kount / 0 /
c
c>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      fof2_art = scaled(1)
      fof1_art = scaled(2)
      hmf2_art = thf2(3)
      em3_art  = scaled(3)
      foe_art  = scaled(9)

c      if(kount.eq.0) write(iartpro,200) sysdes
c      if(kount.eq.0) write(ipolpro,200) sysdes
200   format(a120)

      kount = kount + 1
c
      DO i = 1,666
       fn(i) = 0.
       realht(i) = 0.
       virtht(i) = 0.
       hdash(i) = 0.
      ENDDO
c
      kk = 0        !!        number of points in the profile

c-----The ARTIST E profile
      if(idfi(17).le.0) go to 197
      fmin = freq_e(1)
      f_start = the(1)
      f_end = the(2)

      if(f_start.eq.f_end) go to 197
      hme = the(3)
      DO i = 1,5
      coeffs(i) = the(i+4)
      ENDDO
      do 196 i = 1,9999
      plasmafreq = freq_e(1) + (i - 1) * freq_int
      if(plasmafreq.gt.f_end) go to 197
      kk = kk + 1

      gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
      realht(kk) = z_cheb (coeffs,gg,hme)
      fn(kk) = plasmafreq
      ifreq = (plasmafreq - fmin) / freq_int + 1.5
      ifpol = plasmafreq / freq_int + 1.5
      if(ifreq.gt.idfi(17)) go to 196
      if(plasmafreq.lt.fmin) go to 196
      virtht(kk) = ote(ifreq)
c      print*," ART PROF E",kk,ifreq,fn(kk),virtht(kk)
      hdash(ifpol) = virtht(kk)    !!    needed for POLAN output
196   continue
197   continue
      
c-----The ARTIST F1 profile
      if(idfi(12).le.0) go to 297
      f_start = thf1(1)
      f_end = thf1(2)
      fmin = freq_f1(1)
      if(f_start.eq.f_end) go to 297
      hmf1 = thf1(3)
      DO i = 1,5
       coeffs(i) = thf1(i+4)
      ENDDO
      do 296 i = 1,9999
      plasmafreq = freq_f1(1) + (i - 1) * freq_int
      if(plasmafreq.gt.f_end) go to 297
      kk = kk + 1

      gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
      realht(kk) = z_cheb (coeffs,gg,hmf1)
      fn(kk) = plasmafreq
      ifreq = (plasmafreq - fmin) / freq_int + 1.5
      ifpol = plasmafreq / freq_int + 1.5
      if(ifreq.gt.idfi(12)) go to 296
      if(plasmafreq.lt.fmin) go to 296
      virtht(kk) = otf1(ifreq)
c      print*," ART PROF F1",kk,ifreq,fn(kk),virtht(kk)
      hdash(ifpol) = virtht(kk)    !!    needed for POLAN output
296   continue
297   continue

c-----The ARTIST F2 profile
      if(idfi(7).le.0) go to 397
      f_start = thf2(1)
      f_end = thf2(2)
      fmin = freq_f2(1)
      if(f_start.eq.f_end) go to 397
      hmf2 = thf2(3)
      DO i = 1,5
       coeffs(i) = thf2(i+4)
      ENDDO
      do 396 i = 1,9999
      plasmafreq = freq_f2(1) + (i - 1) * freq_int
      if(plasmafreq.gt.f_end) go to 397
      kk = kk + 1
      gg = alog(plasmafreq/f_end) / alog(f_start/f_end)
      realht(kk) = z_cheb (coeffs,gg,hmf2)
      fn(kk) = plasmafreq
      ifreq = (plasmafreq - fmin) / freq_int + 1.5
      ifpol = plasmafreq / freq_int + 1.5
      if(ifreq.gt.idfi(7)) go to 396
      if(plasmafreq.lt.fmin) go to 396
      virtht(kk) = otf2(ifreq)
c      print*," ART PROF F2",kk,ifreq,fn(kk),virtht(kk)
      hdash(ifpol) = virtht(kk)    !!    needed for POLAN output
396   continue

397   continue
      
      if(kk.eq.1) return  !! no profile

c-----Add in a point for the peak of the F2 layer
      fof2 = scaled(1)
      kk = kk + 1
      fn(kk) = fof2
      realht(kk) = hmf2
      if(kk.ne.1) virtht(kk) = virtht(kk - 1) + 1.
      hdash_last = virtht(kk)    !!    needed for POLAN output
201   format(/1x,i4,i3,i2,5x,i5,' points .. ARTIST')
c
      fn1_art = fn(1)    !! used to match the two files
      DO i = 1,kk
      fnorm = fn(i)/fof2_art
      hnorm = realht(i)/hmf2_art
      elden = (fn(i)/0.00898)**2    !! per cc
      if(i.eq.1) THEN
      write(iartpro,202) fn(i),realht(i),virtht(i),fnorm,hnorm,elden,
     a fmin,iyear,idayno,ihour,imin,foE_ART,foF1_ART,foF2_ART,hmF2_ART,
     a em3_ART
      ENDIF
      if(i.gt.1) THEN
      write(iartpro,202) fn(i),realht(i),virtht(i),fnorm,hnorm,elden,
     a fmin,iyear,idayno,ihour,imin
      ENDIF
c      write(6,202) fn(i),realht(i),virtht(i)
c      PAUSE
      ENDDO
      write(iartpro,202)    !! blank line to separate concatenated plots
202   format(f10.3,2f10.1,2f10.3,1pe15.3,0pf10.3,i5,i3,2i2,4f8.2,f7.3,
     a f7.2)      !! mod 25-march-2009

c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

c-----The POLAN profile 

c      Bypass for stupid values of foF2
      if(fof2_pol.gt.20.) return

c      Check for no profile
      if(numpolpro.eq.0) return

c      Just in case
      if(kall_polan.eq.0) return

c----- MOD 25-march-2009 Calculate profile half width
      fnhalf = 0.707 * foF2_POL    !! 0.5 NmF2
      numhalf = 0
      DO i = 1,numpolpro
      if(polplot_fr(i).gt.foE_ART + 0.2) THEN
      numhalf = numhalf + 1
      xx(numhalf) = polplot_fr(i)
      yy(numhalf) = polplot_hr(i)
      ENDIF
      ENDDO
      call ilios(yy,xx,fnhalf,numhalf,4,ans)
      polHalfWidth = hmF2_POL - ans
c      print*,"polHalfWidth,hmF2_POL ",polHalfWidth,hmF2_POL
c----- MOD 25-march-2009 Calculate profile half width

      DO i = 1,numpolpro
      fnorm = polplot_fr(i) / fof2_pol
      hnorm = polplot_hr(i) / hmf2_pol
      hdashi = abs(polplot_hp(i))    !! in case set negative for cusp
      elden = (polplot_fr(i)/0.00898)**2    !! per cc
      if(i.eq.1) THEN
      write(ipolpro,202) polplot_fr(i),polplot_hr(i),hdashi,fnorm,
     a hnorm,elden,fmin,iyear,idayno,ihour,imin,foE_ART,foF1_ART,
     a foF2_POL,hmF2_POL,em3_smoothed,polHalfWidth
      ENDIF
      if(i.gt.1) THEN
      write(ipolpro,202) polplot_fr(i),polplot_hr(i),hdashi,fnorm,
     a hnorm,elden,fmin,iyear,idayno,ihour,imin
      ENDIF
      ENDDO

      write(ipolpro,202)    !! blank line to separate concatenated plots

      return
      end
      
c**********************************************************************

      subroutine setupPOLANarrays
     a(idfi,freq_e,freq_f1,freq_f2,ote,otf1,otf2,
     ahprime_E,hprime_F1,f2_critfreq,
     andim,nval,freq,h_prime)

c      S/R to set up the freq and h' arrays for POLAN - O ray only.
c      written by leo mcnamara 5-august-2004 (From polsetup)
c      JET'S documentation is included at the end of this file

c---Terminating intermediate layers
c      See JETs doc "intermediate layers are terminated"
c      hprime_E    h'E to use at foE
c      hprime_F1    h'F to use at foF1

c-----f2_critfreq F2 critical frequency
c      zero        POLAN extrapolates its own value
c      fof2_ART    POLAN includes this value in its LSF peak fit

c-----Returned arrays
c      h'(f) array is h_prime(freq), nval values

c      ==================================================================

      integer*2 idfi
      dimension h_prime(*),freq(*),freq_e(*),freq_f1(*),freq_f2(*),
     a        idfi(*),ote(*),otf1(*),otf2(*)

c      ==================================================================

c-----Initialize the arrays
      DO i = 1,ndim
       h_prime(i) = 0.
          freq(i) = 0.
      ENDDO
      kkk = 0
      nval = 0

c      ==================================================================

c-----Insert E-layer points, if any
      if(idfi(17).gt.0) THEN  !! Bypass if no E-layer points

      if(idfi(17).ne.idfi(21)) print*, ' E layer data inconsistent'

      DO i = 1,idfi(17)
       kkk = i
       h_prime(i) = ote(i)
       freq(i) = freq_e(i)
      ENDDO

      idfi17 = idfi(17)
      delfreq = freq_E(idfi17) - freq_E(idfi17 - 1)

      if(h_prime(1).gt.h_prime(2)) h_prime(1) = h_prime(2) !! for ramey 1999/238/194

c-----foE cusp --- assume cusp occurs a bit above the scaled foE
      kkk = kkk + 1
      freq(kkk) = freq(kkk-1) + 0.5 * delfreq
      h_prime(kkk) = hprime_E

      ENDIF !! idfi(17).gt.0

c      ==================================================================
c-----Save the F1 layer data, if any
      if(idfi(12).gt.1) THEN  !! Bypass if no F1-layer points - require 2

      if(idfi(12).ne.idfi(16)) print*, ' F1 layer data inconsistent'
      idfi12 = idfi(12)
      delfreq = freq_F1(idfi12) - freq_F1(idfi12 - 1)

c-----Replace any h'=0 at the start of the trace
      if(otf1(1).eq.0.and.idfi(12).ge.3)
     a      otf1(1) = otf1(2) + (otf1(2) - otf1(3))

      DO i = 1,idfi(12)
       kkk = kkk + 1
       h_prime(kkk) = otf1(i)
       freq(kkk) = freq_f1(i)
      ENDDO

c-----foF1 cusp - we use a simple cusp - no valley, no peak
      if(hprime_F1.le.0.) h_prime(kkk) = -otf1(idfi(12))    !! OVERWRITES smoothed trace
c----- h_prime(kkk) = 10. crashes POLAN 03-april-2009

c-----If F1 trace is very short, this can cause trouble
       if(idfi(12).le.3) h_prime(kkk) = otf1(idfi(12))   

      ENDIF !! idfi(12).gt.0

c      ==================================================================
c      Save the F2 layer data

      no_E_or_F1 = 0
      if(kkk.eq.0) no_E_or_F1 = 1    !! there is no E or F1 data

      if(kkk.eq.0) kkk = 1    !! might have no E or F1 data

      idfi7 = idfi(7)
      delfreq = freq_F2(idfi7) - freq_F2(idfi7 - 1)

c-----Replace any h'=0 at the start of the trace
      if(otf2(1).eq.0.and.idfi(7).ge.3)
     a      otf2(1) = otf2(2) + (otf2(2) - otf2(3))

c-----Need to make sure first F2 freq is > foF1  (tests only first F2 point)
      lowfreq = 1   
      if(freq(kkk).ge.freq_f2(lowfreq)) lowfreq = 2

      if(no_E_or_F1.eq.1) kkk = 0
      DO i = lowfreq,idfi(7)
       kkk = kkk + 1
       if(i.eq.lowfreq) ifirstF2 = kkk        !! counter for first F2 point
       h_prime(kkk) = otf2(i)      !! we are ignoring 9999 heights
       freq(kkk) = freq_f2(i)
      ENDDO

c-----If last freq is not fof2_ART (trace was chopped), add an extra point
      if(freq(kkk).lt.f2_critfreq) THEN
      kkk = kkk + 1
      freq(kkk) = f2_critfreq
      h_prime(kkk) = 0.
      ENDIF
        do ii = 1,2      !! Add in two zero points & return 
        kkk = kkk + 1
       h_prime(kkk) = 0.
       freq(kkk) = 0.
       ENDDO
       nval = kkk

      return
      end
      
c**********************************************************************

      subroutine set_jumpfail(idfi7,njump,htjump,virtht,jumpfail)

c----- s/r to set the jumpfail flag
c----- leo mcnamara 4-august-2004. Code cut from QualCon
c----- jumpfail set to 1, 2, 3 or 4 -- 5-april-2005

c      idfi7    # of O-trace F2 points
c      NJUMP    the number of jumps detected by s/r jumpinht    
c      HTJUMP    the absolute height step h'(i+1) - h'(i)
c      VIRTHT    virtual heights for part of F2 trace

c      jumpfail    set to > 1 if a jump condition is satisfied

      dimension htjump(*),virtht(*)

c-----Flag the ionogram if 
c      (1)        2 or more jumps > 20 km
c      (2)        any 3 or more jumps
c      (3)        jump changes sign
c      (4)        one jump > 100 km

       jumpfail = 0
       kj = 0     !! number of jumps > 20 km
       DO i = 1,njump
        if (abs(htjump(i)).gt.20.) kj = kj + 1
       ENDDO

       kjj = 0
       IF (njump.ge.3) THEN   !! need more than 2 jumps
        DO i = 2,njump
        ratio = (htjump(i)/htjump(i-1))
        if(ratio.lt.0.) kjj = 1 !! sign change
        ENDDO
       ENDIF       !! njump.ge.3

c-----Set the jumpfail flag for 3 cases
       if(kj.ge.2) jumpfail = 1     !! number of jumps > 20 km
       if(njump.ge.3) jumpfail = 2  !! more than 3 jumps
       if(kjj.eq.1) jumpfail = 3    !! sign change
c
c-----Check for jumps > 100 km (full F2 trace)
       DO i = 2,idfi7
        hdiff = (virtht(i) - virtht(i-1))
        IF (hdiff.gt.100.) THEN
         jumpfail = 4
        ENDIF    !! hdiff.gt.100.
        
       ENDDO

      return
      end
      
c**********************************************************************
      subroutine slope_at_end
     a (nval,freq,virt,ioaf,nslope,slope_end)
c     s/r to calculate the average slope over the last NSLOPE points (of F2 trace)
c     written by leo mcnamara 24-oct-01
c----- modified 19-august-2005 to ignore ends of the trace that
c      have no supporting amplitudes
c----- modified 10-july-2007 - second variable removed from c/s (not used)

      integer*2 ioaf(*)    !! amplitudes
      dimension freq(*),virt(*),x(20),y(20)

c----- Apply this test to echoes with non-zero amplitudes
c----- Look for the highest freq with a non-zero amplitude (kamp)

      DO i = 1,nval
      ii = nval - i + 1
      kamp = ii
      if(ioaf(ii).gt.0) go to 10
      ENDDO
10      continue

      kk = nslope + 1
      DO i = 1,kamp
       j = kamp - i + 1
       kk = kk - 1
       IF (kk.gt.0) THEN
        x(kk) = freq(j)
        y(kk) = virt(j)
       ENDIF
      ENDDO

      call correlation 
     a  (x,y,nslope,slope,fint,xbar,ybar,cor_coeff)
      slope_end = slope
c
      return
      end

c**********************************************************************

      subroutine smoothed_m3000
     a (ftof,f2_smoothed,idfi7,fof2_ART,em3_art,iwantusefuls,iusefuls,
     a iflagfile,em3_smoothed)
      
c----- s/r to determine M(3000)F2 for the smoothed F2 trace

c----- leo mcnamara 13-april-2005

c      ftof                F2 frequencies
c      f2_smoothed            smoothed F2 virtual heights
c      idfi7    idfi(7)        # of F2 points
c      fof2_art            ARTIST scaled value of foF2
c      em3_art                ARTIST value of M(3000)F2
c      iwantusefuls        non-zero to save stuff to usefuls.dat
c      iusefuls            unit number for usefuls.dat
c      iflagfile            unit number for flags.dat

c----- returned
c      em3_smoothed        M(3000)F2 for smoothed F2 trace
c                          zero if value comes from end of trace

      dimension ftof(*),f2_smoothed(*)
      dimension paul(500),temp(500)

      em3_smoothed = 0.

c-----Calculate M(3000)F2 using Adolph Paul's method
      kk = 0
      DO i = 1,idfi7
        hprime = f2_smoothed(i)
        IF (hprime.gt.0.) THEN
        paulfac = (67.654 - 0.0149 * hprime ) / sqrt(hprime)
        kk = kk + 1
        paul(kk) = ftof(i) * paulfac
        temp(kk) = ftof(i)
        ENDIF    !! hprime.gt.0.
      ENDDO

c-----Find the freq at which the maximum occurs
      paulmax = paul(1)
      maxp = 1
      DO i = 2,kk
       if(paul(i).gt.paulmax) maxp = i
       if(paul(i).gt.paulmax) paulmax = paul(i)
      ENDDO
      
      iflagpaul = 0
      if(maxp.eq.kk) iflagpaul = 1      !! value came from end of trace

      if(iflagpaul.eq.0) THEN

       em3_smoothed = paulmax / fof2_ART

      if(iwantusefuls.eq.1) THEN
       write(iusefuls,7050) em3_smoothed,em3_art
7050   format(/5x,'Smoothed & ARTIST values of M(3000)F2',3f6.2)
      ENDIF

          ENDIF    !! iflagpaul.eq.0
          
          return
          
          END

c**********************************************************************

      subroutine solar 
     a  (lly,lmon,lday,lh,lm, flat,Elon,reflon,
     A   DECLINATION, ZENITH, SUNRISE, SUNSET)
c
c      s/r to calculate the solar declination, zenith angle, and
c      sunrise & sunset times  - based on Newbern Smith's algorithm
c
c      written by leo mcnamara 1-sep-86
c
c-----      last modified 16-jun-87
c
c.......................................
c
c      lly    local time year [last 2 digits only] --- 4 DIGITS 28-june-2004
c      lmon    local time month
c      lday    local day
c      lh    local hour [at REFLON]
c      lm    local minute [at REFLON]
c      flat    northern latitude in degrees
c      elon    east longitude in degrees
c      reflon    east longitude to which the hour & minute are referred
c
c      declination declination of the sun in degrees
c      zenith        zenith angle of the sun in degrees
c      sunrise        local time of sunrise in hours 
c      sunset        local time of sunset in hours 
c
c.......................................
c
      dimension ndo(12),ndl(12)
c      day numbers for ordinary & leap years
      data ndo,ndl / 1,32,60,91,121,152,182,213,244,274,305,335,
     a                 1,32,61,92,122,153,183,214,245,275,306,336 /
c      amplitudes of Fourier coefficients  --  1955 epoch
      data  p1,p2,p3,p4,p6 /
     a 0.017203534,0.034407068,0.051610602,0.068814136,0.103221204 /
c
      dtr = 0.017453292
      flm = lm
      ly = lly - 1960    !! for Fourier coeffs
      if(mod(lly,4).eq.0) ld = ndl(lmon) + lday - 1
      if(mod(lly,4).ne.0) ld = ndo(lmon) + lday - 1
c
c      s/r is formulated in terms of WEST longitude 
      Eelon = elon
      if(elon.le.0.) Eelon = 360. + elon
      wlon = 360. - Eelon
      Ereflon = reflon
      if(reflon.le.0.) Ereflon = 360. + reflon
      Wreflon = 360. - Ereflon
c
c      time of equinox for that year
      if (ly) 7, 7, 9
7      il = int((3.1 - ly)/4.)
      go to 11
9      il = - int((ly + 0.1)/4.)
11      dt = -0.7819 + 0.24225*ly + il
      td = ld + (lh + flm/60. + Wreflon/15.) / 24.
      te = td - dt
c
c      declination of the sun
      dcl = 23.256 * sin(p1*(te-82.242)) + 0.381 * sin(p2*(te-44.855))
     a      + 0.167 * sin(p3*(te-23.355)) - 0.013 * sin(p4*(te+11.97))
     a      + 0.011 * sin(p6*(te-10.41)) + 0.339137
      declination = dcl
      decrad = dcl * dtr
c
c      the equation of time
      tf = te - 0.5
      eqt = -7.38*sin(p1*(tf-4.)) - 9.87*sin(p2*(tf+9.))
     a      + 0.27*sin(p3*(tf-53.)) - 0.2*cos(p4*(tf-17.))
      et = eqt * dtr / 4.
c
      fa = flat * dtr
      fb = wlon * dtr
      fc = Wreflon * dtr
      pho = fc - fb + et
      t = lh + flm/60.
      phi = 0.26179939 * ( t - 12.) + pho
c
      dc = decrad
      a = sin(fa) * sin(dc)
      b = cos(fa) * cos(dc)
      cosx = a + b * cos(phi)
      zenith = acos(cosx) / dtr
c
c      calculate sunrise and sunset times --  at the ground
c      see Explanatory Supplement to the Ephemeris (1961) pg 401
c      sunrise at height h metres is at
c      chi(h) = 90.83 + 0.0347 * sqrt(h)
c      this includes corrections for horizontal refraction and
c      semi-diameter of the solar disk
      ch = cos(90.83 * dtr)
      cosphi = (ch -a ) / b
c      if abs(secphi) > 1., sun does not rise/set
c      allow for sun never setting - high latitude summer
      secphi = 999999.
      if(cosphi.ne.0.) secphi = 1./cosphi
      sunset = 99.
      sunrise = 99.
      if(secphi.gt.-1.0.and.secphi.le.0.) return
c      allow for sun never rising - high latitude winter
      sunset = -99.
      sunrise = -99.
      if(secphi.gt.0.0.and.secphi.lt.1.) return
c
      phi = acos(cosphi)
      et = et / 0.26179939
cc      type*,' Equation of time in minutes = ',et*60.0
      srlt = 12. - phi / 0.26179939
      term = (wreflon - wlon)/15. + et
      sunrise = srlt - term
      sunset = 24. - srlt - term
      if(sunrise.lt.0.) sunrise = sunrise + 24.
      if(sunrise.ge.24.) sunrise = sunrise - 24.
      if(sunset.lt.0.) sunset = sunset + 24.
      if(sunset.ge.24.) sunset = sunset - 24.
c
      return
      end



c**********************************************************************

      subroutine starth
     a (flat,hourlt,solardec,chi,r12,first_vht,start)

c     s/r to calculate a synoptic value of the real height at 0.5MHz

c     see UAG-93 Sections 6.3 and 6.4. 
c      Titheridge Radio Science 23(5) 831,1988.
c      Titheridge  JATP 48(5) 435-446, 1986

c     written by leo mcnamara 5-feb-86
c     modified 5-sep-93 to bring in the time & zenith angle
c      modified 2-dec-2003 to bring in solar declination & first_vht

c     flat          latitude in degrees
c     hourlt          local time in hours
c      solardec    solar declination (degrees)
c     chi              solar zenith angle in degrees
c     r12              12-month smoothed sunspot numb
c      first_vht    first virtual height

c     start      calculated starting height

      pi = 3.141592654
      degrad = pi / 180.
      
      chi_rad = chi * degrad
      coski = cos(chi_rad)

      start = 90.    !! default, just in case
c
c-----choose between daytime and nighttime models
c     decision is made on the basis of the zenith angle & first h'
c      JET uses h'(1)>165 or xhi > 1.6 rad     (91.67 deg)

c======================================================================
      
      if(chi_rad.lt.1.6.and.first_vht.lt.160.) THEN      !! daytime E layer

c-----Use daytime model to get point in E layer - eqn 12 of Section 6 JATP
      declin = solardec*degrad
      dd = 3.4 * declin

      if(flat.lt.0.) dd = -dd    !! change sign for southern hemisphere
      start = 90.1 - (5.6 * coski + dd + 0.027 * (r12 - 50))
      return
      ENDIF

c======================================================================

c-----Nighttime
      if(chi_rad.ge.1.6.or.first_vht.ge.160.) THEN    !! nighttime

c-----Nighttime models - choose between left & right sides of Fig 4, 
c      See if local time is < or > 12
c     The fits to the curves are given in the JATP paper

c-----Fit for LHS of diagram "hours after sunset"
      a = 1.81
      b = 0.38
      u = 113.
      v = 49.
      if(hourlt.ge.12.) go to 150

c-----use "hours before sunrise" curve
      a = 1.71
      b = 0.14
      u = 117.
      v = 45.

150   chima = chi_rad - a      
      start = u + v * sin (1.9 * chima / ( b + abs(chima) ) )
c
      return
      ENDIF


      end

c**********************************************************************

      subroutine valley_WandD
     a (idayno,lhour,lmin,zenangle,zen_noon,fluxval,statlat,width,depth)

c----- s/r to evaluate the E-F valley width and depth using the model
c      given by Titheridge, JATP 65(129-137), 2003.

c----- leo mcnamara 27-july-2004

c      idayno        day of year
c      lhour        local time hour
c      lmin        local time minute
c      zenangle    solar zenith angle degrees
c      zen_noon    solar zenith angle at noon degrees
c      fluxval        10.7 cm solar flux
c      statlat        station latitude

c      width        valley width
c      depth        valley depth (percent)

c---- See the equations in Section 6 "A numerical model"

      pi = 4.* atan(1.0)
      degrad = pi / 180.

      dy = idayno
      flocaltime = float(lhour) + float(lmin)/60.
      tn = flocaltime - 12.
      tn = abs(tn)

      coschi = cos(zenangle * degrad)
      if(zenangle.gt.78.5) coschi = 0.2

      coschi_noon = cos(zen_noon * degrad)

      chi_term4 = coschi**0.6 / coschi_noon**0.3    !! eqn 4
      chi_term6 = (coschi / coschi_noon)**0.6        !! eqn 6

      s = sin(0.01721 * (dy - 80.5))    !! eqn 8
      if(statlat.lt.0.) s = -s    !! southern hemisphere
      delf = 120. / fluxval - 1.    !! eqn 16

      w_chi = 10.6 + 0.1 * (2. - s - s*s) * tn + 0.7 * (1. - s)**2    !! eqn 20

      w = w_chi / chi_term4    !! eqn 4


c----- Modified 20-May-2009 [It's no use being stupid, unless you show it!]

ccc      if(w.lt.28.) w = 28.
      if(w.gt.28.) w = 28.

c----- Modified 20-May-2009

      w = w + 3. * delf**2    !! eqn 21

      d_chi = 5.0 + (0.35 - 0.25 * s) * tn + s * (s - 1.)    !! eqn 22

      d = d_chi / chi_term6    !! eqn 6

      d = d_chi + 12. * delf**3    !! eqn 23

      width = w
      depth = d

c      write(6,66) zenangle,zen_noon,tn,chi_term4,chi_term6,w_chi,d_chi,w,d
66    format(10f8.2)

      return
      end

c**********************************************************************
